Package: MSA2dist
Type: Package
Title: MSA2dist calculates pairwise distances between all sequences of
        a DNAStringSet or a AAStringSet using a custom score matrix and
        conducts codon based analysis
Version: 1.15.0
Authors@R: 
    person(given = "Kristian K",
           family = "Ullrich",
           role = c("aut", "cre"),
           email = "ullrich@evolbio.mpg.de",
           comment = c(ORCID = "0000-0003-4308-9626"))
Description: MSA2dist calculates pairwise distances between all sequences of
    a DNAStringSet or a AAStringSet using a custom score matrix and conducts
    codon based analysis. It uses scoring matrices to be used in these pairwise
    distance calculations which can be adapted to any scoring for DNA or AA
    characters. E.g. by using literal distances MSA2dist calculates pairwise
    IUPAC distances. DNAStringSet alignments can be analysed as codon alignments
    to look for synonymous and nonsynonymous substitutions (dN/dS) in a
    parallelised fashion using a variety of substitution models. Non-aligned
    coding sequences can be directly used to construct pairwise codon
    alignments (global/local) and calculate dN/dS without any external
    dependencies.
License: GPL-3 + file LICENSE
Encoding: UTF-8
LazyData: false
biocViews: Alignment, Sequencing, Genetics, GO
Depends: R (>= 4.4.0)
Imports: Rcpp, Biostrings, GenomicRanges, IRanges, ape, doParallel,
        dplyr, foreach, methods, parallel, pwalign, rlang, seqinr,
        stats, stringi, stringr, tibble, tidyr, utils
Suggests: rmarkdown, knitr, devtools, testthat, ggplot2, BiocStyle
LinkingTo: Rcpp, RcppThread
VignetteBuilder: knitr
NeedsCompilation: yes
SystemRequirements: C++11
URL: https://gitlab.gwdg.de/mpievolbio-it/MSA2dist,
        https://mpievolbio-it.pages.gwdg.de/MSA2dist/
BugReports: https://gitlab.gwdg.de/mpievolbio-it/MSA2dist/issues
RoxygenNote: 7.3.1
git_url: https://git.bioconductor.org/packages/MSA2dist
git_branch: devel
git_last_commit: 662eead
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
Packaged: 2025-10-31 23:01:04 UTC; biocbuild
Author: Kristian K Ullrich [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4308-9626>)
Maintainer: Kristian K Ullrich <ullrich@evolbio.mpg.de>

# Generated by roxygen2: do not edit by hand

export(GENETIC_CODE_TCAG)
export(aa2selfscore)
export(aabin2aastring)
export(aastring2aabin)
export(aastring2aln)
export(aastring2dist)
export(addmask2string)
export(addpop2string)
export(addpos2string)
export(addregion2string)
export(aln2aastring)
export(aln2dnastring)
export(cds2aa)
export(cds2codonaln)
export(cdsstring2codonaln)
export(codon2numberAMBIG)
export(codon2numberTCAG)
export(codonmat2pnps)
export(codonmat2xy)
export(compareCodons)
export(dnabin2dnastring)
export(dnastring2aln)
export(dnastring2codonmat)
export(dnastring2dist)
export(dnastring2dnabin)
export(dnastring2kaks)
export(getmask)
export(getpos)
export(globalDeletion)
export(globalDeletionAA)
export(granthamMatrix)
export(indices2kaks)
export(iupacMatrix)
export(makePostalignedSeqs)
export(pal2nal)
export(popinteger)
export(popnames)
export(rcpp_KaKs)
export(rcpp_distSTRING)
export(rcpp_pairwiseDeletionAA)
export(rcpp_pairwiseDeletionDNA)
export(region)
export(regionused)
export(string2region)
export(subString)
export(uptriidx)
import(Rcpp)
importFrom(Biostrings,AAMultipleAlignment)
importFrom(Biostrings,AAString)
importFrom(Biostrings,AAStringSet)
importFrom(Biostrings,BString)
importFrom(Biostrings,BStringSet)
importFrom(Biostrings,DNAString)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,GENETIC_CODE)
importFrom(Biostrings,GENETIC_CODE_TABLE)
importFrom(Biostrings,RNAString)
importFrom(Biostrings,RNAStringSet)
importFrom(Biostrings,consensusMatrix)
importFrom(Biostrings,getGeneticCode)
importFrom(Biostrings,readAAStringSet)
importFrom(Biostrings,readDNAStringSet)
importFrom(Biostrings,subseq)
importFrom(Biostrings,translate)
importFrom(Biostrings,width)
importFrom(Biostrings,writeXStringSet)
importFrom(GenomicRanges,GRanges)
importFrom(IRanges,IRanges)
importFrom(IRanges,IRangesList)
importFrom(IRanges,disjoin)
importFrom(IRanges,end)
importFrom(IRanges,findOverlaps)
importFrom(IRanges,overlapsRanges)
importFrom(IRanges,reduce)
importFrom(IRanges,start)
importFrom(ape,as.AAbin)
importFrom(ape,as.DNAbin.alignment)
importFrom(ape,dist.dna)
importFrom(doParallel,registerDoParallel)
importFrom(dplyr,count)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(foreach,"%do%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(methods,is)
importFrom(methods,slot)
importFrom(parallel,makeForkCluster)
importFrom(parallel,stopCluster)
importFrom(pwalign,pairwiseAlignment)
importFrom(rlang,.data)
importFrom(seqinr,as.alignment)
importFrom(seqinr,kaks)
importFrom(stats,setNames)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(stringr,word)
importFrom(tibble,add_column)
importFrom(tibble,column_to_rownames)
importFrom(tidyr,"%>%")
importFrom(tidyr,as_tibble)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,unite)
importFrom(utils,data)
useDynLib(MSA2dist, .registration = TRUE)

#' @title codonmat2pnps
#' @name codonmat2pnps
#' @description This function calculates pn/ps according to \emph{Nei and
#' Gojobori (1986)}.
#' @param codonmat \code{codon matrix} of two columns to be
#' compared [mandatory]
#' @return An object of class \code{pnps} which is a list with the following
#' components:\cr
#' \code{seq1} sequence1 name\cr
#' \code{seq2} sequence2 name\cr
#' \code{Codons} sequence2 name\cr
#' \code{Compared} sequence2 name\cr
#' \code{Ambigiuous} sequence2 name\cr
#' \code{Indels} sequence2 name\cr
#' \code{Ns} sequence2 name\cr
#' \code{Sd} sequence2 name\cr
#' \code{Sn} sequence2 name\cr
#' \code{S} sequence2 name\cr
#' \code{N} sequence2 name\cr
#' \code{ps} sequence2 name\cr
#' \code{pn} sequence2 name\cr
#' \code{pnps} sequence2 name\cr
#' \code{ds} sequence2 name\cr
#' \code{dn} sequence2 name\cr
#' \code{dnds} sequence2 name\cr
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom seqinr kaks
#' @importFrom stats setNames
#' @seealso \code{\link[seqinr]{kaks}}
#' @references Nei and Gojobori. (1986) Simple methods for estimating the
#' numbers of synonymous and nonsynonymous nucleotide substitutions.
#' \emph{Mol. Biol. Evol.}, \bold{3(5)}, 418-426.
#' @references Ganeshan et al. (1997) Human immunodeficiency virus type 1
#' genetic evolution in children with different rates of development of disease.
#' \emph{J. Virology.} \bold{71(1)}, 663-677.
#' @references Yang et al. (2000) Codon-substitution models for heterogeneous
#' selection pressure at amino acid sites. \emph{Genetics.}
#' \bold{155(1)}, 431-449.
#' @examples
#' ## load example sequence data
#' data("hiv", package="MSA2dist")
#' #codonmat2pnps(dnastring2codonmat(hiv)[,c(1, 2)])
#' (hiv |> dnastring2codonmat())[,c(1, 2)] |> codonmat2pnps()
#' @export codonmat2pnps
#' @author Kristian K Ullrich

codonmat2pnps <- function(codonmat){
    stopifnot("Error: input needs to be a codonmat of 2 columns"=
        dim(codonmat)[2] == 2)
    seq1_name <- colnames(codonmat)[1]
    seq2_name <- colnames(codonmat)[2]
    count_codons <- dim(codonmat)[1]
    insertions_idx <- unique(unlist(apply(codonmat, 2,
        function(x) grep("-", x))))
    count_insertions <- length(insertions_idx)
    if(count_insertions > 0){
        codonmat <- codonmat[-insertions_idx, , drop = FALSE]
    }
    codonnumber <- apply(codonmat, 2, MSA2dist::codon2numberTCAG)
    Ns_idx <- unique(unlist(apply(codonnumber, 2, function(x) which(is.na(x)))))
    count_Ns <- length(Ns_idx)
    if(count_Ns > 0){
        codonmat <- codonmat[-Ns_idx, , drop = FALSE]
        codonnumber <- codonnumber[-Ns_idx, , drop = FALSE]
    }
    SA_Nei <- sum(MSA2dist::GENETIC_CODE_TCAG[codonmat[, 1], 4])
    SB_Nei <- sum(MSA2dist::GENETIC_CODE_TCAG[codonmat[, 2], 4])
    identical_codons_idx <- which(codonnumber[, 1] == codonnumber[, 2])
    identical_codons <- length(identical_codons_idx)
    if(identical_codons > 0){
        codonmat <- codonmat[-identical_codons_idx, , drop = FALSE]
        codonnumber <- codonnumber[-identical_codons_idx, , drop = FALSE]
    }
    ## At this point, codonA and codonB are "real" codons (no N's or -'s)
    ## but are not identical
    syn_codons <- 0
    nonsyn_codons <- 0
    if(nrow(codonmat) > 0){
        syn_nonsyn_codons <- apply(codonmat, 1,
            function(x) MSA2dist::compareCodons(x[1], x[2]))
        syn_codons <- syn_codons + sum(syn_nonsyn_codons[1, ])
        nonsyn_codons <- nonsyn_codons + sum(syn_nonsyn_codons[2, ])
    }
    count_ambiguous_codons <- count_insertions + count_Ns
    count_compared_codons <- count_codons - count_ambiguous_codons
    potential_syn <- ((SA_Nei / 3) + (SB_Nei / 3)) / 2
    potential_nonsyn <- (3 * count_compared_codons) - potential_syn
    ps <- syn_codons / potential_syn
    pn <- nonsyn_codons / potential_nonsyn
    ds <- ifelse(ps < 0.75 , ((-3 / 4) * log(1 - (4 * (ps / 3)))), NA)
    dn <- ifelse(pn < 0.75, ((-3 / 4) * log(1 - (4 * (pn / 3)))), NA)
    dnds <- NA
    if(dn != 0 & ds != 0 & !is.na(dn) & !is.na(ds)){ dnds <- dn/ds}
    pnps <- NA
    if(pn != 0 & ps != 0 & !is.na(pn) & !is.na(ps)){ pnps <- pn/ps}
    codonmat_out <- setNames(c(seq1_name, seq2_name, count_codons,
        count_compared_codons, count_ambiguous_codons, count_insertions,
        count_Ns, syn_codons, nonsyn_codons, potential_syn, potential_nonsyn,
        ps, pn, pnps, ds, dn, dnds),
        c("seq1", "seq2", "Codons", "Compared", "Ambigiuous", "Indels", "Ns",
        "Sd", "Sn", "S", "N", "ps", "pn", "pn/ps", "ds", "dn", "dn/ds"))
    attr(codonmat_out, "class") <- "pnps"
    return(codonmat_out)
}

data(hiv)

test_that("codonmat2pnps()", {
    expect_true((hiv |> dnastring2codonmat() |> codonmat2xy())$NonSynSum[1]
    == 40)
    expect_true((hiv |> dnastring2codonmat() |>
    codonmat2xy(threads=2))$NonSynSum[1] == 40)
})

## ----echo=FALSE, results='hide', warning=FALSE, message=FALSE-----------------
suppressPackageStartupMessages({
    library(MSA2dist)
    library(Biostrings)
    library(ape)
    library(dplyr)
    library(tidyr)
    library(ggplot2)
    })

## -----------------------------------------------------------------------------
# load MSA2dist
library(MSA2dist)
# load example data
data(hiv, package="MSA2dist")
data(AAMatrix, package="MSA2dist")
data(woodmouse, package="ape")

## -----------------------------------------------------------------------------
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
## define names
names(cds1.cds2.aln) <- c("seq1", "seq2")
## convert into alignment
cds1.cds2.aln |> dnastring2aln()

## -----------------------------------------------------------------------------
## convert back into DNAStringSet
cds1.cds2.aln |> dnastring2aln() |> aln2dnastring()

## -----------------------------------------------------------------------------
## convert into alignment
cds1.cds2.aln |> dnastring2dnabin()

## -----------------------------------------------------------------------------
## convert back into DNAStringSet
cds1.cds2.aln |> dnastring2dnabin() |> dnabin2dnastring()
## use woodmouse data
woodmouse |> dnabin2dnastring()

## -----------------------------------------------------------------------------
## translate cds into aa
aa1.aa2.aln <- cds1.cds2.aln |> cds2aa()
## convert into alignment
aa1.aa2.aln |> aastring2aln()

## -----------------------------------------------------------------------------
## convert back into AAStringSet
aa1.aa2.aln |> aastring2aln() |> aln2aastring()

## -----------------------------------------------------------------------------
## convert into AAbin
aa1.aa2.aln |> aastring2aabin()

## -----------------------------------------------------------------------------
## convert back into AAStringSet
aa1.aa2.aln |> aastring2aabin() |> aabin2aastring()

## -----------------------------------------------------------------------------
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
## define names
names(cds1.cds2.aln) <- c("seq1", "seq2")
## translate cds into aa
cds1.cds2.aln |> cds2aa()
aa1.aa2.aln <- cds1.cds2.aln |> cds2aa()

## -----------------------------------------------------------------------------
## translate cds into aa using frame = 2
## result is empty due to not multiple of three
cds1.cds2.aln |> cds2aa(frame=2)
## translate cds into aa using frame = 2 and shorten = TRUE
cds1.cds2.aln |> cds2aa(frame=2, shorten=TRUE)
## translate cds into aa using frame = 3 and shorten = TRUE
cds1.cds2.aln |> cds2aa(frame=3, shorten=TRUE)
## use woodmouse data
woodmouse |> dnabin2dnastring() |> cds2aa(shorten=TRUE)

## -----------------------------------------------------------------------------
## alternative genetic code
## use woodmouse data
woodmouse |> dnabin2dnastring() |> cds2aa(shorten=TRUE,
    genetic.code=Biostrings::getGeneticCode("2"))

## -----------------------------------------------------------------------------
## calculate pairwise AA distances based on Grantham's distance
aa.dist <- hiv |> cds2aa() |> aastring2dist(score=granthamMatrix())
## obtain distances
head(aa.dist$distSTRING)
## obtain pairwise sites used
head(aa.dist$sitesUsed)

## -----------------------------------------------------------------------------
## create and plot bionj tree
aa.dist.bionj <- ape::bionj(as.dist(aa.dist$distSTRING))
plot(aa.dist.bionj)

## -----------------------------------------------------------------------------
## use AAMatrix data
head(AAMatrix)
aa.dist.AAMatrix <- hiv |> cds2aa() |> aastring2dist(score=AAMatrix)
head(aa.dist.AAMatrix$distSTRING)

## -----------------------------------------------------------------------------
## use hiv data
dna.dist <- hiv |> dnastring2dist(model="K80")
## obtain distances
head(dna.dist$distSTRING)
## obtain pairwise sites used
head(dna.dist$sitesUsed)

## -----------------------------------------------------------------------------
## create and plot bionj tree
dna.dist.bionj <- ape::bionj(as.dist(dna.dist$distSTRING))

## -----------------------------------------------------------------------------
## creation of the association matrix:
association <- cbind(aa.dist.bionj$tip.label, aa.dist.bionj$tip.label)
## cophyloplot
ape::cophyloplot(aa.dist.bionj,
    dna.dist.bionj,
    assoc=association,
    length.line=4,
    space=28,
    gap=3,
    rotate=FALSE)

## -----------------------------------------------------------------------------
## use hiv data
hiv.dist.iupac <- head(hiv |> dnastring2dist(model="IUPAC"))
head(hiv.dist.iupac$distSTRING)
## run multi-threaded
system.time(hiv |> dnastring2dist(model="IUPAC", threads=1))
system.time(hiv |> dnastring2dist(model="IUPAC", threads=2))

## -----------------------------------------------------------------------------
## use woodmouse data
woodmouse.dist <- woodmouse |> dnabin2dnastring() |> dnastring2dist()
head(woodmouse.dist$distSTRING)

## -----------------------------------------------------------------------------
## use hiv data
## model Li
head(hiv |> dnastring2kaks(model="Li"))
## model NG86
head(hiv |> dnastring2kaks(model="NG86", threads=1))

## -----------------------------------------------------------------------------
## use hiv data
## model MYN
head(hiv |> dnastring2kaks(model="MYN"))
## model YN
head(hiv |> dnastring2kaks(model="YN", threads=1))

## -----------------------------------------------------------------------------
## use hiv data
idx <- list(c(2, 3), c(5,7,9))
## model MYN
head(hiv |> indices2kaks(idx, model="MYN"))
## model YN
head(hiv |> indices2kaks(idx, model="YN", threads=1))

## -----------------------------------------------------------------------------
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
## convert into alignment
cds1.cds2.aln |> dnastring2codonmat()

## -----------------------------------------------------------------------------
## use frame 2 and shorten to circumvent multiple of three error
cds1 <- Biostrings::DNAString("-ATGCAACATTGC-")
cds2 <- Biostrings::DNAString("-ATG---CATTGC-")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
cds1.cds2.aln |> dnastring2codonmat(frame=2, shorten=TRUE)

## -----------------------------------------------------------------------------
## use hiv data
## calculate average behavior
hiv.xy <- hiv |> dnastring2codonmat() |> codonmat2xy()

## -----------------------------------------------------------------------------
print(hiv.xy |> dplyr::select(Codon,SynMean,NonSynMean,IndelMean) |>
    tidyr::gather(variable, values, -Codon) |>
    ggplot2::ggplot(aes(x=Codon, y=values)) + 
    ggplot2::geom_line(aes(colour=factor(variable))) + 
    ggplot2::geom_point(aes(colour=factor(variable))) + 
    ggplot2::ggtitle("HIV-1 sample 136 patient 1 from
        Sweden envelope glycoprotein (env) gene"))

## ----sessionInfo, echo=TRUE---------------------------------------------------
sessionInfo()

#' @title addpop2string
#' @name addpop2string
#' @description This function adds population information to a
#' \code{DNAStringSet} or an \code{AAStringSet} and puts them into the
#' \code{metadata} information.\cr
#' __Note__: All unassigned sequences will be put into pop "unassigned"!\cr
#' Do not use "unassigned" as a population name!\cr
#' __Note__: Names in a population in the poplist must match sequence names!\cr
#' __Note__: Duplicated assignments are allowed!\cr
#' @param seq \code{DNAStringSet} or \code{AAStringSet} [mandatory]
#' @param poplist named \code{list} of populations either as index or names per
#' population (do not mix index and names in one population) [mandatory]
#' @return An object of class \code{DNAStringSet} or \code{AAStringSet}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @seealso \code{\link[MSA2dist]{addmask2string}},
#' \code{\link[MSA2dist]{addregion2string}},
#' \code{\link[MSA2dist]{addpos2string}}
#' @examples
#' ## load example sequence data
#' data(iupac, package="MSA2dist")
#' iupac.aa <- iupac |> cds2aa(shorten = TRUE)
#' ## create poplist
#' poplist <- list(FRA = grep("Mmd.FRA", names(iupac)),
#'     GER = grep("Mmd.GER", names(iupac)),
#'     IRA = grep("Mmd.IRA", names(iupac)),
#'     AFG = grep("Mmm.AFG", names(iupac)))
#' iupac.aa <- iupac.aa |> addpop2string(poplist)
#' #(iupac.aa |> slot("metadata"))$pop.integer
#' iupac.aa |> popinteger()
#' #(iupac.aa |> slot("metadata"))$pop.names
#' iupac.aa |> popnames()
#' ## mxixing index and names
#' poplist <- list(FRA = names(iupac)[grep("Mmd.FRA", names(iupac))],
#'     GER = grep("Mmd.GER", names(iupac)),
#'     IRA = names(iupac)[grep("Mmd.IRA", names(iupac))],
#'     AFG = grep("Mmm.AFG", names(iupac)))
#' iupac.aa <- iupac.aa |> addpop2string(poplist)
#' iupac.aa |> popinteger()
#' iupac.aa |> popnames()
#' ## leaving out some sequences which will be assigned as "unassigned"
#' poplist <- list(FRA = names(iupac)[grep("Mmd.FRA", names(iupac))],
#'     GER = grep("Mmd.GER", names(iupac)),
#'     IRA = names(iupac)[grep("Mmd.IRA", names(iupac))])
#' iupac.aa <- iupac.aa |> addpop2string(poplist)
#' iupac.aa |> popinteger()
#' iupac.aa |> popnames()
#' @export addpop2string
#' @author Kristian K Ullrich

addpop2string <- function(seq, poplist){
    stopifnot("Error: input needs to be a DNAStringSet or AAStringSet"=
        methods::is(seq, "AAStringSet") | methods::is(seq, "DNAStringSet"))
    stopifnot("Error: input needs to be a list"=
        methods::is(poplist, "list"))
    poplist.integer <- poplist
    poplist.names <- poplist
    poplist.type_is_integer <- which(unlist(lapply(poplist, is.integer)))
    poplist.type_is_names <- which(!(names(poplist)
        %in% names(poplist.type_is_integer)))
    poplist.integer[poplist.type_is_names] <- lapply(
        poplist[c(poplist.type_is_names)], function(x) unlist(lapply(x,
        function(y) grep(y, names(seq)))))
    poplist.names[poplist.type_is_integer] <- lapply(
        poplist[c(poplist.type_is_integer)], function(x) unlist(lapply(x,
        function(y) names(seq)[y])))
    unassigned.integer <- which(!(seq(from = 1, to = length(seq))
        %in% unlist(poplist.integer)))
    unassigned.names <- names(seq)[unassigned.integer]
    if(length(unassigned.names)!=0){
        poplist.integer$unassigned <- unassigned.integer
        poplist.names$unassigned <- unassigned.names
    }
    methods::slot(seq, "metadata")$pop.integer <- poplist.integer
    methods::slot(seq, "metadata")$pop.names <- poplist.names
    return(seq)
}

data(iupac)

test_that("addpop2string() outputs DNAStringSet", {
    poplist <- list(FRA = grep("Mmd.FRA", names(iupac)))
    iupac <- iupac |> addpop2string(poplist)
    expect_true(iupac@metadata$pop.integer$FRA[1] == 1)
})

data(iupac)

test_that("popinteger() outputs list", {
    poplist <- list(FRA = grep("Mmd.FRA", names(iupac)))
    iupac <- iupac |> addpop2string(poplist)
    expect_true(popinteger(iupac)$FRA[1] == 1)
})

data(iupac)

test_that("popnames() outputs list", {
    poplist <- list(FRA = grep("Mmd.FRA", names(iupac)))
    iupac <- iupac |> addpop2string(poplist)
    expect_true(popnames(iupac)$FRA[1] == "Mmd.FRA.14")
})

#' @title aastring2aabin
#' @name aastring2aabin
#' @description This function converts an \code{AAStringSet}
#' into an \code{ape} \code{AAbin}.
#' @param aa \code{AAStringSet} [mandatory]
#' @return An object of class \code{AAbin}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' AAMultipleAlignment
#' @importFrom ape as.AAbin
#' @seealso \code{\link[seqinr]{as.alignment}}
#' \code{\link[ape]{as.DNAbin.alignment}}
#' @examples
#' ## define two cds sequences
#' cds1 <- Biostrings::DNAString("ATGCAACATTGC")
#' cds2 <- Biostrings::DNAString("ATG---CATTGC")
#' cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
#'     Biostrings::DNAStringSet(cds2))
#' ## convert into AAbin
#' #aastring2aabin(cds2aa(cds1.cds2.aln))
#' cds1.cds2.aln |> cds2aa() |> aastring2aabin()
#' @export aastring2aabin
#' @author Kristian K Ullrich

aastring2aabin <- function(aa){
    stopifnot("Error: input needs to be an AAStringSet"=
                methods::is(aa, "AAStringSet"))
    return(ape::as.AAbin(Biostrings::AAMultipleAlignment(aa)))
}

data(woodmouse, package="ape")

test_that("aabin2aastring() outputs AAStringSet", {
    expect_true(class(ape::trans(woodmouse[,1:6], 2) |> aabin2aastring())
    == "AAStringSet")
    expect_true(class(Biostrings::AAStringSet("M") |> aastring2aabin() |>
    aabin2aastring()) == "AAStringSet")
})

test_that("aastring2aabin() outputs AAStringSet", {
    cds1 <- Biostrings::DNAString("ATGCAACATTGC")
    cds2 <- Biostrings::DNAString("ATG---CATTGC")
    cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
        Biostrings::DNAStringSet(cds2))
    expect_true(class(cds1.cds2.aln |> cds2aa() |> aastring2aabin())
    == "AAbin")
})

#' @title codon2numberTCAG
#' @name codon2numberTCAG
#' @description This function converts a \code{codon} into a \code{number}.
#' @param codon [mandatory]
#' @return An object of class \code{numeric}
#' @importFrom Biostrings GENETIC_CODE
#' @importFrom stringr str_sub
#' @importFrom stats setNames
#' @seealso \code{\link[Biostrings]{GENETIC_CODE}}
#' @examples
#' #unlist(lapply(names(Biostrings::GENETIC_CODE), codon2numberTCAG))
#' names(Biostrings::GENETIC_CODE) |> codon2numberTCAG()
#' @export codon2numberTCAG
#' @author Kristian K Ullrich

codon2numberTCAG <- function(codon){
    baseNumber <- setNames(c(0, 1, 2, 3), c("T", "C", "A", "G"))
    left <- baseNumber[stringr::str_sub(codon, 1, 1)]
    mid <- baseNumber[stringr::str_sub(codon, 2, 2)]
    right <- baseNumber[stringr::str_sub(codon, 3, 3)]
    codonnumber <- (mid * 16) + (left * 4) + (right) + 1
    names(codonnumber) <- codon
    return(codonnumber)
}

#' @title dnastring2dnabin
#' @name dnastring2dnabin
#' @description This function converts a \code{DNAStringSet} into an
#' \code{ape} \code{DNAbin}.
#' @param dna \code{DNAStringSet} [mandatory]
#' @return An object of class \code{DNAbin}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom seqinr as.alignment
#' @importFrom ape as.DNAbin.alignment
#' @seealso \code{\link[seqinr]{as.alignment}}
#' \code{\link[ape]{as.DNAbin.alignment}}
#' @examples
#' ## define two cds sequences
#' cds1 <- Biostrings::DNAString("ATGCAACATTGC")
#' cds2 <- Biostrings::DNAString("ATG---CATTGC")
#' cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
#'     Biostrings::DNAStringSet(cds2))
#' ## convert into DNAbin
#' #dnastring2dnabin(cds1.cds2.aln)
#' cds1.cds2.aln |> dnastring2dnabin()
#' @export dnastring2dnabin
#' @author Kristian K Ullrich

dnastring2dnabin <- function(dna){
    stopifnot("Error: input needs to be a DNAStringSet"=
        methods::is(dna, "DNAStringSet"))
    alignment.nb <- length(dna)
    alignment.nam <- names(dna)
    alignment.seq <- tolower(as.character(dna))
    names(alignment.seq) <- NULL
    alignment.com <- NA
    alignment <- list(alignment.nb, alignment.nam, alignment.seq, alignment.com)
    names(alignment) <- c("nb", "nam", "seq", "com")
    attr(alignment, "class") <- "alignment"
    alignment.bin <- ape::as.DNAbin.alignment(alignment)
    return(alignment.bin)
}

test_that("dnastring2dnabin() outputs DNAbin", {
    cds1 <- Biostrings::DNAString("ATGCAACATTGC")
    cds2 <- Biostrings::DNAString("ATG---CATTGC")
    cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
        Biostrings::DNAStringSet(cds2))
    expect_true(class(cds1.cds2.aln |> dnastring2dnabin()) == "DNAbin")
})

#' @title addpos2string
#' @name addpos2string
#' @description This function adds \code{GenomicRanges} information,
#' \code{CHROM}, \code{START} and \code{END} to a
#' \code{DNAStringSet} or an \code{AAStringSet} and puts them into the
#' \code{metadata} information.
#' This information can be used to find overlaps with a chromosome wide mask.
#' @param seq \code{DNAStringSet} or \code{AAStringSet} [mandatory]
#' @param chrom chromosome name [mandatory]
#' @param start start position [mandatory]
#' @param end end position [mandatory]
#' @return An object of class \code{DNAStringSet} or \code{AAStringSet}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom GenomicRanges GRanges
#' @importFrom IRanges IRanges IRangesList reduce start end findOverlaps
#' disjoin overlapsRanges
#' @seealso \code{\link[MSA2dist]{addmask2string}},
#' \code{\link[MSA2dist]{addregion2string}},
#' \code{\link[MSA2dist]{addpop2string}}
#' @examples
#' ## load example sequence data
#' data(iupac, package="MSA2dist")
#' ## add position
#' iupac <- iupac |> addpos2string(chrom="chr1", start=1, end=1000)
#' #(iupac |> slot("metadata"))$GRanges
#' iupac |> getpos()
#' @export addpos2string
#' @author Kristian K Ullrich

addpos2string <- function(seq, chrom = NULL, start = NULL, end = NULL){
    stopifnot("Error: input needs to be a DNAStringSet or AAStringSet"=
        methods::is(seq, "AAStringSet") | methods::is(seq, "DNAStringSet"))
    stopifnot("Error: set chromosome name"= !is.null(chrom))
    stopifnot("Error: set start"= !is.null(start))
    stopifnot("Error: set end"= !is.null(end))
    if((end - start + 1)!=unique(width(seq))){
        warning("defined region (start, end) has unequal length to seq")
    }
    seq.GRanges <- GenomicRanges::GRanges(seqnames=chrom,
        ranges=IRanges::IRanges(start=start, end=end))
    methods::slot(seq, "metadata")$GRanges <- seq.GRanges
    return(seq)
}

data(iupac)

test_that("addpos2string() outputs DNAStringSet", {
    iupac <- iupac |> addpos2string(chrom="chr1", start=1, end=1000)
    expect_true(as.character(iupac@metadata$GRanges@seqnames) == "chr1")
})

#' @title AAMatrix-data
#' @name AAMatrix-data
#' @description \code{getAAMatrix()}  from the \code{alakazam} package.
#' @return \code{score matrix}
#' @references Gupta N, Vander Heiden J, Uduman M, Gadala-Maria D, Yaari G,
#' Kleinstein S (2015) Change-O: a toolkit for analyzing large-scale B cell
#' immunoglobulin repertoire sequencing data. \emph{Bioinformatics.}
#' \bold{31(20)}, 3356-3358.
#' @docType data
#' @usage data(AAMatrix)
#' @format an object of class \code{matrix}
#' @keywords datasets
#' @examples
#' data("AAMatrix", package="MSA2dist")

"AAMatrix"

#' @title dnastring2dist
#' @name dnastring2dist
#' @description This function calculates pairwise distances for all
#' combinations of a \code{DNAStringSet}.
#' @param dna \code{DNAStringSet} [mandatory]
#' @param model specify model either "IUPAC" or any model from
#' \code{ape::dist.dna} [default: IUPAC]
#' @param threads number of parallel threads [default: 1]
#' @param symmetric symmetric score matrix [default: TRUE]
#' @param score \code{score matrix} use score matrix to calculate
#' distances [default: NULL]
#' @param mask \code{IRanges} object indicating masked sites
#' [default: NULL]
#' @param region \code{IRanges} object indicating region to use for dist
#' calculation. Default is null, meaning all sites are used [default: NULL]
#' @param ... other \code{ape::dist.dna} parameters
#' (see \code{\link[ape]{dist.dna}})
#' @return A data.frame of pairwise distance values \code{distSTRING} and
#' sites used \code{sitesUsed}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom ape dist.dna
#' @importFrom IRanges IRanges IRangesList reduce start end findOverlaps
#' disjoin overlapsRanges
#' @seealso \code{\link[ape]{dist.dna}}
#' @examples
#' ## load example sequence data
#' data("hiv", package="MSA2dist")
#' #dnastring2dist(hiv, model="IUPAC")
#' hiv |> dnastring2dist(model="IUPAC")
#' #dnastring2dist(hiv, model="K80")
#' hiv |> dnastring2dist(model="K80")
#' data("woodmouse", package="ape")
#' #dnastring2dist(dnabin2dnastring(woodmouse), score=iupacMatrix())
#' woodmouse |> dnabin2dnastring() |> dnastring2dist()
#' #dnastring2dist(hiv, model = "IUPAC", threads = 2)
#' hiv |> dnastring2dist(model = "IUPAC", threads = 2)
#' ## create mask
#' mask1 <- IRanges::IRanges(start=c(1,61,121), end=c(30,90,150))
#' ## use mask
#' hiv |> dnastring2dist(model="IUPAC", mask=mask1)
#' ## use region
#' region1 <- IRanges::IRanges(start=c(1,139), end=c(75,225))
#' hiv |> dnastring2dist(model="IUPAC", region=region1)
#' ## use mask and region
#' hiv |> dnastring2dist(model="IUPAC", mask=mask1, region=region1)
#' ## use asymmetric score matrix
#' myscore <- iupacMatrix()
#' myscore[1, 4] <- 0.5
#' (hiv |> dnastring2dist(score=myscore, symmetric=FALSE))$distSTRING[1:2, 1:2]
#' @export dnastring2dist
#' @author Kristian K Ullrich

dnastring2dist <- function(dna, model="IUPAC", threads=1, symmetric=TRUE,
    score=NULL, mask=NULL, region=NULL, ...){
    stopifnot("Error: Input needs to be DNAStringSet"=
        methods::is(dna, "DNAStringSet"))
    if(symmetric){symmetric_int <- 1}else{symmetric_int <- 0}
    region.dna <- IRanges::IRanges(start=1, end=unique(width(dna)))
    if(!is.null(mask) || !is.null(region)){
        dna.region <- MSA2dist::string2region(dna, mask=mask, region=region)
        dna.char <- as.character(dna.region)
        region.dna <- dna.region@metadata$regionUsed
    } else{dna.char <- as.character(dna)}
    if(!is.null(score)){
        OUT <- rcpp_distSTRING(dnavector=dna.char,
            scoreMatrix=score, ncores=threads, symmetric=symmetric_int)
        OUT$distSTRING <- as.data.frame(OUT$distSTRING)
        OUT$sitesUsed <- as.data.frame(OUT$sitesUsed)
    }
    if(is.null(score)){
        stopifnot("Error: either choose model 'IUPAC' or '?ape::dist.dna'"=
            model %in%
            c("IUPAC", "raw", "N", "TS", "TV", "JC69", "K80", "F81",
            "K81","F84", "BH87", "T92", "TN93", "GG95", "logdet",
            "paralin", "indel", "indelblock"))
        if(model == "IUPAC"){
            OUT <- rcpp_distSTRING(dnavector=dna.char,
                scoreMatrix=iupacMatrix(), ncores=threads,
                symmetric=symmetric_int)
            OUT$distSTRING <- as.data.frame(OUT$distSTRING)
            OUT$sitesUsed <- as.data.frame(OUT$sitesUsed)
        }
        if(model != "IUPAC"){
            if(!is.null(mask) || !is.null(region)){
                distSTRING_ <- as.matrix(ape::dist.dna(
                    x=MSA2dist::dnastring2dnabin(dna.region$seq),
                    model=model, ...))
                sitesUsed_ <- rcpp_pairwiseDeletionDNA(
                    dnavector=dna.char,
                    ncores=threads,
                    symmetric=symmetric_int)
            } else{
                distSTRING_ <- as.matrix(ape::dist.dna(
                    x=MSA2dist::dnastring2dnabin(dna),
                    model=model, ...))
                sitesUsed_ <- rcpp_pairwiseDeletionDNA(
                    dnavector=as.character(dna),
                    ncores=threads,
                    symmetric=symmetric_int)
            }
            OUT <- list(distSTRING = as.data.frame(distSTRING_))
            OUT <- append(OUT, sitesUsed_)
            OUT$sitesUsed <- as.data.frame(OUT$sitesUsed)
        }
    }
    OUT$regionUsed <- region.dna
    return(OUT)
}

data(hiv)

test_that("compareCodons()", {
    myscore <- iupacMatrix()
    myscore[1,1] <- -1
    h <- hiv |> dnastring2dist(score=myscore)
    expect_true(h$sitesUsed[1,1] == 150)
    h <- hiv |> dnastring2dist(model="IUPAC")
    expect_true(h$sitesUsed[1,1] == 273)
    h <- hiv |> dnastring2dist(model="K80")
    expect_true(h$sitesUsed[1,1] == 273)
    mask1 <- IRanges::IRanges(start=c(1,61,121), end=c(30,90,150))
    h <- hiv |> dnastring2dist(model="IUPAC", mask=mask1)
    expect_true(h$sitesUsed[1,1] == 183)
    region1 <- IRanges::IRanges(start=c(1,139), end=c(75,225))
    h <- hiv |> dnastring2dist(model="IUPAC", region=region1)
    expect_true(h$sitesUsed[1,1] == 162)
    h <- hiv |> dnastring2dist(model="IUPAC", mask=mask1, region=region1)
    expect_true(h$sitesUsed[1,1] == 105)
    myscore[1,4] <- 0.5
    h <- hiv |> dnastring2dist(score=myscore)
    expect_true(h$distSTRING[1,2] == h$distSTRING[2,1])
    h <- hiv |> dnastring2dist(score=myscore, symmetric=FALSE)
    expect_true(h$distSTRING[1,2] != h$distSTRING[2,1])
})

#' @title cdsstring2codonaln
#' @name cdsstring2codonaln
#' @description This function takes two sequences as \code{DNAStringSet},
#' and their corresponding \code{AAStringSet}, calculates
#' a global alignment and converts this alignment back into a codon alignment.
#' @param cds two sequences \code{DNAStringSet} [mandatory]
#' @param aa two sequences \code{AAStringSet} [mandatory]
#' @param type type of alignment (see
#' \code{\link[pwalign]{pairwiseAlignment}}) [default: global]
#' @param substitutionMatrix substitution matrix representing the fixed
#' substitution scores for an alignment (see
#' \code{\link[pwalign]{pairwiseAlignment}}) [default: BLOSUM62]
#' @param gapOpening the cost for opening a gap in the alignment (see
#' \code{\link[pwalign]{pairwiseAlignment}}) [default: 10]
#' @param gapExtension the incremental cost incurred along the length of the
#' gap in the alignment (see \code{\link[pwalign]{pairwiseAlignment}})
#' [default: 0.5]
#' @param remove.gaps specify if gaps in the codon alignment should be removed
#' [default: FALSE]
#' @return codon alignment as \code{DNAStringSet}
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom pwalign pairwiseAlignment
#' @importFrom methods is slot
#' @references Pagès, H et al. (2014) Biostrings: Efficient manipulation of
#' biological strings. \emph{R package version}, \bold{2(0)}.
#' @seealso \code{\link[pwalign]{pairwiseAlignment}}
#' @examples
#' ## define two cds sequences
#' cds <- Biostrings::DNAStringSet(c("ATGCAACATTGC", "ATGCATTGC"))
#' names(cds) <- c("cds1", "cds2")
#' ## get protein alignment
#' aa <- MSA2dist::cds2aa(cds)
#' cdsstring2codonaln(cds, aa)
#' @export cdsstring2codonaln
#' @author Kristian K Ullrich

cdsstring2codonaln <- function(cds, aa, type="global",
    substitutionMatrix="BLOSUM62", gapOpening=10, gapExtension=0.5,
    remove.gaps=FALSE){
    stopifnot("Error: cds needs to be DNAStringSet"=
        {methods::is(cds, "DNAStringSet")})
    stopifnot("Error: aa needs to be an AAStringSet"=
        {methods::is(aa, "AAStringSet")})
    if(methods::is(cds, "DNAStringSet")){
        stopifnot("Error: cds needs to only contain two sequences"=
            length(cds) == 2)
    }
    if(methods::is(aa, "AAStringSet")){
        stopifnot("Error: aa needs to only contain two sequences"=
            length(aa) == 2)
    }
    xy.aln <- makePostalignedSeqs(pwalign::pairwiseAlignment(aa[1], aa[2],
        type=type, substitutionMatrix=substitutionMatrix, gapOpening=gapOpening,
        gapExtension=gapExtension))[[1L]]
    names(xy.aln) <- names(aa)
    if(type=="local"){
        xy.aln.x <- gsub("\\*", "X", gsub("-", "", xy.aln[[1]]))
        cds1_local_pos <- gregexpr(xy.aln.x, gsub("\\*", "X", aa[[1]]))
        xy.aln.y <- gsub("\\*", "X", gsub("-", "", xy.aln[[2]]))
        cds2_local_pos <- gregexpr(xy.aln.y, gsub("\\*", "X", aa[[2]]))
        cds1_local <- Biostrings::subseq(cds[[1]],
            (cds1_local_pos[[1]][1]*3)-2,
            (cds1_local_pos[[1]][1]+nchar(xy.aln.x)-1)*3)
        cds2_local <- Biostrings::subseq(cds[[2]],
            (cds2_local_pos[[1]][1]*3)-2,
            (cds2_local_pos[[1]][1]+nchar(xy.aln.y)-1)*3)
        cds <- setNames(Biostrings::DNAStringSet(list(cds1_local,
            cds2_local)), names(cds))
    }
    xy.cds.aln <- MSA2dist::pal2nal(xy.aln, cds, remove.gaps=remove.gaps)
    return(xy.cds.aln)
}

data(hiv)

test_that("cdsstring2codonaln() outputs DNAStringSet", {
    cds <- Biostrings::DNAStringSet(c("ATGCAACATTGC", "ATGCATTGC"))
    names(cds) <- c("cds1", "cds2")
    aa <- MSA2dist::cds2aa(cds)
    cds1.cds2.aln <- cdsstring2codonaln(cds, aa)
    expect_true(as.character(cds1.cds2.aln[1]) == "ATGCAACATTGC")
    expect_true(as.character(cds1.cds2.aln[2]) == "ATG---CATTGC")
    cds1.cds2.aln.rem <- cdsstring2codonaln(cds, aa, remove.gaps=TRUE)
    expect_true(as.character(cds1.cds2.aln.rem[1]) == "ATGCATTGC")
    expect_true(as.character(cds1.cds2.aln.rem[2]) == "ATGCATTGC")
    cds1.cds2.aln.local <- cdsstring2codonaln(cds, aa, type="local")
    expect_true(as.character(cds1.cds2.aln.local[1]) == "CAACATTGC")
    expect_true(as.character(cds1.cds2.aln.local[2]) == "ATGCATTGC")
})

#' @title addmask2string
#' @name addmask2string
#' @description This function adds mask information as an \code{IRanges} object,
#' \code{START} and \code{END} information, to a
#' \code{DNAStringSet} or an \code{AAStringSet} and puts them into the
#' \code{metadata} information.
#' This information can be used to restrict the distance calculation to
#' specific regions of the \code{DNAStringSet} or the \code{AAStringSet}.
#' @param seq \code{DNAStringSet} or \code{AAStringSet} [mandatory]
#' @param mask \code{IRanges} object [mandatory]
#' @param append indicate if mask should be appended or overwritten
#' [default: TRUE]
#' @return An object of class \code{DNAStringSet} or \code{AAStringSet}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom IRanges IRanges IRangesList reduce start end findOverlaps
#' disjoin overlapsRanges
#' @seealso \code{\link[MSA2dist]{addregion2string}},
#' \code{\link[MSA2dist]{addpop2string}},
#' \code{\link[MSA2dist]{addpos2string}}
#' @examples
#' ## load example sequence data
#' data(iupac, package="MSA2dist")
#' iupac.aa <- iupac |> cds2aa(shorten = TRUE)
#' ## create mask
#' mask1 <- IRanges::IRanges(start=c(1,41), end=c(20,50))
#' ## add mask
#' iupac.aa <- iupac.aa |> addmask2string(mask=mask1)
#' #(iupac.aa |> slot("metadata"))$mask
#' iupac.aa |> getmask()
#' ## append mask
#' mask2 <- IRanges::IRanges(start=c(21), end=c(30))
#' iupac.aa <- iupac.aa |> addmask2string(mask=mask2)
#' #(iupac.aa |> slot("metadata"))$mask
#' iupac.aa |> getmask()
#' ## overwrite mask
#' iupac.aa <- iupac.aa |> addmask2string(mask=mask2, append=FALSE)
#' #(iupac.aa |> slot("metadata"))$mask
#' iupac.aa |> getmask()
#' ## reduce by mask
#' #iupac.aa.region <- iupac.aa |> string2region(mask=
#' #    (iupac.aa |> slot("metadata"))$mask)
#' iupac.aa.region <- iupac.aa |> string2region(mask=
#'     getmask(iupac.aa))
#' #iupac.aa.region |> slot("metadata")
#' iupac.aa.region |> getmask()
#' @export addmask2string
#' @author Kristian K Ullrich

addmask2string <- function(seq, mask = NULL, append = TRUE){
    stopifnot("Error: input needs to be a DNAStringSet or AAStringSet"=
        methods::is(seq, "AAStringSet") | methods::is(seq, "DNAStringSet"))
    stopifnot("Error: set mask"= !is.null(mask))
    stopifnot("Error: mask needs to be an IRanges object"=
        methods::is(mask, "IRanges"))
    if(append){
        if(is.null(slot(seq, "metadata")$mask)){
            #seq@metadata$mask <- IRanges::reduce(mask)
            methods::slot(seq, "metadata")$mask <- IRanges::reduce(mask)
        } else{
            methods::slot(seq, "metadata")$mask <- IRanges::reduce(
                c(methods::slot(seq, "metadata")$mask, mask))
        }
    } else{
        methods::slot(seq, "metadata")$mask <- IRanges::reduce(mask)
    }
    return(seq)
}

data(iupac)

test_that("getmask() outputs IRanges object", {
    mask1 <- IRanges::IRanges(start=c(1,41), end=c(20,50))
    iupac <- iupac |> addmask2string(mask=mask1)
    expect_true(getmask(iupac)@width[1] == 20)
})

data(iupac)

test_that("addmask2string() outputs DNAStringSet", {
    mask1 <- IRanges::IRanges(start=c(1,41), end=c(20,50))
    iupac <- iupac |> addmask2string(mask=mask1)
    expect_true(iupac@metadata$mask@width[1] == 20)
    mask2 <- IRanges::IRanges(start=c(21), end=c(30))
    iupac <- iupac |> addmask2string(mask=mask2)
    expect_true(iupac@metadata$mask@width[1] == 30)
    iupac <- iupac |> addmask2string(mask=mask2, append=FALSE)
    expect_true(iupac@metadata$mask@width[1] == 10)
})

#' @title aln2aastring
#' @name aln2aastring
#' @description This function converts a \code{seqinr} \code{alignment} into
#' an \code{AAStringSet}.
#' @param aln \code{seqinr} \code{alignment} [mandatory]
#' @return An object of class \code{AAStringSet}
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom seqinr as.alignment
#' @seealso \code{\link[seqinr]{as.alignment}}
#' \code{\link[Biostrings]{AAStringSet}}
#' @examples
#' ## define two cds sequences
#' cds1 <- Biostrings::DNAString("ATGCAACATTGC")
#' cds2 <- Biostrings::DNAString("ATG---CATTGC")
#' cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
#'     Biostrings::DNAStringSet(cds2))
#' #aastring2aln(cds2aa(cds1.cds2.aln))
#' cds1.cds2.aln |> cds2aa() |> aastring2aln() |> aln2aastring()
#' @export aln2aastring
#' @author Kristian K Ullrich

aln2aastring <- function(aln){
    aa <- Biostrings::AAStringSet(toupper(unlist(aln$seq)))
    names(aa) <- aln$nam
    return(aa)
}

#' @title codonmat2xy
#' @name codonmat2xy
#' @description This function calculates average behavior of each codon for all
#' pairwise comparisons for indels, syn, and nonsyn mutations according to
#' \emph{Nei and Gojobori (1986)}.
#' @param codonmat \code{codon matrix} obtained via
#' \code{\link[MSA2dist]{dnastring2codonmat}} [mandatory]
#' @param threads number of parallel threads [default: 1]
#' @return A \code{data.frame} object with the following components:\cr
#' \code{Codon} Codon index\cr
#' \code{n} number of comparison\cr
#' \code{SynSum} Sum of syn\cr
#' \code{NonSynSum} Sum of nonsyn\cr
#' \code{IndelSum} Sum of indels\cr
#' \code{SynMean} average syn per codon\cr
#' \code{NonSynMean} average nonsyn per codon\cr
#' \code{IndelMean} average indels per codon\cr
#' \code{CumSumSynMean} cumulative average syn per codon\cr
#' \code{CumSumNonSynMean} cumulative average nonsyn per codon\cr
#' \code{CumSumIndelMean} cumulative indels per codon\cr
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom seqinr kaks
#' @importFrom tidyr %>% unite
#' @importFrom tibble add_column
#' @importFrom foreach foreach %do% %dopar%
#' @importFrom parallel makeForkCluster stopCluster
#' @importFrom doParallel registerDoParallel
#' @importFrom dplyr group_by filter count left_join summarise mutate
#' @importFrom rlang .data
#' @seealso \code{\link[MSA2dist]{dnastring2codonmat}}
#' \code{\link[MSA2dist]{codonmat2pnps}}
#' \code{\link[MSA2dist]{dnastring2kaks}}
#' \code{\link[seqinr]{kaks}}
#' @references Nei and Gojobori. (1986) Simple methods for estimating the
#' numbers of synonymous and nonsynonymous nucleotide substitutions.
#' \emph{Mol. Biol. Evol.}, \bold{3(5)}, 418-426.
#' @references Ganeshan et al. (1997) Human immunodeficiency virus type 1
#' genetic evolution in children with different rates of development of disease.
#' \emph{J. Virology.} \bold{71(1)}, 663-677.
#' @references Yang et al. (2000) Codon-substitution models for heterogeneous
#' selection pressure at amino acid sites. \emph{Genetics.}
#' \bold{155(1)}, 431-449.
#' @examples
#' ## load example sequence data
#' data("hiv", package="MSA2dist")
#' #codonmat2xy(dnastring2codonmat(hiv))
#' hiv |> dnastring2codonmat() |> codonmat2xy()
#' #codonmat2xy(dnastring2codonmat(hiv), threads=2)
#' hiv |> dnastring2codonmat() |> codonmat2xy(threads=2)
#' @export codonmat2xy
#' @author Kristian K Ullrich

codonmat2xy <- function(codonmat, threads = 1){
    if(.Platform$OS.type == "windows"){
        cl <- parallel::makeCluster(threads)
    }
    if(.Platform$OS.type != "windows"){
        cl <- parallel::makeForkCluster(threads)
    }
    doParallel::registerDoParallel(cl)
    i <- NULL
    j <- NULL
    k <- NULL
    OUT <- foreach::foreach(i = seq(from = 1, to = nrow(codonmat)),
        .combine=rbind, .packages = c('foreach')) %dopar% {
        codon_i <- table(codonmat[i,])
####
#        foreach::foreach(j = seq(from = 1, to = ncol(codonmat) - 1),
#            .combine=rbind) %do% {
#            foreach::foreach(k = seq(from = j + 1, to = ncol(codonmat)),
#                .combine=rbind) %do% {
#        c(setNames(i, "Codon"),
#            setNames(j, "Comp1"),
#            setNames(k, "Comp2"),
#            setNames(MSA2dist::compareCodons(codonmat[i, j],
#            codonmat[i, k]), c("syn", "nonsyn", "indel")))
####
        foreach::foreach(j = seq(from = 1, to = length(codon_i) ),
            .combine=rbind) %do% {
            foreach::foreach(k = seq(from = j, to = length(codon_i)),
                .combine=rbind) %do% {
                if(j == k){
                    d <- do.call("rbind", rep(list(c(setNames(i, "Codon"),
                        setNames(j, "Comp1"),
                        setNames(k, "Comp2"),
                        setNames(MSA2dist::compareCodons(
                            names(codon_i)[j],
                            names(codon_i)[k]),
                            c("syn", "nonsyn", "indel")))),
                    (codon_i[j] * (codon_i[k] - 1)) / 2))
                }
                if(j != k){
                    d <- do.call("rbind", rep(list(c(setNames(i, "Codon"),
                        setNames(j, "Comp1"),
                        setNames(k, "Comp2"),
                        setNames(MSA2dist::compareCodons(
                            names(codon_i)[j],
                            names(codon_i)[k]),
                            c("syn", "nonsyn", "indel")))),
                    codon_i[j] * codon_i[k]))
                }
                d
            }
        }
    }
    parallel::stopCluster(cl)
    OUT <- as.data.frame(OUT)
    OUT.NAs <- OUT %>% dplyr::group_by(.data$Codon) %>%
        dplyr::filter(!is.na(.data$syn)) %>%
        dplyr::count(.data$Codon)
    OUT.SynSum <- OUT %>% dplyr::group_by(.data$Codon) %>%
        dplyr::summarise(SynSum = sum(.data$syn, na.rm = TRUE))
    OUT.NonSynSum <- OUT %>% dplyr::group_by(.data$Codon) %>%
        dplyr::summarise(NonSynSum = sum(.data$nonsyn, na.rm = TRUE))
    OUT.IndelSum <- OUT %>% dplyr::group_by(.data$Codon) %>%
        dplyr::summarise(IndelSum = sum(.data$indel, na.rm = TRUE))
    OUT.join <- dplyr::left_join(OUT.NAs, OUT.SynSum) %>%
        dplyr::left_join(OUT.NonSynSum) %>% dplyr::left_join(OUT.IndelSum)
    OUT.xy <- OUT.join %>% dplyr::mutate(SynMean = .data$SynSum/.data$n,
        NonSynMean = .data$NonSynSum/.data$n,
        IndelMean = .data$IndelSum/.data$n)
    OUT.xy <- OUT.xy %>%
        tibble::add_column(CumSumSynMean = cumsum(OUT.xy$SynMean),
        CumSumNonSynMean = cumsum(OUT.xy$NonSynMean),
        CumSumIndelMean = cumsum(OUT.xy$IndelMean))
    return(OUT.xy)
}

#' @title getpos
#' @name getpos
#' @description This function shows the position slot from a
#' \code{DNAStringSet} or an \code{AAStringSet}
#' \code{metadata} information.\cr
#' @param seq \code{DNAStringSet} or \code{AAStringSet} [mandatory]
#' @return \code{GenomicRanges} information from \code{metadata}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @seealso \code{\link[MSA2dist]{addpop2string}}
#' @examples
#' ## load example sequence data
#' data(iupac, package="MSA2dist")
#' ## add position
#' iupac <- iupac |> addpos2string(chrom="chr1", start=1, end=1000)
#' #(iupac |> slot("metadata"))$GRanges
#' iupac |> getpos()
#' @export getpos
#' @author Kristian K Ullrich

getpos <- function(seq){
    stopifnot("Error: input needs to be a DNAStringSet or AAStringSet"=
        methods::is(seq, "AAStringSet") | methods::is(seq, "DNAStringSet"))
    return(methods::slot(seq, "metadata")$GRanges)
}

#' @title region
#' @name region
#' @description This function shows the region slot from a
#' \code{DNAStringSet} or an \code{AAStringSet}
#' \code{metadata} information.\cr
#' @param seq \code{DNAStringSet} or \code{AAStringSet} [mandatory]
#' @return region \code{IRanges} object from \code{metadata}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @seealso \code{\link[MSA2dist]{addpop2string}}
#' @examples
#' ## load example sequence data
#' data(iupac, package="MSA2dist")
#' iupac.aa <- iupac |> cds2aa(shorten = TRUE)
#' ## create region
#' region1 <- IRanges::IRanges(start=c(1,41), end=c(20,50))
#' ## add region
#' iupac.aa <- iupac.aa |> addregion2string(region=region1)
#' iupac.aa |> region()
#' @export region
#' @author Kristian K Ullrich

region <- function(seq){
    stopifnot("Error: input needs to be a DNAStringSet or AAStringSet"=
        methods::is(seq, "AAStringSet") | methods::is(seq, "DNAStringSet"))
    return(methods::slot(seq, "metadata")$region)
}

data(iupac)

test_that("region() outputs IRanges object", {
    region1 <- IRanges::IRanges(start=c(1,41), end=c(20,50))
    iupac <- iupac |> addregion2string(region=region1)
    expect_true(region(iupac)@width[1] == 20)
})

#' @title subString
#' @name subString
#' @description This function gets a subsequence from a \code{DNAString},
#' \code{RNAString}, \code{AAString}, \code{BString}, \code{DNAStringSet},
#' \code{RNAStringSet}, \code{AAStringSet}, \code{BStringSet}
#' object from the \code{Biostrings} package.
#' @return subsequence of an \code{Biostrings} object
#' @importFrom Biostrings DNAString RNAString AAString BString DNAStringSet
#' RNAStringSet AAStringSet BStringSet
#' @param x \code{DNAStringSet}, \code{RNAString}, \code{AAString},
#' \code{BString}, \code{DNAStringSet}, \code{RNAStringSet},
#' \code{AAStringSet}, \code{BStringSet} [mandatory]
#' @param s start vector [mandatory]
#' @param e end vector [mandatory]
#' @examples
#' ## define two cds sequences
#' cds1 <- Biostrings::DNAString("ATGCAACATTGC")
#' cds2 <- Biostrings::DNAString("ATG---CATTGC")
#' cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
#'     Biostrings::DNAStringSet(cds2))
#' subString(cds1.cds2.aln, c(1,7), c(3,12))
#' @seealso \link[XVector]{subseq}
#' @export subString
#' @author Kristian K Ullrich

subString<-function(x, s, e){
    x.class<-class(x)[1]
    se.matrix<-cbind(s, e)
    x.len <- length(x)
    if(x.class=="DNAString"){
        myDNAString<-x
        newDNAString<-Biostrings::DNAString(paste(
            vapply(
                apply(se.matrix, 1, function(x) {
                    Biostrings::subseq(myDNAString, x[1], x[2])
                }), function(x) {
                paste0(x)
            }, rep("", 1)), sep="", collapse=""))
        names(newDNAString)<-names(x)
        return(newDNAString)
    }
    if(x.class=="RNAString"){
        myRNAString<-x
        newRNAString<-Biostrings::RNAString(paste(
            vapply(
                apply(se.matrix, 1, function(x) {
                    Biostrings::subseq(myRNAString, x[1], x[2])
                }), function(x) {
                paste0(x)
            }, rep("", 1)), sep="", collapse=""))
        names(newRNAString)<-names(x)
        return(newRNAString)
    }
    if(x.class=="AAString"){
        myAAString<-x
        newAAString<-Biostrings::AAString(paste(
            vapply(
                apply(se.matrix, 1, function(x) {
                    Biostrings::subseq(myAAString, x[1], x[2])
                }), function(x) {
                paste0(x)
            }, rep("", 1)), sep="", collapse=""))
        names(newAAString)<-names(x)
        return(newAAString)
    }
    if(x.class=="BString"){
        myBString<-x
        newBString<-Biostrings::BString(paste(
            vapply(
                apply(se.matrix, 1, function(x) {
                    Biostrings::subseq(myBString, x[1], x[2])
                }), function(x) {
                paste0(x)
            }, rep("", 1)), sep="", collapse=""))
        names(newBString)<-names(x)
        return(newBString)
    }
    if(x.class=="DNAStringSet"){
        myDNAStringSet<-x
        if(length(myDNAStringSet)>1){
            newDNAStringSet<-Biostrings::DNAStringSet(
                apply(
                    vapply(
                        apply(se.matrix, 1, function(x) {
                            Biostrings::subseq(myDNAStringSet, x[1], x[2])
                        }), function(x) {
                        paste0(x)
                    }, rep("", length(myDNAStringSet))), 1, function(x) {
                    paste(x, sep="", collapse="")
                }))
            names(newDNAStringSet)<-names(x)
            return(newDNAStringSet)
        }
        if(length(myDNAStringSet)==1){
            newDNAStringSet<-Biostrings::DNAStringSet(paste(
                vapply(
                    apply(se.matrix, 1, function(x) {
                        Biostrings::subseq(myDNAStringSet, x[1], x[2])
                    }), function(x) {
                    paste0(x)
                }, rep("", length(myDNAStringSet))), sep="", collapse=""))
            names(newDNAStringSet) <- names(x)
            return(newDNAStringSet)
        }
    }
    if(x.class=="RNAStringSet"){
        myRNAStringSet<-x
        if(length(myRNAStringSet)>1){
            newRNAStringSet<-Biostrings::RNAStringSet(
                apply(
                    vapply(
                        apply(se.matrix, 1, function(x) {
                            Biostrings::subseq(myRNAStringSet, x[1], x[2])
                        }), function(x) {paste0(x)
                    }, rep("", length(myRNAStringSet))), 1, function(x) {
                    paste(x, sep="", collapse="")
                }))
            names(newRNAStringSet)<-names(x)
            return(newRNAStringSet)
        }
        if(length(myRNAStringSet)==1){
            newRNAStringSet<-Biostrings::RNAStringSet(paste(
                vapply(
                    apply(se.matrix, 1, function(x) {
                        Biostrings::subseq(myRNAStringSet, x[1], x[2])
                    }), function(x) {
                    paste0(x)
                }, rep("", length(myRNAStringSet))), sep="", collapse=""))
            names(newRNAStringSet)<-names(x)
            return(newRNAStringSet)
        }
    }
    if(x.class=="AAStringSet"){
        myAAStringSet<-x
        if(length(myAAStringSet)>1){
            newAAStringSet<-Biostrings::AAStringSet(
                apply(
                    vapply(
                        apply(se.matrix, 1, function(x) {
                            Biostrings::subseq(myAAStringSet, x[1], x[2])
                        }), function(x) {
                        paste0(x)
                    }, rep("", length(myAAStringSet))), 1, function(x) {
                    paste(x, sep="", collapse="")
                }))
            names(newAAStringSet)<-names(x)
            return(newAAStringSet)
        }
        if(length(myAAStringSet)==1){
            newAAStringSet<-Biostrings::AAStringSet(paste(
                vapply(
                    apply(se.matrix, 1, function(x) {
                        Biostrings::subseq(myAAStringSet, x[1], x[2])
                    }), function(x) {
                    paste0(x)
                }, rep("", length(myAAStringSet))), sep="", collapse=""))
            names(newAAStringSet)<-names(x)
            return(newAAStringSet)
        }
    }
    if(x.class=="BStringSet"){
        myBStringSet<-x
        if(length(myBStringSet)>1){
            newBStringSet<-Biostrings::BStringSet(
                apply(
                    vapply(
                        apply(se.matrix, 1, function(x) {
                            Biostrings::subseq(myBStringSet, x[1], x[2])
                        }), function(x) {
                        paste0(x)
                    }, rep("", length(myBStringSet))), 1, function(x) {
                    paste(x, sep="", collapse="")
                }))
            names(newBStringSet)<-names(x)
            return(newBStringSet)
        }
        if(length(myBStringSet)==1){
            newBStringSet<-Biostrings::BStringSet(paste(
                vapply(
                    apply(se.matrix, 1, function(x) {
                        Biostrings::subseq(myBStringSet, x[1], x[2])
                        }), function(x) {
                        paste0(x)
                }, rep("", length(myBStringSet))), sep="", collapse=""))
            names(newBStringSet)<-names(x)
            return(newBStringSet)
        }
    }
}

data(hiv)

test_that("subString()", {
    expect_true(as.character(subString(hiv,c(1,2),c(3,4))[1]) == "GTATAG")
    expect_true(as.character(subString(Biostrings::DNAStringSet("ATG"),1,2))
    == "AT")
    expect_true(as.character(subString(Biostrings::DNAString("ATG"),1,2))
    == "AT")
        expect_true(as.character(subString(Biostrings::BStringSet("ATG"),1,2))
    == "AT")
    expect_true(as.character(subString(Biostrings::BString("ATG"),1,2))
    == "AT")
        expect_true(as.character(subString(Biostrings::RNAStringSet("AUG"),1,2))
    == "AU")
    expect_true(as.character(subString(Biostrings::RNAString("AUG"),1,2))
    == "AU")
        expect_true(as.character(subString(Biostrings::AAStringSet("MGP"),1,2))
    == "MG")
    expect_true(as.character(subString(Biostrings::AAString("MGP"),1,2))
    == "MG")
})

#' @title aa2selfscore
#' @name aa2selfscore
#' @description This function return the selfscore from an \code{AAStringSet}.
#' @param aa \code{AAStringSet} [mandatory]
#' @param scorematrix score matrix to use [default: BLOSUM62]
#' @return \code{data.frame}
#' @importFrom methods is slot
#' @importFrom Biostrings AAString AAStringSet
#' @importFrom stringr word
#' @importFrom utils data
#' @seealso \code{\link[Biostrings]{XStringSet-class}},
#' \code{\link[pwalign]{substitution_matrices}}
#' @examples
#' data(woodmouse, package="ape")
#' #cds2aa(dnabin2dnastring(woodmouse), shorten=TRUE,
#' #genetic.code=Biostrings::getGeneticCode("2"))
#' woodmouse |> dnabin2dnastring() |> cds2aa(shorten=TRUE,
#' genetic.code=Biostrings::getGeneticCode("2")) |> aa2selfscore()
#' @export aa2selfscore
#' @author Kristian K Ullrich

aa2selfscore <- function(aa, scorematrix="BLOSUM62"){
    stopifnot("Error: input needs to be a AAStringSet"=
        methods::is(aa, "AAStringSet"))
    stopifnot("Error: score needs to be BLOSUM45, BLOSUM50, BLOSUM62,
        BLOSUM80, BLOSUM100, PAM30, PAM40, PAM70,
        PAM120 or PAM250"= scorematrix %in%
        c("BLOSUM45", "BLOSUM50", "BLOSUM62", "BLOSUM80", "BLOSUM100", "PAM30",
        "PAM40", "PAM70", "PAM120", "PAM250"))
    if(!is.null(names(aa))){
        names(aa) <- stringr::word(names(aa), 1)
    }
    BLOSUM45<-NULL
    BLOSUM50<-NULL
    BLOSUM62<-NULL
    BLOSUM80<-NULL
    BLOSUM100<-NULL
    PAM30<-NULL
    PAM40<-NULL
    PAM70<-NULL
    PAM120<-NULL
    PAM250<-NULL
    if(scorematrix == "BLOSUM45"){utils::data(BLOSUM45, package="pwalign",
        envir=environment())
        score.matrix <- BLOSUM45}
    if(scorematrix == "BLOSUM50"){utils::data(BLOSUM50, package="pwalign",
        envir=environment())
        score.matrix <- BLOSUM50}
    if(scorematrix == "BLOSUM62"){utils::data(BLOSUM62, package="pwalign",
        envir=environment())
        score.matrix <- BLOSUM62}
    if(scorematrix == "BLOSUM80"){utils::data(BLOSUM80, package="pwalign",
        envir=environment())
        score.matrix <- BLOSUM80}
    if(scorematrix == "BLOSUM100"){utils::data(BLOSUM100, package="pwalign",
        envir=environment())
        score.matrix <- BLOSUM100}
    if(scorematrix == "PAM30"){utils::data(PAM30, package="pwalign",
        envir=environment())
        score.matrix <- PAM30}
    if(scorematrix == "PAM40"){utils::data(PAM40, package="pwalign",
        envir=environment())
        score.matrix <- PAM40}
    if(scorematrix == "PAM70"){utils::data(PAM70, package="pwalign",
        envir=environment())
        score.matrix <- PAM70}
    if(scorematrix == "PAM120"){utils::data(PAM120, package="pwalign",
        envir=environment())
        score.matrix <- PAM120}
    if(scorematrix == "PAM250"){utils::data(PAM250, package="pwalign",
        envir=environment())
        score.matrix <- PAM250}
    aa_selfscore_list <- lapply(aa, function(x) {
        sum(unlist(lapply(strsplit(as.character(x), ""),
        function(y){diag(score.matrix)[y]})))})
    aa_selfscore <- data.frame(name=names(aa_selfscore_list),
        score=unlist(aa_selfscore_list))
    return(aa_selfscore)
}

data(woodmouse, package="ape")

test_that("aa2selfscore() outputs data.frame", {
    aa <- woodmouse |> dnabin2dnastring() |> cds2aa(shorten=TRUE,
        genetic.code=Biostrings::getGeneticCode("2"))
    expect_true(class(aa) == "AAStringSet")
    expect_true((aa |> aa2selfscore(scorematrix = "BLOSUM45"))[1, 2] == 2043)
    expect_true((aa |> aa2selfscore(scorematrix = "BLOSUM50"))[1, 2] == 2156)
    expect_true((aa |> aa2selfscore(scorematrix = "BLOSUM62"))[1, 2] == 1690)
    expect_true((aa |> aa2selfscore(scorematrix = "BLOSUM80"))[1, 2] == 1809)
    expect_true((aa |> aa2selfscore(scorematrix = "BLOSUM100"))[1, 2] == 3132)
    expect_true((aa |> aa2selfscore(scorematrix = "PAM30"))[1, 2] == 2484)
    expect_true((aa |> aa2selfscore(scorematrix = "PAM40"))[1, 2] == 2438)
    expect_true((aa |> aa2selfscore(scorematrix = "PAM70"))[1, 2] == 2178)
    expect_true((aa |> aa2selfscore(scorematrix = "PAM120"))[1, 2] == 1788)
    expect_true((aa |> aa2selfscore(scorematrix = "PAM250"))[1, 2] == 1747)
})

#' @title GENETIC_CODE_TCAG
#' @name GENETIC_CODE_TCAG
#' @description \code{GENETIC_CODE} from \code{Biostrings} extended by codon
#' number and number of syn sites.
#' @param codon \code{codon} [mandatory]
#' @return An object of class \code{numeric}
#' @importFrom Biostrings GENETIC_CODE
#' @importFrom stats setNames
#' @seealso \code{\link[Biostrings]{GENETIC_CODE}}
#' @examples
#' GENETIC_CODE_TCAG
#' @export GENETIC_CODE_TCAG
#' @author Kristian K Ullrich

codon2number <- function(codon){
    baseNumber <- setNames(c(0, 1, 2, 3), c("T", "C", "A", "G"))
    left <- baseNumber[str_sub(codon, 1, 1)]
    mid <- baseNumber[str_sub(codon, 2, 2)]
    right <- baseNumber[str_sub(codon, 3, 3)]
    codonnumber <- (mid * 16) + (left * 4) + (right) + 1
    names(codonnumber) <- codon
    return(codonnumber)
}

TMP_GENETIC_CODE <- Biostrings::GENETIC_CODE
TMP_CODON_NUMBER <- unlist(lapply(names(TMP_GENETIC_CODE), codon2number))
TMP_GENETIC_CODE <- TMP_GENETIC_CODE[order(TMP_CODON_NUMBER)]
TMP_CODON_NUMBER <- TMP_CODON_NUMBER[order(TMP_CODON_NUMBER)]
TMP_SYN_SITES <- setNames(c(1,1,2,2,
                            3,3,4,4,
                            2,2,2,0,
                            3,3,3,3,
                            3,3,3,3,
                            3,3,3,3,
                            3,3,3,3,
                            3,3,3,3,
                            1,1,2,1,
                            1,1,1,1,
                            1,1,1,1,
                            1,1,1,1,
                            1,1,1,0,
                            3,3,4,4,
                            1,1,2,2,
                            3,3,3,3), names(TMP_CODON_NUMBER))

GENETIC_CODE_TCAG <- data.frame(CODON = names(TMP_GENETIC_CODE),
                                GENETIC_CODE = TMP_GENETIC_CODE,
                                CODON_NUMBER = TMP_CODON_NUMBER,
                                SYN_SITES = TMP_SYN_SITES)

#' @title makePostalignedSeqs
#' @name makePostalignedSeqs
#' @description This function is a fork from an internal function from
#' \code{Biostrings}
#' @param x x
#' @return get internal function makePostalignedSeqs
#' @seealso \code{\link[pwalign]{pairwiseAlignment}},
#' \link[MSA2dist]{cds2codonaln}
#' @examples
#' ## define two cds sequences
#' cds1 <- Biostrings::DNAString("ATGCAACATTGC")
#' cds2 <- Biostrings::DNAString("ATGCATTGC")
#' makePostalignedSeqs(pwalign::pairwiseAlignment(
#'     cds2aa(Biostrings::DNAStringSet(cds1)),
#'     cds2aa(Biostrings::DNAStringSet(cds2))))
#' @export makePostalignedSeqs
#' @author Kristian K Ullrich

makePostalignedSeqs <- get('.makePostalignedSeqs',
    envir=asNamespace('pwalign'), inherits=FALSE)

#' @title aastring2aln
#' @name aastring2aln
#' @description This function converts a \code{AAStringSet} into an
#' \code{seqinr} \code{alignment}.
#' @param aa \code{AAStringSet} [mandatory]
#' @return An object of class \code{alignment} which is a list with the
#' following components:\cr
#' \code{nb} the number of aligned sequences\cr
#' \code{nam} a vector of strings containing the names of the aligned
#' sequences\cr
#' \code{seq} a vector of strings containing the aligned sequences\cr
#' \code{com} a vector of strings containing the commentaries for each sequence
#' or \code{NA} if there are no comments
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom seqinr as.alignment
#' @seealso \code{\link[seqinr]{as.alignment}}
#' @examples
#' ## define two cds sequences
#' cds1 <- Biostrings::DNAString("ATGCAACATTGC")
#' cds2 <- Biostrings::DNAString("ATG---CATTGC")
#' cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
#'     Biostrings::DNAStringSet(cds2))
#' #aastring2aln(cds2aa(cds1.cds2.aln))
#' cds1.cds2.aln |> cds2aa() |> aastring2aln()
#' @export aastring2aln
#' @author Kristian K Ullrich

aastring2aln <- function(aa){
    stopifnot("Error: input needs to be an AAStringSet"=
        methods::is(aa, "AAStringSet"))
    alignment.nb <- length(aa)
    alignment.nam <- names(aa)
    alignment.seq <- tolower(as.character(aa))
    names(alignment.seq) <- NULL
    alignment.com <- NA
    alignment <- list(alignment.nb, alignment.nam, alignment.seq, alignment.com)
    names(alignment) <- c("nb", "nam", "seq", "com")
    attr(alignment, "class") <- "alignment"
    return(alignment)
}

#' @title cds2codonaln
#' @name cds2codonaln
#' @description This function takes two single sequence \code{DNAString}'s or
#' two single sequence \code{DNAStringSet}'s, converts them into aa, calculates
#' a global alignment and converts this alignment back into a codon alignment.
#' @param cds1 single sequence \code{DNAStringSet} or \code{DNAString}
#' [mandatory]
#' @param cds2 single sequence \code{DNAStringSet} or \code{DNAString}
#' [mandatory]
#' @param type type of alignment (see
#' \code{\link[pwalign]{pairwiseAlignment}}) [default: global]
#' @param substitutionMatrix substitution matrix representing the fixed
#' substitution scores for an alignment (see
#' \code{\link[pwalign]{pairwiseAlignment}}) [default: BLOSUM62]
#' @param gapOpening the cost for opening a gap in the alignment (see
#' \code{\link[pwalign]{pairwiseAlignment}}) [default: 10]
#' @param gapExtension the incremental cost incurred along the length of the
#' gap in the alignment (see \code{\link[pwalign]{pairwiseAlignment}})
#' [default: 0.5]
#' @param remove.gaps specify if gaps in the codon alignment should be removed
#' [default: FALSE]
#' @param ... other cds2aa parameters
#' @return codon alignment as \code{DNAStringSet}
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom pwalign pairwiseAlignment
#' @importFrom methods is slot
#' @references Pagès, H et al. (2014) Biostrings: Efficient manipulation of
#' biological strings. \emph{R package version}, \bold{2(0)}.
#' @seealso \code{\link[pwalign]{pairwiseAlignment}}
#' @examples
#' ## define two cds sequences
#' cds1 <- Biostrings::DNAString("ATGCAACATTGC")
#' cds2 <- Biostrings::DNAString("ATGCATTGC")
#' cds2codonaln(cds1, cds2)
#' @export cds2codonaln
#' @author Kristian K Ullrich

cds2codonaln <- function(cds1, cds2, type="global",
    substitutionMatrix="BLOSUM62", gapOpening=10, gapExtension=0.5,
    remove.gaps=FALSE, ...){
    stopifnot("Error: cds1 needs to be either DNAString or DNAStringSet"=
        {methods::is(cds1, "DNAString") || methods::is(cds1, "DNAStringSet")})
    stopifnot("Error: cds2 needs to be either DNAString or DNAStringSet"=
        {methods::is(cds2, "DNAString") || methods::is(cds2, "DNAStringSet")})
    if(methods::is(cds1, "DNAString")){
        x.aa <- MSA2dist::cds2aa(Biostrings::DNAStringSet(cds1), ...)[[1]]
        x.name <- "cds1"
        cds1 <- MSA2dist::cds2aa(Biostrings::DNAStringSet(cds1),
            return.cds=TRUE, ...)[[1]]
    }
    if(methods::is(cds2, "DNAString")){
        y.aa <- MSA2dist::cds2aa(Biostrings::DNAStringSet(cds2), ...)[[1]]
        y.name <- "cds2"
        cds2 <- MSA2dist::cds2aa(Biostrings::DNAStringSet(cds2),
            return.cds=TRUE, ...)[[1]]
    }
    if(methods::is(cds1, "DNAStringSet")){
        stopifnot("Error: cds1 needs to only contain one sequence"=
            length(cds1) == 1)
        x.aa <- MSA2dist::cds2aa(cds1, ...)[[1]]
        x.name <- names(cds1)
        cds1 <- MSA2dist::cds2aa(cds1, return.cds=TRUE, ...)[[1]]
    }
    if(methods::is(cds2, "DNAStringSet")){
        stopifnot("Error: cds2 needs to only contain one sequence"=
            length(cds2) == 1)
        y.aa <- MSA2dist::cds2aa(cds2, ...)[[1]]
        y.name <- names(cds2)
        cds2 <- MSA2dist::cds2aa(cds2, return.cds=TRUE, ...)[[1]]
    }
    xy.aln <- makePostalignedSeqs(pwalign::pairwiseAlignment(x.aa, y.aa,
        type=type, substitutionMatrix=substitutionMatrix, gapOpening=gapOpening,
        gapExtension=gapExtension))[[1L]]
    names(xy.aln) <- c(x.name, y.name)
    if(type=="local"){
        xy.aln.x <- gsub("\\*", "X", gsub("-", "", xy.aln[[1]]))
        cds1_local_pos <- gregexpr(xy.aln.x, gsub("\\*", "X", x.aa))
        xy.aln.y <- gsub("\\*", "X", gsub("-", "", xy.aln[[2]]))
        cds2_local_pos <- gregexpr(xy.aln.y, gsub("\\*", "X", y.aa))
        cds1_local <- Biostrings::subseq(cds1,
            (cds1_local_pos[[1]][1]*3)-2,
            (cds1_local_pos[[1]][1]+nchar(xy.aln.x)-1)*3)
        cds2_local <- Biostrings::subseq(cds2,
            (cds2_local_pos[[1]][1]*3)-2,
            (cds2_local_pos[[1]][1]+nchar(xy.aln.y)-1)*3)
        xy.cds <- setNames(Biostrings::DNAStringSet(list(cds1_local,
            cds2_local)), c(x.name, y.name))
    } else{
        xy.cds <- setNames(Biostrings::DNAStringSet(list(cds1, cds2)),
            c(x.name, y.name))
    }
    xy.cds.aln <- MSA2dist::pal2nal(xy.aln, xy.cds, remove.gaps=remove.gaps)
    return(xy.cds.aln)
}

data(hiv)

test_that("cds2codonaln() outputs DNAStringSet", {
    cds1 <- Biostrings::DNAString("ATGCAACATTGC")
    cds2 <- Biostrings::DNAString("ATGCATTGC")
    cds1.cds2.aln <- cds2codonaln(cds1, cds2)
    expect_true(as.character(cds1.cds2.aln[1]) == "ATGCAACATTGC")
    expect_true(as.character(cds1.cds2.aln[2]) == "ATG---CATTGC")
    cds1.cds2.aln.rem <- cds2codonaln(cds1, cds2, remove.gaps=TRUE)
    expect_true(as.character(cds1.cds2.aln.rem[1]) == "ATGCATTGC")
    expect_true(as.character(cds1.cds2.aln.rem[2]) == "ATGCATTGC")
    cds1.cds2.aln.local <- cds2codonaln(cds1, cds2, type="local")
    expect_true(as.character(cds1.cds2.aln.local[1]) == "CAACATTGC")
    expect_true(as.character(cds1.cds2.aln.local[2]) == "ATGCATTGC")
})

#' @title string2region
#' @name string2region
#' @description This function subsets a \code{DNAStringSet} or an
#' \code{AAStringSet} by a \code{mask} and \code{region} given one or both
#' options as \code{IRanges}.
#' @param seq \code{DNAStringSet} or \code{AAStringSet} [mandatory]
#' @param mask \code{IRanges} object indicating masked sites [default: NULL]
#' @param region \code{IRanges} object indicating region to use for dist
#' calculation (by default all sites are used) [default: NULL]
#' @param add indicate if mask and region should be added to \code{metadata}
#' [default: TRUE]
#' @return A \code{list} object with the following components:\cr
#' \code{DNAStringSet} or \code{AAStringSet}\cr
#' \code{regionUsed}\cr
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom IRanges IRanges IRangesList reduce start end findOverlaps
#' disjoin overlapsRanges
#' @seealso \code{\link[MSA2dist]{dnastring2dist}}
#' @examples
#' ## load example sequence data
#' data("hiv", package="MSA2dist")
#' ## create mask
#' mask1 <- IRanges::IRanges(start=c(11,41,71), end=c(20,50,80))
#' ## use mask
#' hiv.region <- hiv |> cds2aa() |> string2region(mask=mask1)
#' #(hiv.region |> slot("metadata"))$regionUsed
#' hiv.region |> regionused()
#' ## use region
#' region1 <- IRanges::IRanges(start=c(1,75), end=c(45,85))
#' hiv.region <- hiv |> cds2aa() |> string2region(region=region1)
#' #(hiv.region |> slot("metadata"))$regionUsed
#' hiv.region |> regionused()
#' ## use mask and region
#' hiv.region <- hiv |> cds2aa() |> string2region(mask=mask1, region=region1)
#' #(hiv.region |> slot("metadata"))$regionUsed
#' hiv.region |> regionused()
#' @export string2region
#' @author Kristian K Ullrich

string2region <- function(seq,
    mask = NULL,
    region = NULL,
    add = TRUE){
    stopifnot("Error: input needs to be a DNAStringSet or AAStringSet"=
        methods::is(seq, "AAStringSet") | methods::is(seq, "DNAStringSet"))
    region.seq <- IRanges::IRanges(start=1, end=unique(width(seq)))
    if(!is.null(mask)){
        mask <- mask[!(IRanges::start(mask)>unique(width(seq)))]
        IRanges::start(mask[IRanges::start(mask)<1]) <- 1
        IRanges::end(mask[IRanges::end(mask)>unique(width(seq))]) <-
            unique(width(seq))
        mask <- IRanges::reduce(mask)
        region.seq.mask.overlaps <- IRanges::findOverlaps(region.seq,
            IRanges::reduce(mask))
        if(length(region.seq.mask.overlaps) != 0){
            region.seq.mask.disjoin <- IRanges::disjoin(c(region.seq, mask))
            region.seq <- IRanges::reduce(region.seq.mask.disjoin[
                -(IRanges::findOverlaps(region.seq.mask.disjoin, mask)@from)])
        }
    }
    if(!is.null(region)){
        region <- region[!(IRanges::start(region)>unique(width(seq)))]
        IRanges::start(region[IRanges::start(region)<1]) <- 1
        IRanges::end(region[IRanges::end(region)>unique(width(seq))])<-
            unique(width(seq))
        region <- IRanges::reduce(region)
        region.seq <- IRanges::reduce(IRanges::overlapsRanges(
            region.seq, region))
        stopifnot("Error: specified region already masked or outside"=
            length(region.seq) != 0)
    }
    if(!is.null(mask) || !is.null(region)){
        seq.region <- MSA2dist::subString(
            seq, IRanges::start(region.seq), IRanges::end(region.seq))
        methods::slot(seq.region, "metadata") <- methods::slot(seq, "metadata")
        methods::slot(seq.region, "metadata")$regionUsed <- region.seq
        if(add){
            if(!is.null(mask)){
                MSA2dist::addmask2string(seq=seq.region, mask=mask,
                    append=FALSE)
            }
            if(!is.null(region)){
                MSA2dist::addregion2string(seq=seq.region, region=region,
                    append=FALSE)
            }
        }
    } else{seq.region <- seq
        methods::slot(seq.region, "metadata")$regionUsed <- region.seq
        if(!is.null(mask)){
            MSA2dist::addmask2string(seq=seq.region, mask=mask, append=FALSE)
        }
        if(!is.null(region)){
            MSA2dist::addregion2string(seq=seq.region, region=region,
                append=FALSE)
        }
    }
    return(seq.region)
}

data(iupac)

test_that("string2region() outputs DNAStringSet", {
    mask1 <- IRanges::IRanges(start=c(11,41,71), end=c(20,50,80))
    hiv.region <- hiv |> cds2aa() |> string2region(mask=mask1)
    expect_true(regionused(hiv.region)@start[1] == 1)
})

#' @title aabin2aastring
#' @name aabin2aastring
#' @description This function converts an \code{ape} \code{AAbin} into
#' \code{AAStringSet}.
#' @param aabin \code{ape} \code{AAbin} [mandatory]
#' @return An object of class \code{AAStringSet}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom seqinr as.alignment
#' @importFrom ape as.DNAbin.alignment
#' @seealso \code{\link[seqinr]{as.alignment}}
#' \code{\link[ape]{as.DNAbin.alignment}}
#' \code{\link[Biostrings]{AAStringSet}}
#' @examples
#' data(woodmouse, package="ape")
#' ## convert into AAStringSet
#' #aabin2aastring(ape::trans(woodmouse, 2))
#' ape::trans(woodmouse, 2) |> aabin2aastring()
#' @export aabin2aastring
#' @author Kristian K Ullrich

aabin2aastring <- function(aabin){
    stopifnot("Error: input needs to be an AAbin"=
        methods::is(aabin, "AAbin"))
    if(!is.null(dim(aabin))){
        aa <- setNames(Biostrings::AAStringSet(unlist(apply(
            as.character(aabin), 1, function(x) paste0(x, collapse = "")))),
            labels(aabin))
    }
    if(is.null(dim(aabin))){
        aa <- setNames(Biostrings::AAStringSet(unlist(lapply(
            as.character(aabin), function(x) paste0(x, collapse = "")))),
            labels(aabin))
    }
    return(aa)
}

#' @title globalDeletionAA
#' @name globalDeletionAA
#' @description This function returns an \code{AAStringSet} reduced by all
#' sites containing any gaps ("-", "+", ".") or missing ("X") sites.
#' @return \code{AAStringSet}
#' @importFrom methods is slot
#' @importFrom Biostrings consensusMatrix
#' @param aa \code{AAStringSet} [mandatory]
#' @examples
#' ## define two cds sequences
#' cds1 <- Biostrings::DNAString("ATGCAACATTGC")
#' cds2 <- Biostrings::DNAString("ATG---CATTGC")
#' cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
#'     Biostrings::DNAStringSet(cds2))
#' #globalDeletionAA(cds2aa(cds1.cds2.aln))
#' cds1.cds2.aln |> cds2aa() |> globalDeletionAA()
#' @export globalDeletionAA
#' @author Kristian K Ullrich

globalDeletionAA<-function(aa){
    stopifnot("Error: input needs to be an AAStringSet"=
        methods::is(aa, "AAStringSet"))
    cM <- Biostrings::consensusMatrix(aa)
    globalDeletionSites <- which(apply(cM, 2, function(x) {
        sum(x[c(26, 28:30)])} >= 1))
    if(length(globalDeletionSites) == 0){
        return(aa)
    }
    return(MSA2dist::aabin2aastring(
        MSA2dist::aastring2aabin(aa)[, -(globalDeletionSites)]))
}

test_that("globalDeletionAA()", {
    cds1 <- Biostrings::DNAString("ATGCAACATTGC")
    cds2 <- Biostrings::DNAString("ATG---CATTGC")
    cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
        Biostrings::DNAStringSet(cds2))
    aa.aln <- cds1.cds2.aln |> cds2aa()
    expect_true(as.character(globalDeletionAA(aa.aln)[1]) == "MHC")
})

#' @title compareCodons
#' @name compareCodons
#' @description This function compares two codons and returns the number of syn
#'and non-syn sites according to \emph{Nei and Gojobori (1986)}.
#' @param codA \code{codon} A [mandatory]
#' @param codB \code{codon} B [mandatory]
#' @return vector of syn and non-syn sites
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom seqinr kaks
#' @seealso \code{\link[seqinr]{kaks}}
#' @references Nei and Gojobori. (1986) Simple methods for estimating the
#' numbers of synonymous and nonsynonymous nucleotide substitutions.
#' \emph{Mol. Biol. Evol.}, \bold{3(5)}, 418-426.
#' @references Ganeshan et al. (1997) Human immunodeficiency virus type 1
#' genetic evolution in children with different rates of development of disease.
#' \emph{J. Virology.} \bold{71(1)}, 663-677.
#' @references Yang et al. (2000) Codon-substitution models for heterogeneous
#' selection pressure at amino acid sites. \emph{Genetics.}
#' \bold{155(1)}, 431-449.
#' @examples
#' compareCodons("AAA","TTA")
#' compareCodons("AAA","TAT")
#' compareCodons("AAA","ATT")
#' compareCodons("AAA","TTT")
#' ## load example sequence data
#' data("hiv", package="MSA2dist")
#' compareCodons(dnastring2codonmat(hiv)[1,1], dnastring2codonmat(hiv)[1,2])
#' @export compareCodons
#' @author Kristian K Ullrich

compareCodons <- function(codA, codB){
    type <- "syn"
    if(codA == codB){
        return(c(0, 0, 0))
    }
    codAsplit <- strsplit(codA, "")[[1]]
    codBsplit <- strsplit(codB, "")[[1]]
    if("N" %in% codAsplit | "N" %in% codBsplit){
        type <- "Ns"
        return(c(NA, NA, NA))
    }
    if("-" %in% codAsplit | "-" %in% codBsplit){
        type <- "indel"
        return(c(0, 0, 1))
    }
    if(MSA2dist::GENETIC_CODE_TCAG[codA, 2] !=
        MSA2dist::GENETIC_CODE_TCAG[codB, 2]){
        type <- "nonsyn"
    }
    codAcodB_diff_idx <- which(codAsplit != codBsplit)
    if(length(codAcodB_diff_idx) == 1){
        if(type == "syn"){
            return(c(1, 0, 0))
        }
        if(type == "nonsyn"){
            return(c(0, 1, 0))
        }
    }
    if(length(codAcodB_diff_idx) == 2){
        ## AAA (Lys) -> TTA (Leu)
        ## AGC (Ser) -> TCC (Ser)
        if(all(codAcodB_diff_idx == c(1, 2))){
            codC <- paste0(codBsplit[1], codAsplit[2], codAsplit[3])
            codD <- paste0(codAsplit[1], codBsplit[2], codAsplit[3])
        }
        ## AAA -> TAT
        if(all(codAcodB_diff_idx == c(1, 3))){
            codC <- paste0(codBsplit[1], codAsplit[2], codAsplit[3])
            codD <- paste0(codAsplit[1], codAsplit[2], codBsplit[3])
        }
        ## AAA -> ATT
        if(all(codAcodB_diff_idx == c(2, 3))){
            codC <- paste0(codAsplit[1], codBsplit[2], codAsplit[3])
            codD <- paste0(codAsplit[1], codAsplit[2], codBsplit[3])
        }
        tmp_syn <- 0
        if(MSA2dist::GENETIC_CODE_TCAG[codA, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codC, 2]){
            tmp_syn <- tmp_syn + 1
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codA, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codD, 2]){
            tmp_syn <- tmp_syn + 1
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codB, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codC, 2]){
            tmp_syn <- tmp_syn + 1
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codB, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codD, 2]){
            tmp_syn <- tmp_syn + 1
        }
        tmp_syn <- tmp_syn / 2
        return(c(tmp_syn, 2 - tmp_syn, 0))
    }
    if(length(codAcodB_diff_idx) == 3){
        ## AAA -> TTT
        codC <- paste0(codBsplit[1], codAsplit[2], codAsplit[3])
        codD <- paste0(codAsplit[1], codBsplit[2], codAsplit[3])
        codE <- paste0(codAsplit[1], codAsplit[2], codBsplit[3])
        codF <- paste0(codBsplit[1], codBsplit[2], codAsplit[3])
        codG <- paste0(codBsplit[1], codAsplit[2], codBsplit[3])
        codH <- paste0(codAsplit[1], codBsplit[2], codBsplit[3])
        tmp_syn <- 0
        if(MSA2dist::GENETIC_CODE_TCAG[codA, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codC, 2]){
            tmp_syn <- tmp_syn + 1
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codA, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codD, 2]){
            tmp_syn <- tmp_syn + 1
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codA, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codE, 2]){
            tmp_syn <- tmp_syn + 1
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codB, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codF, 2]){
            tmp_syn <- tmp_syn + 1
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codB, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codG, 2]){
            tmp_syn <- tmp_syn + 1
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codB, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codH, 2]){
            tmp_syn <- tmp_syn + 1
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codC, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codF, 2]){
            tmp_syn <- tmp_syn + 0.5
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codC, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codG, 2]){
            tmp_syn <- tmp_syn + 0.5
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codD, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codF, 2]){
            tmp_syn <- tmp_syn + 0.5
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codD, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codH, 2]){
            tmp_syn <- tmp_syn + 0.5
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codE, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codG, 2]){
            tmp_syn <- tmp_syn + 0.5
        }
        if(MSA2dist::GENETIC_CODE_TCAG[codE, 2] ==
            MSA2dist::GENETIC_CODE_TCAG[codH, 2]){
            tmp_syn <- tmp_syn + 0.5
        }
        tmp_syn <- tmp_syn / 3
        return(c(tmp_syn, 3 - tmp_syn, 0))
    }
}

data(hiv)

test_that("compareCodons()", {
    idx <- list(c(1,2), c(1,3))
    h <- hiv[1:3] |> indices2kaks(idx, model="Li")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="Li",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="NG86")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="NG86",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="NG")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="NG",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="LWL")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="LWL",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="LPB")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="LPB",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="MLWL")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="MLWL",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="MLPB")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="MLPB",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GY")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GY",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="YN")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="YN",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="MYN")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="MYN",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="MS")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="MS",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="MA")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="MA",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GNG")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GNG",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GLWL")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GLWL",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GLPB")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GLPB",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GMLWL")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GMLWL",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GMLPB")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GMLPB",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GYN")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GYN",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GMYN")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> indices2kaks(idx, model="GMYN",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
})

data(hiv)

test_that("compareCodons()", {
    h <- hiv[1:3] |> dnastring2kaks(model="Li")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="Li",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="NG86")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="NG86",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="NG")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="NG",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="LWL")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="LWL",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="LPB")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="LPB",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="MLWL")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="MLWL",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="MLPB")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="MLPB",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GY")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GY",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="YN")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="YN",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="MYN")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="MYN",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="MS")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="MS",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="MA")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="MA",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GNG")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GNG",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GLWL")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GLWL",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GLPB")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GLPB",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GMLWL")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GMLWL",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GMLPB")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GMLPB",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GYN")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GYN",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GMYN")
    expect_true(h$seq1[1] == "U68496")
    h <- hiv[1:3] |> dnastring2kaks(model="GMYN",
        isMSA=FALSE)
    expect_true(h$seq1[1] == "U68496")
})

#' @title uptriidx
#' @name uptriidx
#' @description This function returns upper tri index for usage with
#' \code{pivot_long} reduction.
#' @param n dimension of initial matrix [mandatory]
#' @param diag indicate if diag should be retained [default: FALSE]
#' @return list of positions
#' @examples
#' uptriidx(10)
#' @export uptriidx
#' @author Kristian K Ullrich

uptriidx <- function(n, diag = FALSE){
    stopifnot("Error: n needs to larger than 1"= n>1)
    if(diag == TRUE){
        tmp <- cbind(seq(1, n), seq(0, n - 1))
    }
    if(diag == FALSE){
        tmp <- cbind(seq(1, n - 1), seq(0, n - 2))
    }
    tmp <- cbind(tmp, apply(tmp, 1, function(x) x[2] * n + x[1]))
    tmp <- cbind(tmp, apply(tmp, 1, function(x) x[1] * n))
    if(diag == TRUE){
        idx <- unlist(apply(tmp, 1, function(x) seq(x[3], x[4])))
    }
    if(diag == FALSE){
        idx <- unlist(apply(tmp, 1, function(x) seq(x[3] + 1, x[4])))
    }
    return(idx)
}

#' @title popinteger
#' @name popinteger
#' @description This function shows the population integer slot from a
#' \code{DNAStringSet} or an \code{AAStringSet}
#' \code{metadata} information.\cr
#' @param seq \code{DNAStringSet} or \code{AAStringSet} [mandatory]
#' @return population integer from \code{metadata}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @seealso \code{\link[MSA2dist]{addpop2string}}
#' @examples
#' ## load example sequence data
#' data(iupac, package="MSA2dist")
#' iupac.aa <- iupac |> cds2aa(shorten = TRUE)
#' ## create poplist
#' poplist <- list(FRA = grep("Mmd.FRA", names(iupac)),
#'     GER = grep("Mmd.GER", names(iupac)),
#'     IRA = grep("Mmd.IRA", names(iupac)),
#'     AFG = grep("Mmm.AFG", names(iupac)))
#' iupac.aa <- iupac.aa |> addpop2string(poplist)
#' popinteger(iupac.aa)
#' @export popinteger
#' @author Kristian K Ullrich

popinteger <- function(seq){
    stopifnot("Error: input needs to be a DNAStringSet or AAStringSet"=
        methods::is(seq, "AAStringSet") | methods::is(seq, "DNAStringSet"))
    return(methods::slot(seq, "metadata")$pop.integer)
}

#' @title dnastring2kaks
#' @name dnastring2kaks
#' @description This function calculates Ka/Ks (pN/pS)
#' for all combinations of a \code{DNAStringSet}.
#' If the sequences in the \code{DNAStringSet} are not a multiple-sequence
#' alignment, pairwise codon alignments can be calculated on the fly.
#' Models used and implemented according to
#' \emph{Li (1993)} (using \code{seqinr}) or
#' \emph{Nei and Gojobori (1986)} (own implementation) or models from
#' \code{KaKs_Calculator2} ported to \code{MSA2dist} with \code{Rcpp}.
#' @param cds \code{DNAStringSet} coding sequence alignment [mandatory]
#' @param model specify codon model either "Li" or "NG86" or
#' one of \code{KaKs_Calculator2} model "NG", "LWL", "LPB", "MLWL", "MLPB",
#' "GY", "YN", "MYN", "MS", "MA", "GNG", "GLWL", "GLPB", "GMLWL", "GMLPB",
#' "GYN", "GMYN" [default: Li]
#' @param threads number of parallel threads [default: 1]
#' @param isMSA cds \code{DNAStringSet} represents MSA [default: TRUE]
#' @param sgc standard genetic code (for KaKs Calculator models)
#' [default: 1]
#' @param verbose verbosity (for KaKs Calculator models) [default: FALSE]
#' @param ... other codon alignment parameters
#' @return A \code{data.frame} of \code{KaKs} values
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom seqinr kaks
#' @importFrom parallel makeForkCluster stopCluster
#' @importFrom doParallel registerDoParallel
#' @importFrom foreach foreach %do% %dopar%
#' @importFrom tidyr %>% as_tibble pivot_longer
#' @importFrom dplyr slice left_join
#' @importFrom tibble column_to_rownames add_column
#' @importFrom stringr str_split
#' @seealso \code{\link[seqinr]{kaks}}
#' @references "MS/MA/GNG/GLWL/GLPB/GMLWL/GMLPB/GYN:" Wang et al. (2010)
#' KaKs_Calculator 2.0: a toolkit incorporating
#' gamma-series methods and sliding window strategies.\emph{Genomics,
#' proteomics & bioinformatics.} \bold{8(1)}, 77-80.
#' @references "Li/LWL:" Li et al. (1985) A new method for estimating synonymous
#' and nonsynonymous rates of nucleotide substitution considering the relative
#' likelihood of nucleotide and codon changes. \emph{Mol. Biol. Evol.},
#' \bold{2(2)}, 150-174.
#' @references "Li/LPB:" Li (1993). Unbiased estimation of the rates of
#' synonymous and nonsynonymous substitution. Journal of molecular evolution,
#' 36(1), pp.96-99.
#' @references "NG86/NG:" Nei and Gojobori. (1986) Simple methods for estimating
#' the numbers of synonymous and nonsynonymous nucleotide substitutions.
#' \emph{Mol. Biol. Evol.}, \bold{3(5)}, 418-426.
#' @references "LPB:" Pamilo and Bianchi. (1993) Evolution of the Zfx and Zfy
#' genes: Rates and interdependence between genes. \emph{Mol. Biol. Evol.},
#' \bold{10}, 271-281.
#' @references "MLWL/MLPB:" Tzeng et al. (2004). Comparison of three methods for
#' estimating rates of synonymous and nonsynonymous nucleotide substitutions.
#' \emph{Mol. Biol. Evol.}, \bold{21(12)}, 2290-2298.
#' @references "GY:" Goldman and Yang (1994). A codon-based model of nucleotide
#' substitution for protein-coding DNA sequences. \emph{Mol. Biol. Evol.},
#' \bold{11(5)} 725-736.
#' @references "YN:" Yang et al. (2000) Codon-substitution models for
#' heterogeneous selection pressure at amino acid sites. \emph{Genetics.}
#' \bold{155(1)}, 431-449.
#' @references "MYN:" Zhang et al. (2006). Computing Ka and Ks with a
#' consideration of unequal transitional substitutions.
#' \emph{BMC evolutionary biology}, \bold{6(1)}, 1-10.
#' @references "data(hiv):" Ganeshan et al. (1997) Human immunodeficiency virus
#' type 1 genetic evolution in children with different rates of development of
#' disease. \emph{J. Virology.} \bold{71(1)}, 663-677.
#' @references Wang et al. (2009). gamma-MYN: a new algorithm for estimating Ka
#' and Ks with consideration of variable substitution rates.
#' \emph{Biology Direct}, \bold{4(1)}, 1-18.
#' @examples
#' ## load example sequence data
#' data("hiv", package="MSA2dist")
#' #dnastring2kaks(hiv, model="Li")
#' hiv |> dnastring2kaks(model="Li")
#' #dnastring2kaks(hiv, model="NG86")
#' hiv |> dnastring2kaks(model="NG86")
#' #dnastring2kaks(hiv, model="NG86", threads=2)
#' hiv |> dnastring2kaks(model="NG86", threads=2)
#'
#' ## define three unaligned cds sequences
#' cds1 <- Biostrings::DNAString("ATGCAACATTGC")
#' cds2 <- Biostrings::DNAString("ATGCATTGC")
#' cds3 <- Biostrings::DNAString("ATGCAATGC")
#' cds_sequences <- Biostrings::DNAStringSet(list(cds1, cds2, cds3))
#' names(cds_sequences) <- c("cds1", "cds2", "cds3")
#' ## set isMSA to FALSE to automatically create pairwise codon alignments
#' #dnastring2kaks(cds_sequences, model="Li", isMSA=FALSE)
#' cds_sequences |> dnastring2kaks(model="Li", isMSA=FALSE)
#' @export dnastring2kaks
#' @author Kristian K Ullrich

dnastring2kaks <- function(cds,
    model = "Li",
    threads = 1,
    isMSA = TRUE,
    sgc = "1",
    verbose = FALSE,
    ...){
    stopifnot("Error: input needs to be a DNAStringSet"=
        methods::is(cds, "DNAStringSet"))
    local_cds2aa <- function(...,
        type,
        substitutionMatrix,
        gapOpening,
        gapExtension,
        remove.gaps) {MSA2dist::cds2aa(...)}
    local_cdsstring2codonaln <- function(...,
        shorten,
        frame,
        framelist,
        genetic.code,
        return.cds) {MSA2dist::cdsstring2codonaln(...)}
    Comp1 <- FALSE
    Comp2 <- FALSE
    seq1 <- FALSE
    seq2 <- FALSE
    cds.names <- names(cds)
    cds.names <- gsub(" ", "_",
        gsub(" $", "", gsub("\\s+", " ", cds.names)))
    names(cds) <- cds.names
    stopifnot("Error: either choose model 'Li' or 'NG86'
        or KaKs_Calculator2 model"=
        model %in% c("Li", "NG86",
            "NG", "LWL", "LPB", "MLWL", "MLPB", "GY", "YN",
            "MYN", "MS", "MA", "GNG", "GLWL", "GLPB", "GMLWL",
            "GMLPB", "GYN", "GMYN"))
    if(model == "Li"){
        if("Comp1" %in% names(cds)){
            names(cds)[which(names(cds) == "Comp1")] <- "_Comp1"
            Comp1 <- TRUE
        }
        if("Comp2" %in% names(cds)){
            names(cds)[which(names(cds) == "Comp2")] <- "_Comp2"
            Comp2 <- TRUE
        }
        if("seq1" %in% names(cds)){
            names(cds)[which(names(cds) == "seq1")] <- "_seq1"
            seq1 <- TRUE
        }
        if("seq2" %in% names(cds)){
            names(cds)[which(names(cds) == "seq2")] <- "_seq2"
            seq2 <- TRUE
        }
        if(isMSA){
            OUT <- seqinr::kaks(dnastring2aln(cds))
            OUT.ka <- as.matrix(OUT$ka)
            OUT.ks <- as.matrix(OUT$ks)
            OUT.vka <- as.matrix(OUT$vka)
            OUT.vks <- as.matrix(OUT$vks)
            OUT.ka <- OUT.ka %>% tidyr::as_tibble(rownames = "seq1") %>%
                tidyr::pivot_longer(colnames(OUT.ka),
                names_to = "seq2", values_to = "ka") %>%
                dplyr::slice(uptriidx(ncol(OUT.ka)))
            OUT.ks <- OUT.ks %>% tidyr::as_tibble(rownames = "seq1") %>%
                tidyr::pivot_longer(colnames(OUT.ks),
                names_to = "seq2", values_to = "ks") %>%
                dplyr::slice(uptriidx(ncol(OUT.ks)))
            OUT.vka <- OUT.vka %>% tidyr::as_tibble(rownames = "seq1") %>%
                tidyr::pivot_longer(colnames(OUT.vka),
                names_to = "seq2", values_to = "vka") %>%
                dplyr::slice(uptriidx(ncol(OUT.vka)))
            OUT.vks <- OUT.vks %>% tidyr::as_tibble(rownames = "seq1") %>%
                tidyr::pivot_longer(colnames(OUT.vks),
                names_to = "seq2", values_to = "vks") %>%
                dplyr::slice(uptriidx(ncol(OUT.vks)))
            OUT <- OUT.ka %>% dplyr::left_join(OUT.ks) %>%
                dplyr::left_join(OUT.vka) %>% dplyr::left_join(OUT.vks)
            OUT <- as.data.frame(cbind(Comp1=match(OUT$seq1, cds.names),
                Comp2=match(OUT$seq2, cds.names), OUT))
            if(Comp1){
                OUT$seq1 <- gsub("_Comp1", "Comp1", OUT$seq1)
                OUT$seq2 <- gsub("_Comp1", "Comp1", OUT$seq2)
            }
            if(Comp2){
                OUT$seq1 <- gsub("_Comp2", "Comp2", OUT$seq1)
                OUT$seq2 <- gsub("_Comp2", "Comp2", OUT$seq2)
            }
            if(seq1){
                OUT$seq1 <- gsub("_seq1", "seq1", OUT$seq1)
                OUT$seq2 <- gsub("_seq1", "seq1", OUT$seq2)
            }
            if(seq2){
                OUT$seq1 <- gsub("_seq2", "seq2", OUT$seq1)
                OUT$seq2 <- gsub("_seq2", "seq2", OUT$seq2)
            }
            OUT[["Ka"]] <- OUT[["ka"]]
            OUT[["Ks"]] <- OUT[["ks"]]
            OUT[["Ka/Ks"]] <- as.numeric(OUT[["ka"]])/
                as.numeric(OUT[["ks"]])
            attr(OUT, "model") <- "Li"
            attr(OUT, "align") <- "FALSE"
            attr(OUT, "MSA2dist.class") <- "dnastring2kaks"
            return(OUT)
        } else {
            if(.Platform$OS.type == "windows"){
                cl <- parallel::makeCluster(threads)
            }
            if(.Platform$OS.type != "windows"){
                cl <- parallel::makeForkCluster(threads)
            }
            doParallel::registerDoParallel(cl)
            i <- NULL
            j <- NULL
            aa <- local_cds2aa(cds, ...)
            cds <- local_cds2aa(cds, return.cds=TRUE, ...)
            OUT <- foreach(i = seq(from = 1, to = length(cds) - 1),
                .combine=rbind, .packages = c('foreach')) %dopar% {
                foreach(j = seq(from = i + 1, to = length(cds)),
                    .combine=rbind) %do% {
                    c(setNames(i, "Comp1"),
                    setNames(j, "Comp2"),
                    setNames(cds.names[i], "seq1"),
                    setNames(cds.names[j], "seq2"),
                    unlist(seqinr::kaks(MSA2dist::dnastring2aln(
                        local_cdsstring2codonaln(cds[c(i, j)], aa[c(i, j)],
                            ...)))))
                }
            }
            parallel::stopCluster(cl)
            if(is.null(dim(OUT))){
                OUT <- as.data.frame(t(OUT))
            } else {
                OUT <- as.data.frame(OUT)
            }
            OUT[["Ka"]] <- OUT[["ka"]]
            OUT[["Ks"]] <- OUT[["ks"]]
            OUT[["Ka/Ks"]] <- as.numeric(OUT[["ka"]])/
                as.numeric(OUT[["ks"]])
            attr(OUT, "model") <- "Li"
            attr(OUT, "align") <- "TRUE"
            attr(OUT, "MSA2dist.class") <- "dnastring2kaks"
            return(OUT)
        }
    }
    if(model == "NG86"){
        if(isMSA){
            if(.Platform$OS.type == "windows"){
                cl <- parallel::makeCluster(threads)
            }
            if(.Platform$OS.type != "windows"){
                cl <- parallel::makeForkCluster(threads)
            }
            doParallel::registerDoParallel(cl)
            i <- NULL
            j <- NULL
            codonmat <- MSA2dist::dnastring2codonmat(cds)
            OUT <- foreach(i = seq(from = 1, to = ncol(codonmat) - 1),
                .combine=rbind, .packages = c('foreach')) %dopar% {
                foreach(j = seq(from = i + 1, to = ncol(codonmat)),
                    .combine=rbind) %do% {
                    c(setNames(i, "Comp1"),
                    setNames(j, "Comp2"),
                    MSA2dist::codonmat2pnps(codonmat[, c(i, j)]))
                }
            }
            parallel::stopCluster(cl)
            OUT <- as.data.frame(OUT)
            OUT[["Ka"]] <- OUT[["dn"]]
            OUT[["Ks"]] <- OUT[["ds"]]
            OUT[["Ka/Ks"]] <- OUT[["dn/ds"]]
            attr(OUT, "model") <- "NG86"
            attr(OUT, "align") <- "FALSE"
            attr(OUT, "MSA2dist.class") <- "dnastring2kaks"
            return(OUT)
        } else {
            if(.Platform$OS.type == "windows"){
                cl <- parallel::makeCluster(threads)
            }
            if(.Platform$OS.type != "windows"){
                cl <- parallel::makeForkCluster(threads)
            }
            doParallel::registerDoParallel(cl)
            i <- NULL
            j <- NULL
            aa <- local_cds2aa(cds, ...)
            cds <- local_cds2aa(cds, return.cds=TRUE, ...)
            OUT <- foreach(i = seq(from = 1, to = length(cds) - 1),
                .combine=rbind, .packages = c('foreach')) %dopar% {
                foreach(j = seq(from = i + 1, to = length(cds)),
                    .combine=rbind) %do% {
                    c(setNames(i, "Comp1"),
                    setNames(j, "Comp2"),
                    MSA2dist::codonmat2pnps(
                    MSA2dist::dnastring2codonmat(
                    local_cdsstring2codonaln(cds[c(i, j)], aa[c(i, j)],
                        ...))))
                }
            }
            parallel::stopCluster(cl)
            if(is.null(dim(OUT))){
                OUT <- as.data.frame(t(OUT))
            } else {
                OUT <- as.data.frame(OUT)
            }
            OUT[["Ka"]] <- OUT[["dn"]]
            OUT[["Ks"]] <- OUT[["ds"]]
            OUT[["Ka/Ks"]] <- OUT[["dn/ds"]]
            attr(OUT, "model") <- "NG86"
            attr(OUT, "align") <- "TRUE"
            attr(OUT, "MSA2dist.class") <- "dnastring2kaks"
            return(OUT)
        }
    }
    if(model != "Li" && model != "NG86"){
        if(isMSA){
            OUT <- rcpp_KaKs(cdsstr = as.character(cds),
                sgc = sgc, method = model, verbose = verbose)
            OUT <- as.data.frame(t(
                tibble::column_to_rownames(tidyr::as_tibble(
                setNames(stringr::str_split(OUT$results_vec,
                "\t"), OUT$results_names)) %>%
                tibble::add_column(rownames=OUT$rownames),
                "rownames")))
            attr(OUT, "model") <- model
            attr(OUT, "align") <- "FALSE"
            attr(OUT, "MSA2dist.class") <- "dnastring2kaks"
            return(OUT)
        } else{
            if(.Platform$OS.type == "windows"){
                cl <- parallel::makeCluster(threads)
            }
            if(.Platform$OS.type != "windows"){
                cl <- parallel::makeForkCluster(threads)
            }
            doParallel::registerDoParallel(cl)
            i <- NULL
            j <- NULL
            aa <- local_cds2aa(cds, ...)
            cds <- local_cds2aa(cds, return.cds=TRUE, ...)
            OUT <- foreach(i = seq(from = 1, to = length(cds) - 1),
                .combine=rbind, .packages = c('foreach')) %dopar% {
                foreach(j = seq(from = i + 1, to = length(cds)),
                    .combine=rbind) %do% {
                    tmp_out <- rcpp_KaKs(
                        cdsstr = as.character(
                        local_cdsstring2codonaln(cds[c(i, j)], aa[c(i, j)],
                            ...)),
                        sgc = sgc,
                        method = model)
                    tmp_out <- as.data.frame(t(
                        tibble::column_to_rownames(
                        tidyr::as_tibble(
                        setNames(stringr::str_split(
                        tmp_out$results_vec,
                        "\t"), tmp_out$results_names)) %>%
                        tibble::add_column(
                        rownames=tmp_out$rownames),
                        "rownames")))
                    tmp_out["Comp1"] <- i
                    tmp_out["Comp2"] <- j
                    tmp_out
                }
            }
            parallel::stopCluster(cl)
            OUT <- as.data.frame(OUT)
            attr(OUT, "model") <- model
            attr(OUT, "align") <- "TRUE"
            attr(OUT, "MSA2dist.class") <- "dnastring2kaks"
            return(OUT)
        }
    }
}

#' @title getmask
#' @name getmask
#' @description This function shows the mask slot from a
#' \code{DNAStringSet} or an \code{AAStringSet}
#' \code{metadata} information.\cr
#' @param seq \code{DNAStringSet} or \code{AAStringSet} [mandatory]
#' @return \code{IRanges} information from \code{metadata}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @seealso \code{\link[MSA2dist]{addpop2string}}
#' @examples
#' ## load example sequence data
#' data(iupac, package="MSA2dist")
#' iupac.aa <- iupac |> cds2aa(shorten = TRUE)
#' ## create mask
#' mask1 <- IRanges::IRanges(start=c(1,41), end=c(20,50))
#' ## add mask
#' iupac.aa <- iupac.aa |> addmask2string(mask=mask1)
#' #(iupac.aa |> slot("metadata"))$mask
#' iupac.aa |> getmask()
#' @export getmask
#' @author Kristian K Ullrich

getmask <- function(seq){
    stopifnot("Error: input needs to be a DNAStringSet or AAStringSet"=
        methods::is(seq, "AAStringSet") | methods::is(seq, "DNAStringSet"))
    return(methods::slot(seq, "metadata")$mask)
}

#' @title aln2dnastring
#' @name aln2dnastring
#' @description This function converts a \code{seqinr} \code{alignment} into
#' an \code{DNAStringSet}.
#' @param aln \code{seqinr} \code{alignment} [mandatory]
#' @return An object of class \code{DNAStringSet}
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom seqinr as.alignment
#' @seealso \code{\link[seqinr]{as.alignment}}
#' \code{\link[Biostrings]{DNAStringSet}}
#' @examples
#' ## define two cds sequences
#' cds1 <- Biostrings::DNAString("ATGCAACATTGC")
#' cds2 <- Biostrings::DNAString("ATG---CATTGC")
#' cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
#'     Biostrings::DNAStringSet(cds2))
#' ## convert into alignment
#' #dnastring2aln(cds1.cds2.aln)
#' cds1.cds2.aln |> dnastring2aln()
#' ## convert back into DNAStringSet
#' #aln2dnastring(dnastring2aln(cds1.cds2.aln))
#' cds1.cds2.aln |> dnastring2aln() |> aln2dnastring()
#' @export aln2dnastring
#' @author Kristian K Ullrich

aln2dnastring <- function(aln){
    dna <- Biostrings::DNAStringSet(unlist(aln$seq))
    names(dna) <- aln$nam
    return(dna)
}

#' @title dnabin2dnastring
#' @name dnabin2dnastring
#' @description This function converts an \code{ape} \code{DNAbin} into a
#' \code{DNAStringSet}.
#' @param dnabin \code{ape} \code{DNAbin} [mandatory]
#' @return An object of class \code{DNAStringSet}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom seqinr as.alignment
#' @importFrom ape as.DNAbin.alignment
#' @seealso \code{\link[seqinr]{as.alignment}}
#' \code{\link[ape]{as.DNAbin.alignment}}
#' \code{\link[Biostrings]{DNAStringSet}}
#' @examples
#' data(woodmouse, package="ape")
#' ## convert into DNAStringSet
#' #dnabin2dnastring(woodmouse)
#' woodmouse |> dnabin2dnastring()
#' @export dnabin2dnastring
#' @author Kristian K Ullrich

dnabin2dnastring <- function(dnabin){
    stopifnot("Error: input needs to be a DNAbin"=
        methods::is(dnabin, "DNAbin"))
    dna <- setNames(Biostrings::DNAStringSet(ape::as.alignment(dnabin)$seq),
        labels(dnabin))
    return(dna)
}

data(woodmouse, package="ape")

test_that("rcpp_pairwiseDeletionDNA()", {
    w <- woodmouse |> dnabin2dnastring() |> as.character()
    expect_true(rcpp_pairwiseDeletionDNA(dnavector=w, ncores=1)$sitesUsed[2,2]
    == 962)
    expect_true(rcpp_pairwiseDeletionDNA(dnavector=w, ncores=1,
    symmetric=0)$sitesUsed[2,2] == 962)
})

#' @title regionused
#' @name regionused
#' @description This function shows the region used slot from a
#' \code{DNAStringSet} or an \code{AAStringSet}
#' \code{metadata} information.\cr
#' @param seq \code{DNAStringSet} or \code{AAStringSet} [mandatory]
#' @return population names from \code{metadata}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @seealso \code{\link[MSA2dist]{addpop2string}}
#' @examples
#' ## load example sequence data
#' data("hiv", package="MSA2dist")
#' ## create mask
#' mask1 <- IRanges::IRanges(start=c(11,41,71), end=c(20,50,80))
#' ## use mask
#' hiv.region <- hiv |> cds2aa() |> string2region(mask=mask1)
#' #(hiv.region |> slot("metadata"))$regionUsed
#' hiv.region |> regionused()
#' @export regionused
#' @author Kristian K Ullrich

regionused <- function(seq){
    stopifnot("Error: input needs to be a DNAStringSet or AAStringSet"=
        methods::is(seq, "AAStringSet") | methods::is(seq, "DNAStringSet"))
    return(methods::slot(seq, "metadata")$regionUsed)
}

#' @title cds2aa
#' @name cds2aa
#' @description This function translates a \code{DNAStringSet} into an
#' \code{AAStringSet}.
#' @param cds \code{DNAStringSet} [mandatory]
#' @param shorten shorten all sequences to multiple of three [default: FALSE]
#' @param frame  indicates the first base of a the first codon [default: 1]
#' @param framelist  supply vector of frames for each entry [default: NULL]
#' @param genetic.code The genetic code to use for the translation of codons
#' into Amino Acid letters [default: NULL]
#' @param return.cds return shorten cds instead of aa [default: FALSE]
#' @return \code{AAStringSet}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq translate
#' getGeneticCode GENETIC_CODE GENETIC_CODE_TABLE
#' @importFrom stringr word
#' @seealso \code{\link[Biostrings]{XStringSet-class}},
#' \code{\link[seqinr]{translate}}
#' @examples
#' ## define two cds sequences
#' cds1 <- Biostrings::DNAString("ATGCAACATTGC")
#' cds2 <- Biostrings::DNAString("ATG---CATTGC")
#' cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
#'     Biostrings::DNAStringSet(cds2))
#' #cds2aa(cds1.cds2.aln)
#' cds1.cds2.aln |> cds2aa()
#' ## alternative genetic code
#' data(woodmouse, package="ape")
#' #cds2aa(dnabin2dnastring(woodmouse), shorten=TRUE)
#' woodmouse |> dnabin2dnastring() |> cds2aa(shorten=TRUE)
#' #cds2aa(dnabin2dnastring(woodmouse), shorten=TRUE,
#' #genetic.code=Biostrings::getGeneticCode("2"))
#' woodmouse |> dnabin2dnastring() |> cds2aa(shorten=TRUE,
#' genetic.code=Biostrings::getGeneticCode("2"))
#' woodmouse |> dnabin2dnastring() |> cds2aa(shorten=TRUE, return.cds=TRUE) |>
#' cds2aa(genetic.code=Biostrings::getGeneticCode("2"))
#' @export cds2aa
#' @author Kristian K Ullrich

cds2aa <- function(cds, shorten=FALSE, frame=1, framelist=NULL,
    genetic.code=NULL, return.cds=FALSE){
    stopifnot("Error: input needs to be a DNAStringSet"=
        methods::is(cds, "DNAStringSet"))
    stopifnot("Error: frame needs to be 1 or 2 or 3"= frame %in% c(1, 2, 3))
    if(!is.null(framelist)){
        stopifnot("Error: framelist needs to be of equal length as cds"=
            length(framelist) == length(cds))
    }
    if(!is.null(names(cds))){
        names(cds) <- stringr::word(names(cds), 1)
    }
    if(is.null(framelist)){
        cds <- Biostrings::subseq(cds, frame, Biostrings::width(cds))
    }
    if(!is.null(framelist)){
        cds <- Biostrings::subseq(cds, framelist, Biostrings::width(cds))
    }
    if(shorten){
        cds <- Biostrings::subseq(cds, 1,
            Biostrings::width(cds) - Biostrings::width(cds) %% 3)
    }
    cds_not_multiple_of_three.idx <- which(Biostrings::width(cds) %% 3 != 0)
    if(length(cds_not_multiple_of_three.idx) > 0){
        cds_not_multiple_of_three <- cds[cds_not_multiple_of_three.idx]
        cds <- cds[-cds_not_multiple_of_three.idx]
    }
    cds <- Biostrings::DNAStringSet(gsub("-", "N", cds))
    cds <- Biostrings::DNAStringSet(gsub("X", "N", cds))
    if(return.cds){
        return(cds)
    }
    if(!is.null(genetic.code)){
        aa <- Biostrings::translate(cds, genetic.code=genetic.code,
            if.fuzzy.codon="X")
        return(aa)
    }
    aa <- Biostrings::translate(cds, if.fuzzy.codon="X")
    return(aa)
}

data(hiv)

test_that("aastring2dist()", {
    h <- hiv |> cds2aa() |> aastring2dist(score=granthamMatrix())
    expect_true(h$sitesUsed[1,1] == 91)
    h <- hiv |> cds2aa() |> aastring2dist(score=granthamMatrix(), threads=2)
    expect_true(h$sitesUsed[1,1] == 91)
    mask1 <- IRanges::IRanges(start=c(11,41,71), end=c(20,50,80))
    h <- hiv |> cds2aa() |> aastring2dist(score=granthamMatrix(), mask=mask1)
    expect_true(h$sitesUsed[1,1] == 61)
    region1 <- IRanges::IRanges(start=c(1,75), end=c(45,85))
    h <- hiv |> cds2aa() |> aastring2dist(score=granthamMatrix(),
        region=region1)
    expect_true(h$sitesUsed[1,1] == 56)
    h <- hiv |> cds2aa() |> aastring2dist(score=granthamMatrix(),
        mask=mask1, region=region1)
    expect_true(h$sitesUsed[1,1] == 35)
    myscore <- granthamMatrix()
    myscore[5,6]<-0
    h <- hiv |> cds2aa() |> aastring2dist(score=myscore)
    expect_true(h$distSTRING[1,2] == h$distSTRING[2,1])
    h <- hiv |> cds2aa() |> aastring2dist(score=myscore, symmetric=FALSE)
    expect_true(h$distSTRING[1,2] != h$distSTRING[2,1])
})

data(hiv)

test_that("rcpp_pairwiseDeletionAA()", {
    h <- hiv |> cds2aa() |> as.character()
    expect_true(rcpp_pairwiseDeletionAA(aavector=h, ncores=1)$sitesUsed[1,2]
    == 91)
    expect_true(rcpp_pairwiseDeletionAA(aavector=h, ncores=1,
    symmetric=0)$sitesUsed[1,2] == 91)
})

data(hiv)

test_that("cds2aa() outputs AAStringSet", {
    expect_true(class(cds2aa(hiv)) == "AAStringSet")
    expect_true(as.character(cds2aa(Biostrings::DNAStringSet("ATGCAACATTGC")))
    == "MQHC")
    expect_true(length(cds2aa(Biostrings::DNAStringSet("ATGCAACATTGC"),
    frame = 2)) == 0)
    expect_true(as.character(cds2aa(Biostrings::DNAStringSet("ATGCAACATTGC"),
    frame = 2, shorten = TRUE)) == "CNI")
    cds1 <- Biostrings::DNAString("ATGCAACATTGC")
    cds2 <- Biostrings::DNAString("ATG---CATTGC")
    cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
        Biostrings::DNAStringSet(cds2))
    expect_true(as.character(cds2aa(cds1.cds2.aln,framelist=c(2,3),
    shorten=TRUE))[2] == "XXL")
    expect_true(as.character(cds2aa(Biostrings::DNAStringSet("GTG"),
    genetic.code=Biostrings::getGeneticCode("2"))) == "M")
    expect_true(as.character(cds2aa(cds1.cds2.aln, return.cds=TRUE))[1]
    == "ATGCAACATTGC")
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @useDynLib MSA2dist, .registration = TRUE
#' @import Rcpp
#' @title rcpp_KaKs
#' @name rcpp_KaKs
#' @description calculates KaKs as implememted in
#' KaKs Calculator 2.0 \code{MSA2dist} with \code{Rcpp}.
#' @return \code{list}
#' @param cdsstr StringVector [mandatory]
#' @param sgc standard genetic code to use [default: 1]
#' @param method KaKs Calculator 2.0 codon model [default: YN]
#' @param verbose specify if verbose output [default: FALSE]
#' @references Wang et al. (2010) KaKs_Calculator 2.0: a toolkit incorporating
#' gamma-series methods and sliding window strategies.\emph{Genomics,
#' proteomics & bioinformatics.} \bold{8(1)}, 77-80.
#' @examples
#' ## load example sequence data
#' data("hiv", package="MSA2dist")
#' rcpp_KaKs(cdsstr=as.character(hiv[1:3]))
#' @export rcpp_KaKs
#' @author Kristian K Ullrich
rcpp_KaKs <- function(cdsstr, sgc = "1", method = "YN", verbose = FALSE) {
    .Call(`_MSA2dist_rcpp_KaKs`, cdsstr, sgc, method, verbose)
}

#' @useDynLib MSA2dist, .registration = TRUE
#' @import Rcpp
#' @title rcpp_distSTRING
#' @name rcpp_distSTRING
#' @description calculates pairwise distances using a score matrix
#' @return list
#' @param dnavector StringVector [mandatory]
#' @param scoreMatrix NumericMatrix [mandatory]
#' @param ncores number of cores [default: 1]
#' @param symmetric symmetric score matrix [default: 1]
#' @examples
#' ## load example sequence data
#' data("hiv", package="MSA2dist")
#' rcpp_distSTRING(dnavector=as.character(hiv), scoreMatrix=iupacMatrix())
#' @export rcpp_distSTRING
#' @author Kristian K Ullrich
rcpp_distSTRING <- function(dnavector, scoreMatrix,
    ncores = 1L, symmetric = 1L) {
    .Call(`_MSA2dist_rcpp_distSTRING`, dnavector, scoreMatrix,
    ncores, symmetric)
}

#' @useDynLib MSA2dist, .registration = TRUE
#' @import Rcpp
#' @title rcpp_pairwiseDeletionAA
#' @name rcpp_pairwiseDeletionAA
#' @description returns number of AA sites used
#' @return list
#' @param aavector StringVector [mandatory]
#' @param ncores number of cores [default: 1]
#' @param symmetric symmetric score matrix [default: 1]
#' @examples
#' ## load example sequence data
#' data("hiv", package="MSA2dist")
#' h <- hiv |> cds2aa() |> as.character()
#' rcpp_pairwiseDeletionAA(aavector=h, ncores=1)
#' @export rcpp_pairwiseDeletionAA
#' @author Kristian K Ullrich
rcpp_pairwiseDeletionAA <- function(aavector, ncores = 1L, symmetric = 1L) {
    .Call(`_MSA2dist_rcpp_pairwiseDeletionAA`, aavector, ncores, symmetric)
}

#' @useDynLib MSA2dist, .registration = TRUE
#' @import Rcpp
#' @title rcpp_pairwiseDeletionDNA
#' @name rcpp_pairwiseDeletionDNA
#' @description returns number of DNA sites used
#' @return list
#' @param dnavector StringVector [mandatory]
#' @param ncores number of cores [default: 1]
#' @param symmetric symmetric score matrix [default: 1]
#' @examples
#' ## load example sequence data
#' data("woodmouse", package="ape")
#' w <- woodmouse |> dnabin2dnastring() |> as.character()
#' rcpp_pairwiseDeletionDNA(dnavector=w, ncores=1)
#' @export rcpp_pairwiseDeletionDNA
#' @author Kristian K Ullrich
rcpp_pairwiseDeletionDNA <- function(dnavector, ncores = 1L, symmetric = 1L) {
    .Call(`_MSA2dist_rcpp_pairwiseDeletionDNA`, dnavector, ncores, symmetric)
}

data(hiv)

test_that("rcpp_distSTRING()", {
    expect_true(rcpp_distSTRING(dnavector=as.character(hiv),
    scoreMatrix=iupacMatrix())$sitesUsed[1,2] == 273)
    expect_true(rcpp_distSTRING(dnavector=as.character(hiv),
    scoreMatrix=iupacMatrix(), ncores=2)$sitesUsed[1,2] == 273)
    myscore <- iupacMatrix()
    myscore[1,4] <- 0.5
    h <- rcpp_distSTRING(dnavector=as.character(hiv), scoreMatrix=myscore)
    expect_true(h$distSTRING[1,2] == h$distSTRING[2,1])
    h <- rcpp_distSTRING(dnavector=as.character(hiv), scoreMatrix=myscore,
        symmetric=0)
    expect_true(h$distSTRING[1,2] != h$distSTRING[2,1])
})

#' @title dnastring2codonmat
#' @name dnastring2codonmat
#' @description This function converts a \code{DNAStringSet} into a
#' \code{codon matrix}.
#' @param cds \code{DNAStringSet} [mandatory]
#' @param shorten shorten all sequences to multiple of three [default: FALSE]
#' @param frame  indicates the first base of a the first codon [default: 1]
#' @param framelist  supply vector of frames for each entry [default: NULL]
#' @return An object of class \code{alignment} which is a list with the
#' following components:\cr
#' \code{nb} the number of aligned sequences\cr
#' \code{nam} a vector of strings containing the names of the aligned
#' sequences\cr
#' \code{seq} a vector of strings containing the aligned sequences\cr
#' \code{com} a vector of strings containing the commentaries for each sequence
#' or \code{NA} if there are no comments
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom stringr word
#' @seealso \code{\link[seqinr]{as.alignment}}
#' @examples
#' ## define two cds sequences
#' cds1 <- Biostrings::DNAString("ATGCAACATTGC")
#' cds2 <- Biostrings::DNAString("ATG---CATTGC")
#' cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
#'     Biostrings::DNAStringSet(cds2))
#' ## convert into alignment
#' #dnastring2codonmat(cds1.cds2.aln)
#' cds1.cds2.aln |> dnastring2codonmat()
#' ## use frame 2 and shorten to circumvent multiple of three error
#' cds1 <- Biostrings::DNAString("-ATGCAACATTGC-")
#' cds2 <- Biostrings::DNAString("-ATG---CATTGC-")
#' cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
#'     Biostrings::DNAStringSet(cds2))
#' cds1.cds2.aln |> dnastring2codonmat(frame=2, shorten=TRUE)
#' @export dnastring2codonmat
#' @author Kristian K Ullrich

dnastring2codonmat <- function(cds, shorten = FALSE, frame = 1, framelist=NULL){
    stopifnot("Error: input needs to be a DNAStringSet"=
        methods::is(cds, "DNAStringSet"))
    stopifnot("Error: input needs to be a Alignment of equal width"=
        length(unique(Biostrings::width(cds)))==1)
    stopifnot("Error: frame needs to be 1 or 2 or 3"= frame %in% c(1, 2, 3))
    if(!is.null(framelist)){
        stopifnot("Error: framelist needs to be of equal length as cds"=
            length(framelist) != length(cds))
    }
    if(!is.null(names(cds))){
        names(cds) <- stringr::word(names(cds), 1)
    }
    if(is.null(framelist)){
        cds <- Biostrings::subseq(cds, frame, Biostrings::width(cds))
    }
    if(!is.null(framelist)){
        cds <- Biostrings::subseq(cds, framelist, Biostrings::width(cds))
    }
    if(shorten){
        cds <- Biostrings::subseq(cds, 1,
            Biostrings::width(cds) - Biostrings::width(cds) %% 3)
    }
    cds_not_multiple_of_three.idx <- which(Biostrings::width(cds) %% 3 != 0)
    if(length(cds_not_multiple_of_three.idx) > 0){
        cds_not_multiple_of_three <- cds[cds_not_multiple_of_three.idx]
        cds <- cds[-cds_not_multiple_of_three.idx]
    }
    # remove IUPAC chars
    cds <- Biostrings::DNAStringSet(gsub("R", "N", cds))
    cds <- Biostrings::DNAStringSet(gsub("Y", "N", cds))
    cds <- Biostrings::DNAStringSet(gsub("S", "N", cds))
    cds <- Biostrings::DNAStringSet(gsub("W", "N", cds))
    cds <- Biostrings::DNAStringSet(gsub("K", "N", cds))
    cds <- Biostrings::DNAStringSet(gsub("M", "N", cds))
    cds <- Biostrings::DNAStringSet(gsub("B", "N", cds))
    cds <- Biostrings::DNAStringSet(gsub("D", "N", cds))
    cds <- Biostrings::DNAStringSet(gsub("H", "N", cds))
    cds <- Biostrings::DNAStringSet(gsub("V", "N", cds))
    cds.codonmat <- apply(cbind(Biostrings::width(cds), as.character(cds)), 1,
        function(x) {stringi::stri_sub(x[2], seq(1, as.numeric(x[1]), by = 3),
        length = 3)})
    return(cds.codonmat)
}

#' @title iupac-data
#' @name iupac-data
#' @description Example IUPAC sequences created with \code{angsd}
#' from different house mouse (Mus musculus) sub-populations
#' from Harr et al. (2016) \code{DNAStringSet}.
#' @importFrom Biostrings DNAStringSet
#' @references Harr et al. (2016) Genomic resources for wild populations of the
#' house mouse, Mus musculus and its close relative
#' Mus spretus. \emph{Scientific data.} \bold{3(1)}, 1-14.
#' @docType data
#' @usage data(iupac)
#' @format an object of class \code{DNAStringSet}
#' see \code{\link[Biostrings]{XStringSet-class}}
#' @keywords datasets
#' @examples
#' data("iupac", package="MSA2dist")

"iupac"

#' @title addregion2string
#' @name addregion2string
#' @description This function adds region information as an \code{IRanges}
#' object, \code{START} and \code{END} information, to a
#' \code{DNAStringSet} or an \code{AAStringSet} and puts them into the
#' \code{metadata} information.
#' This information can be used to restrict the distance calculation to
#' specific regions of the \code{DNAStringSet} or the \code{AAStringSet}.
#' @param seq \code{DNAStringSet} or \code{AAStringSet} [mandatory]
#' @param region \code{IRanges} object [mandatory]
#' @param append indicate if region should be appended or overwritten
#' [default: TRUE]
#' @return An object of class \code{DNAStringSet} or \code{AAStringSet}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom IRanges IRanges IRangesList reduce start end findOverlaps
#' disjoin overlapsRanges
#' @seealso \code{\link[MSA2dist]{addmask2string}},
#' \code{\link[MSA2dist]{addpop2string}},
#' \code{\link[MSA2dist]{addpos2string}}
#' @examples
#' ## load example sequence data
#' data(iupac, package="MSA2dist")
#' iupac.aa <- iupac |> cds2aa(shorten = TRUE)
#' ## create region
#' region1 <- IRanges::IRanges(start=c(1,41), end=c(20,50))
#' ## add region
#' iupac.aa <- iupac.aa |> addregion2string(region=region1)
#' #(iupac.aa |> slot("metadata"))$region
#' iupac.aa |> region()
#' ## append region
#' region2 <- IRanges::IRanges(start=c(21), end=c(30))
#' iupac.aa <- iupac.aa |> addregion2string(region=region2)
#' #(iupac.aa |> slot("metadata"))$region
#' iupac.aa |> region()
#' ## overwrite region
#' iupac.aa <- iupac.aa |> addregion2string(region=region2, append=FALSE)
#' #(iupac.aa |> slot("metadata"))$region
#' iupac.aa |> region()
#' ## reduce by region
#' #iupac.aa.region <- iupac.aa |> string2region(region=
#' #    (iupac.aa |> slot("metadata"))$region)
#' iupac.aa.region <- iupac.aa |> string2region(region=
#'     region(iupac.aa))
#' #iupac.aa.region |> slot("metadata")
#' iupac.aa.region |> region()
#' @export addregion2string
#' @author Kristian K Ullrich

addregion2string <- function(seq, region = NULL, append = TRUE){
    stopifnot("Error: input needs to be a DNAStringSet or AAStringSet"=
        methods::is(seq, "AAStringSet") | methods::is(seq, "DNAStringSet"))
    stopifnot("Error: set region"= !is.null(region))
    stopifnot("Error: region needs to be an IRanges object"=
        methods::is(region, "IRanges"))
    if(append){
        if(is.null(seq@metadata$region)){
            methods::slot(seq, "metadata")$region <- IRanges::reduce(region)
        } else{
            methods::slot(seq, "metadata")$region <- IRanges::reduce(
                c(methods::slot(seq, "metadata")$region, region))
        }
    } else{
        methods::slot(seq, "metadata")$region <- IRanges::reduce(region)
    }
    return(seq)
}

data(iupac)

test_that("addregion2string() outputs DNAStringSet", {
    region1 <- IRanges::IRanges(start=c(1,41), end=c(20,50))
    iupac <- iupac |> addregion2string(region=region1)
    expect_true(iupac@metadata$region@width[1] == 20)
    region2 <- IRanges::IRanges(start=c(21), end=c(30))
    iupac <- iupac |> addregion2string(region=region2)
    expect_true(iupac@metadata$region@width[1] == 30)
    iupac <- iupac |> addregion2string(region=region2, append=FALSE)
    expect_true(iupac@metadata$region@width[1] == 10)
})

#' @title codon2numberAMBIG
#' @name codon2numberAMBIG
#' @description This function converts a \code{codon} into a \code{number},
#' but accept N and -.
#' @param codon [mandatory]
#' @return An object of class \code{numeric}
#' @importFrom Biostrings GENETIC_CODE
#' @importFrom stringr str_sub
#' @importFrom stats setNames
#' @seealso \code{\link[Biostrings]{GENETIC_CODE}}
#' @examples
#' #unlist(lapply(names(Biostrings::GENETIC_CODE), codon2numberAMBIG))
#' names(Biostrings::GENETIC_CODE) |> codon2numberAMBIG()
#' @export codon2numberAMBIG
#' @author Kristian K Ullrich

codon2numberAMBIG <- function(codon){
    baseNumber <- setNames(c(0, 1, 2, 3, 4, 5), c("T", "C", "A", "G", "N", "-"))
    left <- baseNumber[stringr::str_sub(codon, 1, 1)]
    mid <- baseNumber[stringr::str_sub(codon, 2, 2)]
    right <- baseNumber[stringr::str_sub(codon, 3, 3)]
    codonnumber <- (mid * 36) + (left * 6) + (right) + 1
    names(codonnumber) <- codon
    return(codonnumber)
}

#' @title globalDeletion
#' @name globalDeletion
#' @description This function returns a \code{DNAStringSet} reduced by all
#' sites containing any gaps ("-", "+", ".") or missing ("N") sites.
#' @return \code{DNAStringSet}
#' @importFrom methods is slot
#' @importFrom Biostrings consensusMatrix
#' @param dna \code{DNAStringSet} [mandatory]
#' @examples
#' ## define two cds sequences
#' cds1 <- Biostrings::DNAString("ATGCAACATTGC")
#' cds2 <- Biostrings::DNAString("ATG---CATTGC")
#' cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
#'     Biostrings::DNAStringSet(cds2))
#' globalDeletion(cds1.cds2.aln)
#' @export globalDeletion
#' @author Kristian K Ullrich

globalDeletion<-function(dna){
    stopifnot("Error: input needs to be a DNAStringSet"=
        methods::is(dna, "DNAStringSet"))
    cM <- Biostrings::consensusMatrix(dna)
    globalDeletionSites <- which(apply(cM, 2, function(x) sum(x[15:18]) >= 1))
    if(length(globalDeletionSites) == 0){
        return(dna)
    }
    return(MSA2dist::dnabin2dnastring(
        MSA2dist::dnastring2dnabin(dna)[, -(globalDeletionSites)]))
}

test_that("globalDeletion()", {
    cds1 <- Biostrings::DNAString("ATGCAACATTGC")
    cds2 <- Biostrings::DNAString("ATG---CATTGC")
    cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
        Biostrings::DNAStringSet(cds2))
    expect_true(as.character(globalDeletion(cds1.cds2.aln)[1]) == "ATGCATTGC")
})

#' @title hiv-data
#' @name hiv-data
#' @description Example cds sequences from HIV-1 sample 136 patient 1 from
#' Sweden envelope glycoprotein (env) gene, V3 region as \code{DNAStringSet}.
#' @importFrom Biostrings DNAStringSet
#' @references Yang et al. (2000) Codon-substitution models for heterogeneous
#' selection pressure at amino acid sites. \emph{Genetics.}
#' \bold{155(1)}, 431-449.
#' @docType data
#' @usage data(hiv)
#' @format an object of class \code{DNAStringSet}
#' see \code{\link[Biostrings]{XStringSet-class}}
#' @keywords datasets
#' @examples
#' data("hiv", package="MSA2dist")

"hiv"

#' @title aastring2dist
#' @name aastring2dist
#' @description This function calculates pairwise distances for all combinations
#' of a \code{AAStringSet}.
#' @param aa \code{AAStringSet} [mandatory]
#' @param threads number of parallel threads [default: 1]
#' @param symmetric symmetric score matrix [default: TRUE]
#' @param score \code{score matrix} use a score matrix to calculate distances
#' [mandatory]
#' @param mask \code{IRanges} object indicating masked sites
#' [default: NULL]
#' @param region \code{IRanges} object indicating region to use for dist
#' calculation (by default all sites are used) [default: NULL]
#' @return A \code{data.frame} of pairwise distance values
#' \code{distSTRING}, sites used \code{sitesUsed} and region used
#' \code{regionUsed}
#' @importFrom methods is slot
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom IRanges IRanges IRangesList reduce start end findOverlaps
#' disjoin overlapsRanges
#' @seealso \code{\link[MSA2dist]{dnastring2dist}}
#' @examples
#' ## load example sequence data
#' data("hiv", package="MSA2dist")
#' #aastring2dist(cds2aa(hiv), score=granthamMatrix())
#' hiv |> cds2aa() |> aastring2dist(score=granthamMatrix())
#' ## create mask
#' mask1 <- IRanges::IRanges(start=c(11,41,71), end=c(20,50,80))
#' ## use mask
#' hiv |> cds2aa() |> aastring2dist(score=granthamMatrix(), mask=mask1)
#' ## use region
#' region1 <- IRanges::IRanges(start=c(1,75), end=c(45,85))
#' hiv |> cds2aa() |> aastring2dist(score=granthamMatrix(), region=region1)
#' ## use mask and region
#' hiv |> cds2aa() |> aastring2dist(score=granthamMatrix(),
#'     mask=mask1, region=region1)
#' ## use asymmetric score matrix
#' myscore <- granthamMatrix()
#' myscore[5, 6] <- 0
#' h <- hiv |> cds2aa() |> aastring2dist(score=myscore, symmetric=FALSE)
#' h$distSTRING[1:2, 1:2]
#' @export aastring2dist
#' @author Kristian K Ullrich

aastring2dist <- function(aa, threads=1, symmetric=TRUE, score=NULL,
    mask=NULL, region=NULL){
    stopifnot("Error: input needs to be an AAStringSet"=
        methods::is(aa, "AAStringSet"))
    stopifnot("Error: set score matrix e.g 'granthamMatrix()'"= !is.null(score))
    if(symmetric){symmetric_int <- 1}else{symmetric_int <- 0}
    region.aa <- IRanges::IRanges(start=1, end=unique(width(aa)))
    if(!is.null(mask) || !is.null(region)){
        aa.region <- MSA2dist::string2region(aa, mask=mask, region=region)
        aa.char <- as.character(aa.region)
        region.aa <- methods::slot(aa.region, "metadata")$regionUsed
    } else{aa.char <- as.character(aa)}
    OUT <- MSA2dist::rcpp_distSTRING(dnavector=aa.char,
        scoreMatrix=score, ncores=threads, symmetric=symmetric_int)
    OUT$distSTRING <- as.data.frame(OUT$distSTRING)
    OUT$sitesUsed <- as.data.frame(OUT$sitesUsed)
    OUT$regionUsed <- region.aa
    return(OUT)
}

#' @title pal2nal
#' @name pal2nal
#' @description This function takes an \code{AAStringSet} alignment and
#' its corresponding coding sequences \code{DNAStringSet} and converts
#' the protein alignment into a codon alignment.
#' @param pal \code{AAStringSet} [mandatory]
#' @param nal \code{DNAStringSet} [mandatory]
#' @param remove.gaps specify if gaps in the codon alignment should be removed
#' [default: FALSE]
#' @return codon alignment as \code{DNAStringSet}
#' @importFrom Biostrings DNAString DNAStringSet AAString AAStringSet
#' readDNAStringSet readAAStringSet writeXStringSet width subseq
#' @importFrom pwalign pairwiseAlignment
#' @importFrom methods is slot
#' @references Pagès, H et al. (2014) Biostrings: Efficient manipulation of
#' biological strings. \emph{R package version}, \bold{2(0)}.
#' @seealso \code{\link[pwalign]{pairwiseAlignment}}
#' @examples
#' ## define two cds sequences
#' cds <- Biostrings::DNAStringSet(c("ATGCAACATTGC", "ATGCATTGC"))
#' names(cds) <- c("cds1", "cds2")
#' ## get protein alignment
#' aa <- MSA2dist::cds2aa(cds)
#' msa <- makePostalignedSeqs(pwalign::pairwiseAlignment(aa[1], aa[2]))[[1L]]
#' names(msa) <- names(aa)
#' ## get codon alignment
#' nal <- MSA2dist::pal2nal(pal=msa, nal=cds)
#' nal
#' @export pal2nal
#' @author Kristian K Ullrich

pal2nal <- function(pal,
    nal,
    remove.gaps=FALSE){
    nal <- nal[names(pal)]
    pal.gap.pos <- lapply(pal, function(x) {
        gregexpr("-+", x)})
    pal.gap.len <- lapply(pal.gap.pos, function(x) {
        attr(x[[1]], "match.length")})
    pal.gap.pos <- lapply(pal.gap.pos, function(x) {
        ifelse(x[[1]]!=-1, x[[1]], -1)})
    pal.gap.len <- lapply(pal.gap.len, function(x) {
        if (x[1] != -1) return(x) else return(0)})
    pal.gap.len.cumsum <- lapply(pal.gap.len, cumsum)
    nal_out <- Biostrings::DNAStringSet()
    for(i in seq(from=1, to=length(pal))){
        if(pal.gap.pos[[i]][1] == -1){
            nal_out <- c(nal_out, nal[i])
        } else {
            nal_i <- as.character(nal[[i]])
            n_i_codons <- nchar(nal_i)/3
            n_i <- ""
            n_i_codons_added <- 0
            for(j in seq(from=1, to=length(pal.gap.pos[[i]]))){
                gap_pos <- pal.gap.pos[[i]][j]
                gap_len <- pal.gap.len[[i]][j]
                gap_len_cumsum <- pal.gap.len.cumsum[[i]][j]
                gap <- paste0(rep("---", gap_len), collapse="")
                if(gap_pos==1){
                    n_i <- paste0(n_i, gap)
                } else {
                    n_i_codons_to_add <- gap_pos-n_i_codons_added-
                        gap_len_cumsum+gap_len-1
                    n_i<- paste0(n_i, substr(nal_i,
                        (n_i_codons_added*3)+1,
                        (n_i_codons_added+n_i_codons_to_add)*3))
                    n_i <- paste0(n_i, gap)
                    n_i_codons_added <- n_i_codons_added+n_i_codons_to_add
                }
            }
            if(n_i_codons_added!=n_i_codons){
                n_i<- paste0(n_i, substr(nal_i,
                    (n_i_codons_added*3)+1,
                    n_i_codons*3))
            }
            nal_out <- c(nal_out,
                setNames(Biostrings::DNAStringSet(n_i),
                names(nal)[i]))
        }
    }
    if(remove.gaps){
        nal_out <- Biostrings::DNAStringSet(apply(as.matrix(nal_out)[,
            apply(as.matrix(nal_out), 2, function(x) !any(x=="-"))], 1,
            function(x) paste(x, collapse="")))
        names(nal_out) <- names(nal)
    }
    return(nal_out)
}

data(hiv)

test_that("pal2nal() outputs DNAStringSet", {
    cds1 <- Biostrings::DNAString("ATGCAACATTGC")
    cds2 <- Biostrings::DNAString("ATGCATTGC")
    cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
        Biostrings::DNAStringSet(cds2))
    names(cds1.cds2.aln) <- c("cds1", "cds2")
    aa1 <- Biostrings::AAString("MQHC")
    aa2 <- Biostrings::AAString("M-HC")
    aa1.aa2.aln <- c(Biostrings::AAStringSet(aa1),
        Biostrings::AAStringSet(aa2))
    names(aa1.aa2.aln) <- c("cds1", "cds2")
    expect_true(class(pal2nal(aa1.aa2.aln, cds1.cds2.aln)) == "DNAStringSet")
})

---
title: "MSA2dist Vignette"
author: "Kristian K Ullrich"
date: "`r Sys.Date()`"
abstract: >
    MSA2dist calculates pairwise distances between all sequences of
    a DNAStringSet or a AAStringSet using a custom score matrix and conducts
    codon based analysis. It uses scoring matrices to be used in these pairwise
    distance calculations which can be adapted to any scoring for DNA or AA
    characters. E.g. by using literal distances MSA2dist calcualtes pairwise
    IUPAC distances. DNAStringSet alignments can be analysed as codon alignments
    to look for synonymous and nonsynonymous substitutions (dN/dS) in a
    parallelised fashion using a variety of substitution models. Non-aligned
    coding sequences can be directly used to construct pairwise codon
    alignments (global/local) and calculate dN/dS without any external
    dependencies.
bibliography: bibliography.bib
nocite: '@*'
output: BiocStyle::html_document
vignette: >
    %\VignetteIndexEntry{MSA2dist Vignette}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r echo=FALSE, results='hide', warning=FALSE, message=FALSE}
suppressPackageStartupMessages({
    library(MSA2dist)
    library(Biostrings)
    library(ape)
    library(dplyr)
    library(tidyr)
    library(ggplot2)
    })
```

# Introduction

Calculating pairwise distances of either DNA or AA sequences is a common task
for evolutionary biologist. The distance calculations are either based on
specific nucleotide, codon or amino acid models or on a scoring matrix.

__Note:__ Sequences need to be pre-aligned into so called multiple sequence
alignments (MSA), which can be done with a multitude of existing software.
Just to mention for example [mafft](https://mafft.cbrc.jp/alignment/server/),
[muscle](https://www.ebi.ac.uk/Tools/msa/muscle/) or the R package
[`msa`](https://bioconductor.org/packages/release/bioc/html/msa.html).

The R package [`ape`](https://cran.r-project.org/web/packages/ape/index.html)
for example offers the `ape::dist.dna()` function, which has implemented a
collection of different evolutionary models.
[`MSA2dist`](http://bioconductor.org/
packages/release/bioc/html/MSA2dist.html)
extends the possibility to directly calculate pairwise nucloetide
distances of an `Biostrings::DNAStringSet` object or pairwise amino acid
distances of an `Biostrings::AAStringSet` object. The scoring matrix based
calculations are implemented in `c++` with `RcppThread` to parallelise pairwise
combinations.

It is a non-trivial part to resolve haploid (1n) sequences
from a diploid (2n) individual (aka phasing) to further use the
haploid sequences for distance calculations. To cope with this situation,
`MSA2dist` uses a literal distance [@chang2017whole] which can be directly
applied on `IUPAC` nucleotide ambiguity encoded sequences with the
`dnastring2dist()` function. `IUPAC` sequences can be for example obtained
directly from mapped `BAM` files and the [angsd](http://www.popgen.dk/
angsd/index.php/Fasta) `-doFasta 4` option [@korneliussen2014angsd].

The Grantham's score [@grantham1974amino] attempts to predict the distance
between two amino acids, in an evolutionary sense considering the amino acid
composition, polarity and molecular volume. `MSA2dist` offers with the
`aastring2dist()` function the possibility to obtain pairwise distances of all
sequences in an `Biostrings::AAStringSet` (needs to be pre-aligned). The
resulting distance matrix can be used to calculate neighbor-joining trees via
the R package [`ape`](https://cran.r-project.org/web/packages/ape/index.html).

Calculating synonymous (Ks) and nonsynonymous (Ka) substitutions from coding
sequences and its ratio Ka/Ks can be used as an indicator of selective
pressure acting on a protein. The `dnastring2kaks()` function can be applied on
pre-aligned `Biostrings::DNAStringSet()` objects to calculate these values
either according to [@li1993unbiased] via the R package
[`seqinr`](https://cran.r-project.org/web/packages/seqinr/index.html) or
according to the model of [@nei1986simple].

Further, all codons can be evaluated among the coding sequence alignment and
be plotted to for example protein domains with substitutions or indels with the
`codonmat2xy()` function.

# Installation

To install this package, start R (version "4.1") and enter:

```
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("MSA2dist")
```

# Load MSA2dist

```{r}
# load MSA2dist
library(MSA2dist)
# load example data
data(hiv, package="MSA2dist")
data(AAMatrix, package="MSA2dist")
data(woodmouse, package="ape")
```

# Sequence Format conversion

To be able to use distance calculation functions from other R packages, like
[`ape`](https://cran.r-project.org/web/packages/ape/index.html) or
[`seqinr`](https://cran.r-project.org/web/packages/seqinr/index.html), it is
necessary to have dedicated sequence format conversion functions. Here, some
examples are shown, how to convert from and to a `Biostrings::DNAStringSet`
object.

`?Biostrings::DNAStringSet()` >>> `?seqinr::as.alignment()`

```{r}
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
## define names
names(cds1.cds2.aln) <- c("seq1", "seq2")
## convert into alignment
cds1.cds2.aln |> dnastring2aln()
```

`?seqinr::as.alignment()` >>> `?Biostrings::DNAStringSet()`

```{r}
## convert back into DNAStringSet
cds1.cds2.aln |> dnastring2aln() |> aln2dnastring()
```

`?Biostrings::DNAStringSet()` >>> `?ape::DNAbin()`

```{r}
## convert into alignment
cds1.cds2.aln |> dnastring2dnabin()
```

`?ape::DNAbin()` >>> `?Biostrings::DNAStringSet()`

```{r}
## convert back into DNAStringSet
cds1.cds2.aln |> dnastring2dnabin() |> dnabin2dnastring()
## use woodmouse data
woodmouse |> dnabin2dnastring()
```

`?Biostrings::AAStringSet()` >>> `?seqinr::as.alignment()`

```{r}
## translate cds into aa
aa1.aa2.aln <- cds1.cds2.aln |> cds2aa()
## convert into alignment
aa1.aa2.aln |> aastring2aln()
```

`?seqinr::as.alignment()` >>> `?Biostrings::AAStringSet()`

```{r}
## convert back into AAStringSet
aa1.aa2.aln |> aastring2aln() |> aln2aastring()
```

`?Biostrings::AAStringSet()` >>> `?ape::as.AAbin()`

```{r}
## convert into AAbin
aa1.aa2.aln |> aastring2aabin()
```

`?ape::as.AAbin()` >>> `?Biostrings::AAStringSet()`

```{r}
## convert back into AAStringSet
aa1.aa2.aln |> aastring2aabin() |> aabin2aastring()
```

# Frame aware `Biostrings::DNAStringSet` translation (`cds2aa()`)

To be able to translate a coding sequence into amino acids, sometimes the
sequences do not start at the first frame. The `cds2aa` function can take an
alternative codon start site into account (`frame = 1` or `frame = 2` or
`frame = 3`). However, sometimes it is also
necessary that the resulting coding sequence length is a multiple of three.
This can be forced by using the `shorten = TRUE` option.

Simple translation:

```{r}
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
## define names
names(cds1.cds2.aln) <- c("seq1", "seq2")
## translate cds into aa
cds1.cds2.aln |> cds2aa()
aa1.aa2.aln <- cds1.cds2.aln |> cds2aa()
```

Translation keeping multiple of three sequence length:

```{r}
## translate cds into aa using frame = 2
## result is empty due to not multiple of three
cds1.cds2.aln |> cds2aa(frame=2)
## translate cds into aa using frame = 2 and shorten = TRUE
cds1.cds2.aln |> cds2aa(frame=2, shorten=TRUE)
## translate cds into aa using frame = 3 and shorten = TRUE
cds1.cds2.aln |> cds2aa(frame=3, shorten=TRUE)
## use woodmouse data
woodmouse |> dnabin2dnastring() |> cds2aa(shorten=TRUE)
```

Translation using alternative genetic code:

As you can see from the above example, the initial amino acids `I` will change
into `M` due to the mitochondrial translation code and also some `*` stop
codons will change into a `W` amino acid.

```{r}
## alternative genetic code
## use woodmouse data
woodmouse |> dnabin2dnastring() |> cds2aa(shorten=TRUE,
    genetic.code=Biostrings::getGeneticCode("2"))
```

# Pairwise sequence comparison

## Calculate pairwise AA distances (`aastring2dist()`)

### Grantham's distance

```{r}
## calculate pairwise AA distances based on Grantham's distance
aa.dist <- hiv |> cds2aa() |> aastring2dist(score=granthamMatrix())
## obtain distances
head(aa.dist$distSTRING)
## obtain pairwise sites used
head(aa.dist$sitesUsed)
```

```{r}
## create and plot bionj tree
aa.dist.bionj <- ape::bionj(as.dist(aa.dist$distSTRING))
plot(aa.dist.bionj)
```

To use a different score matrix, here as an example the `AAMatrix` from the
R package [`alakazam`](https://cran.r-project.org/web/packages/ape/index.html)
is used:

```{r}
## use AAMatrix data
head(AAMatrix)
aa.dist.AAMatrix <- hiv |> cds2aa() |> aastring2dist(score=AAMatrix)
head(aa.dist.AAMatrix$distSTRING)
```

## Calculate pairwise DNA distances (`dnastring2dist()`)

### `ape::dist.dna` models

```{r}
## use hiv data
dna.dist <- hiv |> dnastring2dist(model="K80")
## obtain distances
head(dna.dist$distSTRING)
## obtain pairwise sites used
head(dna.dist$sitesUsed)
```

```{r}
## create and plot bionj tree
dna.dist.bionj <- ape::bionj(as.dist(dna.dist$distSTRING))
```

It is also possible to compare the amino acid and nucleotide based trees:

```{r}
## creation of the association matrix:
association <- cbind(aa.dist.bionj$tip.label, aa.dist.bionj$tip.label)
## cophyloplot
ape::cophyloplot(aa.dist.bionj,
    dna.dist.bionj,
    assoc=association,
    length.line=4,
    space=28,
    gap=3,
    rotate=FALSE)
```

### `IUPAC` distance 

```{r}
## use hiv data
hiv.dist.iupac <- head(hiv |> dnastring2dist(model="IUPAC"))
head(hiv.dist.iupac$distSTRING)
## run multi-threaded
system.time(hiv |> dnastring2dist(model="IUPAC", threads=1))
system.time(hiv |> dnastring2dist(model="IUPAC", threads=2))
```

Woodmouse data example:

```{r}
## use woodmouse data
woodmouse.dist <- woodmouse |> dnabin2dnastring() |> dnastring2dist()
head(woodmouse.dist$distSTRING)
```

# Coding sequences

## Calculating synonymous and nonsynonymous substitutions (`dnastring2kaks()`)

```{r}
## use hiv data
## model Li
head(hiv |> dnastring2kaks(model="Li"))
## model NG86
head(hiv |> dnastring2kaks(model="NG86", threads=1))
```

## Using any model from KaKs_Calculator 2.0

models to choose from KaKs_Calculator 2.0 [@wang2010kaks_calculator] are:

- MS [@wang2010kaks_calculator]
- MA [@wang2010kaks_calculator]
- GNG [@wang2010kaks_calculator]
- GLWL [@wang2010kaks_calculator]
- GLPB [@wang2010kaks_calculator]
- GMLWL [@wang2010kaks_calculator]
- GMLPB [@wang2010kaks_calculator]
- GYN [@wang2010kaks_calculator]
- LWL [@li1985new]
- LPB [@li1993unbiased, @pamilo1993evolution]
- NG [@nei1986simple]
- MLPB [@tzeng2004comparison]
- GY [@goldman1994codon]
- YN [@yang2000codon]
- MYN [@zhang2006computing, @wang2009gamma]

```{r}
## use hiv data
## model MYN
head(hiv |> dnastring2kaks(model="MYN"))
## model YN
head(hiv |> dnastring2kaks(model="YN", threads=1))
```

## Using indices to calculate Ka/Ks

```{r}
## use hiv data
idx <- list(c(2, 3), c(5,7,9))
## model MYN
head(hiv |> indices2kaks(idx, model="MYN"))
## model YN
head(hiv |> indices2kaks(idx, model="YN", threads=1))
```

## Codon comparison

As an example for the codon comparison data from the Human Immunodeficiency
Virus Type 1 is used [@ganeshan1997human], [@yang2000codon].

The window plots are constructed with the R package
[`ggplot2`](https://cran.r-project.org/web/packages/ggplot2/index.html).

### Create codon matrix (`dnastring2codonmat()`)

```{r}
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
## convert into alignment
cds1.cds2.aln |> dnastring2codonmat()
```

Like the `cds2aa()` function, also the `dnastring2codonmat()` function is
implemented to handle different frames.

```{r}
## use frame 2 and shorten to circumvent multiple of three error
cds1 <- Biostrings::DNAString("-ATGCAACATTGC-")
cds2 <- Biostrings::DNAString("-ATG---CATTGC-")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
cds1.cds2.aln |> dnastring2codonmat(frame=2, shorten=TRUE)
```

### Calculate average behavior of each codon (`codonmat2xy()`)

```{r}
## use hiv data
## calculate average behavior
hiv.xy <- hiv |> dnastring2codonmat() |> codonmat2xy()
```

### Plot average behavior of each codon

```{r}
print(hiv.xy |> dplyr::select(Codon,SynMean,NonSynMean,IndelMean) |>
    tidyr::gather(variable, values, -Codon) |>
    ggplot2::ggplot(aes(x=Codon, y=values)) + 
    ggplot2::geom_line(aes(colour=factor(variable))) + 
    ggplot2::geom_point(aes(colour=factor(variable))) + 
    ggplot2::ggtitle("HIV-1 sample 136 patient 1 from
        Sweden envelope glycoprotein (env) gene"))
```

# References

<div id="refs"></div>

# Session Info

```{r sessionInfo, echo=TRUE}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/codonmat2xy.R
\name{codonmat2xy}
\alias{codonmat2xy}
\title{codonmat2xy}
\usage{
codonmat2xy(codonmat, threads = 1)
}
\arguments{
\item{codonmat}{\code{codon matrix} obtained via
\code{\link[MSA2dist]{dnastring2codonmat}} [mandatory]}

\item{threads}{number of parallel threads [default: 1]}
}
\value{
A \code{data.frame} object with the following components:\cr
\code{Codon} Codon index\cr
\code{n} number of comparison\cr
\code{SynSum} Sum of syn\cr
\code{NonSynSum} Sum of nonsyn\cr
\code{IndelSum} Sum of indels\cr
\code{SynMean} average syn per codon\cr
\code{NonSynMean} average nonsyn per codon\cr
\code{IndelMean} average indels per codon\cr
\code{CumSumSynMean} cumulative average syn per codon\cr
\code{CumSumNonSynMean} cumulative average nonsyn per codon\cr
\code{CumSumIndelMean} cumulative indels per codon\cr
}
\description{
This function calculates average behavior of each codon for all
pairwise comparisons for indels, syn, and nonsyn mutations according to
\emph{Nei and Gojobori (1986)}.
}
\examples{
## load example sequence data
data("hiv", package="MSA2dist")
#codonmat2xy(dnastring2codonmat(hiv))
hiv |> dnastring2codonmat() |> codonmat2xy()
#codonmat2xy(dnastring2codonmat(hiv), threads=2)
hiv |> dnastring2codonmat() |> codonmat2xy(threads=2)
}
\references{
Nei and Gojobori. (1986) Simple methods for estimating the
numbers of synonymous and nonsynonymous nucleotide substitutions.
\emph{Mol. Biol. Evol.}, \bold{3(5)}, 418-426.

Ganeshan et al. (1997) Human immunodeficiency virus type 1
genetic evolution in children with different rates of development of disease.
\emph{J. Virology.} \bold{71(1)}, 663-677.

Yang et al. (2000) Codon-substitution models for heterogeneous
selection pressure at amino acid sites. \emph{Genetics.}
\bold{155(1)}, 431-449.
}
\seealso{
\code{\link[MSA2dist]{dnastring2codonmat}}
\code{\link[MSA2dist]{codonmat2pnps}}
\code{\link[MSA2dist]{dnastring2kaks}}
\code{\link[seqinr]{kaks}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aabin2aastring.R
\name{aabin2aastring}
\alias{aabin2aastring}
\title{aabin2aastring}
\usage{
aabin2aastring(aabin)
}
\arguments{
\item{aabin}{\code{ape} \code{AAbin} [mandatory]}
}
\value{
An object of class \code{AAStringSet}
}
\description{
This function converts an \code{ape} \code{AAbin} into
\code{AAStringSet}.
}
\examples{
data(woodmouse, package="ape")
## convert into AAStringSet
#aabin2aastring(ape::trans(woodmouse, 2))
ape::trans(woodmouse, 2) |> aabin2aastring()
}
\seealso{
\code{\link[seqinr]{as.alignment}}
\code{\link[ape]{as.DNAbin.alignment}}
\code{\link[Biostrings]{AAStringSet}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dnastring2codonmat.R
\name{dnastring2codonmat}
\alias{dnastring2codonmat}
\title{dnastring2codonmat}
\usage{
dnastring2codonmat(cds, shorten = FALSE, frame = 1, framelist = NULL)
}
\arguments{
\item{cds}{\code{DNAStringSet} [mandatory]}

\item{shorten}{shorten all sequences to multiple of three [default: FALSE]}

\item{frame}{indicates the first base of a the first codon [default: 1]}

\item{framelist}{supply vector of frames for each entry [default: NULL]}
}
\value{
An object of class \code{alignment} which is a list with the
following components:\cr
\code{nb} the number of aligned sequences\cr
\code{nam} a vector of strings containing the names of the aligned
sequences\cr
\code{seq} a vector of strings containing the aligned sequences\cr
\code{com} a vector of strings containing the commentaries for each sequence
or \code{NA} if there are no comments
}
\description{
This function converts a \code{DNAStringSet} into a
\code{codon matrix}.
}
\examples{
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
## convert into alignment
#dnastring2codonmat(cds1.cds2.aln)
cds1.cds2.aln |> dnastring2codonmat()
## use frame 2 and shorten to circumvent multiple of three error
cds1 <- Biostrings::DNAString("-ATGCAACATTGC-")
cds2 <- Biostrings::DNAString("-ATG---CATTGC-")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
cds1.cds2.aln |> dnastring2codonmat(frame=2, shorten=TRUE)
}
\seealso{
\code{\link[seqinr]{as.alignment}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cdsstring2codonaln.R
\name{cdsstring2codonaln}
\alias{cdsstring2codonaln}
\title{cdsstring2codonaln}
\usage{
cdsstring2codonaln(
  cds,
  aa,
  type = "global",
  substitutionMatrix = "BLOSUM62",
  gapOpening = 10,
  gapExtension = 0.5,
  remove.gaps = FALSE
)
}
\arguments{
\item{cds}{two sequences \code{DNAStringSet} [mandatory]}

\item{aa}{two sequences \code{AAStringSet} [mandatory]}

\item{type}{type of alignment (see
\code{\link[pwalign]{pairwiseAlignment}}) [default: global]}

\item{substitutionMatrix}{substitution matrix representing the fixed
substitution scores for an alignment (see
\code{\link[pwalign]{pairwiseAlignment}}) [default: BLOSUM62]}

\item{gapOpening}{the cost for opening a gap in the alignment (see
\code{\link[pwalign]{pairwiseAlignment}}) [default: 10]}

\item{gapExtension}{the incremental cost incurred along the length of the
gap in the alignment (see \code{\link[pwalign]{pairwiseAlignment}})
[default: 0.5]}

\item{remove.gaps}{specify if gaps in the codon alignment should be removed
[default: FALSE]}
}
\value{
codon alignment as \code{DNAStringSet}
}
\description{
This function takes two sequences as \code{DNAStringSet},
and their corresponding \code{AAStringSet}, calculates
a global alignment and converts this alignment back into a codon alignment.
}
\examples{
## define two cds sequences
cds <- Biostrings::DNAStringSet(c("ATGCAACATTGC", "ATGCATTGC"))
names(cds) <- c("cds1", "cds2")
## get protein alignment
aa <- MSA2dist::cds2aa(cds)
cdsstring2codonaln(cds, aa)
}
\references{
Pagès, H et al. (2014) Biostrings: Efficient manipulation of
biological strings. \emph{R package version}, \bold{2(0)}.
}
\seealso{
\code{\link[pwalign]{pairwiseAlignment}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dnabin2dnastring.R
\name{dnabin2dnastring}
\alias{dnabin2dnastring}
\title{dnabin2dnastring}
\usage{
dnabin2dnastring(dnabin)
}
\arguments{
\item{dnabin}{\code{ape} \code{DNAbin} [mandatory]}
}
\value{
An object of class \code{DNAStringSet}
}
\description{
This function converts an \code{ape} \code{DNAbin} into a
\code{DNAStringSet}.
}
\examples{
data(woodmouse, package="ape")
## convert into DNAStringSet
#dnabin2dnastring(woodmouse)
woodmouse |> dnabin2dnastring()
}
\seealso{
\code{\link[seqinr]{as.alignment}}
\code{\link[ape]{as.DNAbin.alignment}}
\code{\link[Biostrings]{DNAStringSet}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uptriidx.R
\name{uptriidx}
\alias{uptriidx}
\title{uptriidx}
\usage{
uptriidx(n, diag = FALSE)
}
\arguments{
\item{n}{dimension of initial matrix [mandatory]}

\item{diag}{indicate if diag should be retained [default: FALSE]}
}
\value{
list of positions
}
\description{
This function returns upper tri index for usage with
\code{pivot_long} reduction.
}
\examples{
uptriidx(10)
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iupac-data.R
\docType{data}
\name{iupac-data}
\alias{iupac-data}
\alias{iupac}
\title{iupac-data}
\format{
an object of class \code{DNAStringSet}
see \code{\link[Biostrings]{XStringSet-class}}
}
\usage{
data(iupac)
}
\description{
Example IUPAC sequences created with \code{angsd}
from different house mouse (Mus musculus) sub-populations
from Harr et al. (2016) \code{DNAStringSet}.
}
\examples{
data("iupac", package="MSA2dist")
}
\references{
Harr et al. (2016) Genomic resources for wild populations of the
house mouse, Mus musculus and its close relative
Mus spretus. \emph{Scientific data.} \bold{3(1)}, 1-14.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rcpp_pairwiseDeletionAA}
\alias{rcpp_pairwiseDeletionAA}
\title{rcpp_pairwiseDeletionAA}
\usage{
rcpp_pairwiseDeletionAA(aavector, ncores = 1L, symmetric = 1L)
}
\arguments{
\item{aavector}{StringVector [mandatory]}

\item{ncores}{number of cores [default: 1]}

\item{symmetric}{symmetric score matrix [default: 1]}
}
\value{
list
}
\description{
returns number of AA sites used
}
\examples{
## load example sequence data
data("hiv", package="MSA2dist")
h <- hiv |> cds2aa() |> as.character()
rcpp_pairwiseDeletionAA(aavector=h, ncores=1)
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aastring2aln.R
\name{aastring2aln}
\alias{aastring2aln}
\title{aastring2aln}
\usage{
aastring2aln(aa)
}
\arguments{
\item{aa}{\code{AAStringSet} [mandatory]}
}
\value{
An object of class \code{alignment} which is a list with the
following components:\cr
\code{nb} the number of aligned sequences\cr
\code{nam} a vector of strings containing the names of the aligned
sequences\cr
\code{seq} a vector of strings containing the aligned sequences\cr
\code{com} a vector of strings containing the commentaries for each sequence
or \code{NA} if there are no comments
}
\description{
This function converts a \code{AAStringSet} into an
\code{seqinr} \code{alignment}.
}
\examples{
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
#aastring2aln(cds2aa(cds1.cds2.aln))
cds1.cds2.aln |> cds2aa() |> aastring2aln()
}
\seealso{
\code{\link[seqinr]{as.alignment}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/popnames.R
\name{popnames}
\alias{popnames}
\title{popnames}
\usage{
popnames(seq)
}
\arguments{
\item{seq}{\code{DNAStringSet} or \code{AAStringSet} [mandatory]}
}
\value{
population names from \code{metadata}
}
\description{
This function shows the population names slot from a
\code{DNAStringSet} or an \code{AAStringSet}
\code{metadata} information.\cr
}
\examples{
## load example sequence data
data(iupac, package="MSA2dist")
iupac.aa <- iupac |> cds2aa(shorten = TRUE)
## create poplist
poplist <- list(FRA = grep("Mmd.FRA", names(iupac)),
    GER = grep("Mmd.GER", names(iupac)),
    IRA = grep("Mmd.IRA", names(iupac)),
    AFG = grep("Mmm.AFG", names(iupac)))
iupac.aa <- iupac.aa |> addpop2string(poplist)
popnames(iupac.aa)
}
\seealso{
\code{\link[MSA2dist]{addpop2string}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rcpp_KaKs}
\alias{rcpp_KaKs}
\title{rcpp_KaKs}
\usage{
rcpp_KaKs(cdsstr, sgc = "1", method = "YN", verbose = FALSE)
}
\arguments{
\item{cdsstr}{StringVector [mandatory]}

\item{sgc}{standard genetic code to use [default: 1]}

\item{method}{KaKs Calculator 2.0 codon model [default: YN]}

\item{verbose}{specify if verbose output [default: FALSE]}
}
\value{
\code{list}
}
\description{
calculates KaKs as implememted in
KaKs Calculator 2.0 \code{MSA2dist} with \code{Rcpp}.
}
\examples{
## load example sequence data
data("hiv", package="MSA2dist")
rcpp_KaKs(cdsstr=as.character(hiv[1:3]))
}
\references{
Wang et al. (2010) KaKs_Calculator 2.0: a toolkit incorporating
gamma-series methods and sliding window strategies.\emph{Genomics,
proteomics & bioinformatics.} \bold{8(1)}, 77-80.
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makePostalignedSeqs.R
\name{makePostalignedSeqs}
\alias{makePostalignedSeqs}
\title{makePostalignedSeqs}
\usage{
makePostalignedSeqs(x)
}
\arguments{
\item{x}{x}
}
\value{
get internal function makePostalignedSeqs
}
\description{
This function is a fork from an internal function from
\code{Biostrings}
}
\examples{
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATGCATTGC")
makePostalignedSeqs(pwalign::pairwiseAlignment(
    cds2aa(Biostrings::DNAStringSet(cds1)),
    cds2aa(Biostrings::DNAStringSet(cds2))))
}
\seealso{
\code{\link[pwalign]{pairwiseAlignment}},
\link[MSA2dist]{cds2codonaln}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aastring2aabin.R
\name{aastring2aabin}
\alias{aastring2aabin}
\title{aastring2aabin}
\usage{
aastring2aabin(aa)
}
\arguments{
\item{aa}{\code{AAStringSet} [mandatory]}
}
\value{
An object of class \code{AAbin}
}
\description{
This function converts an \code{AAStringSet} into an \code{ape}
\code{AAbin}.
}
\examples{
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
## convert into AAbin
#aastring2aabin(cds2aa(cds1.cds2.aln))
cds1.cds2.aln |> cds2aa() |> aastring2aabin()
}
\seealso{
\code{\link[seqinr]{as.alignment}}
\code{\link[ape]{as.DNAbin.alignment}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addpos2string.R
\name{addpos2string}
\alias{addpos2string}
\title{addpos2string}
\usage{
addpos2string(seq, chrom = NULL, start = NULL, end = NULL)
}
\arguments{
\item{seq}{\code{DNAStringSet} or \code{AAStringSet} [mandatory]}

\item{chrom}{chromosome name [mandatory]}

\item{start}{start position [mandatory]}

\item{end}{end position [mandatory]}
}
\value{
An object of class \code{DNAStringSet} or \code{AAStringSet}
}
\description{
This function adds \code{GenomicRanges} information,
\code{CHROM}, \code{START} and \code{END} to a
\code{DNAStringSet} or an \code{AAStringSet} and puts them into the
\code{metadata} information.
This information can be used to find overlaps with a chromosome wide mask.
}
\examples{
## load example sequence data
data(iupac, package="MSA2dist")
## add position
iupac <- iupac |> addpos2string(chrom="chr1", start=1, end=1000)
#(iupac |> slot("metadata"))$GRanges
iupac |> getpos()
}
\seealso{
\code{\link[MSA2dist]{addmask2string}},
\code{\link[MSA2dist]{addregion2string}},
\code{\link[MSA2dist]{addpop2string}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aln2aastring.R
\name{aln2aastring}
\alias{aln2aastring}
\title{aln2aastring}
\usage{
aln2aastring(aln)
}
\arguments{
\item{aln}{\code{seqinr} \code{alignment} [mandatory]}
}
\value{
An object of class \code{AAStringSet}
}
\description{
This function converts a \code{seqinr} \code{alignment} into
an \code{AAStringSet}.
}
\examples{
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
#aastring2aln(cds2aa(cds1.cds2.aln))
cds1.cds2.aln |> cds2aa() |> aastring2aln() |> aln2aastring()
}
\seealso{
\code{\link[seqinr]{as.alignment}}
\code{\link[Biostrings]{AAStringSet}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/string2region.R
\name{string2region}
\alias{string2region}
\title{string2region}
\usage{
string2region(seq, mask = NULL, region = NULL, add = TRUE)
}
\arguments{
\item{seq}{\code{DNAStringSet} or \code{AAStringSet} [mandatory]}

\item{mask}{\code{IRanges} object indicating masked sites [default: NULL]}

\item{region}{\code{IRanges} object indicating region to use for dist
calculation (by default all sites are used) [default: NULL]}

\item{add}{indicate if mask and region should be added to \code{metadata}
[default: TRUE]}
}
\value{
A \code{list} object with the following components:\cr
\code{DNAStringSet} or \code{AAStringSet}\cr
\code{regionUsed}\cr
}
\description{
This function subsets a \code{DNAStringSet} or an
\code{AAStringSet} by a \code{mask} and \code{region} given one or both
options as \code{IRanges}.
}
\examples{
## load example sequence data
data("hiv", package="MSA2dist")
## create mask
mask1 <- IRanges::IRanges(start=c(11,41,71), end=c(20,50,80))
## use mask
hiv.region <- hiv |> cds2aa() |> string2region(mask=mask1)
#(hiv.region |> slot("metadata"))$regionUsed
hiv.region |> regionused()
## use region
region1 <- IRanges::IRanges(start=c(1,75), end=c(45,85))
hiv.region <- hiv |> cds2aa() |> string2region(region=region1)
#(hiv.region |> slot("metadata"))$regionUsed
hiv.region |> regionused()
## use mask and region
hiv.region <- hiv |> cds2aa() |> string2region(mask=mask1, region=region1)
#(hiv.region |> slot("metadata"))$regionUsed
hiv.region |> regionused()
}
\seealso{
\code{\link[MSA2dist]{dnastring2dist}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getpos.R
\name{getpos}
\alias{getpos}
\title{getpos}
\usage{
getpos(seq)
}
\arguments{
\item{seq}{\code{DNAStringSet} or \code{AAStringSet} [mandatory]}
}
\value{
\code{GenomicRanges} information from \code{metadata}
}
\description{
This function shows the position slot from a
\code{DNAStringSet} or an \code{AAStringSet}
\code{metadata} information.\cr
}
\examples{
## load example sequence data
data(iupac, package="MSA2dist")
## add position
iupac <- iupac |> addpos2string(chrom="chr1", start=1, end=1000)
#(iupac |> slot("metadata"))$GRanges
iupac |> getpos()
}
\seealso{
\code{\link[MSA2dist]{addpop2string}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dnastring2dist.R
\name{dnastring2dist}
\alias{dnastring2dist}
\title{dnastring2dist}
\usage{
dnastring2dist(
  dna,
  model = "IUPAC",
  threads = 1,
  symmetric = TRUE,
  score = NULL,
  mask = NULL,
  region = NULL,
  ...
)
}
\arguments{
\item{dna}{\code{DNAStringSet} [mandatory]}

\item{model}{specify model either "IUPAC" or any model from
\code{ape::dist.dna} [default: IUPAC]}

\item{threads}{number of parallel threads [default: 1]}

\item{symmetric}{symmetric score matrix [default: TRUE]}

\item{score}{\code{score matrix} use score matrix to calculate
distances [default: NULL]}

\item{mask}{\code{IRanges} object indicating masked sites
[default: NULL]}

\item{region}{\code{IRanges} object indicating region to use for dist
calculation. Default is null, meaning all sites are used [default: NULL]}

\item{...}{other \code{ape::dist.dna} parameters
(see \code{\link[ape]{dist.dna}})}
}
\value{
A data.frame of pairwise distance values \code{distSTRING} and
sites used \code{sitesUsed}
}
\description{
This function calculates pairwise distances for all
combinations of a \code{DNAStringSet}.
}
\examples{
## load example sequence data
data("hiv", package="MSA2dist")
#dnastring2dist(hiv, model="IUPAC")
hiv |> dnastring2dist(model="IUPAC")
#dnastring2dist(hiv, model="K80")
hiv |> dnastring2dist(model="K80")
data("woodmouse", package="ape")
#dnastring2dist(dnabin2dnastring(woodmouse), score=iupacMatrix())
woodmouse |> dnabin2dnastring() |> dnastring2dist()
#dnastring2dist(hiv, model = "IUPAC", threads = 2)
hiv |> dnastring2dist(model = "IUPAC", threads = 2)
## create mask
mask1 <- IRanges::IRanges(start=c(1,61,121), end=c(30,90,150))
## use mask
hiv |> dnastring2dist(model="IUPAC", mask=mask1)
## use region
region1 <- IRanges::IRanges(start=c(1,139), end=c(75,225))
hiv |> dnastring2dist(model="IUPAC", region=region1)
## use mask and region
hiv |> dnastring2dist(model="IUPAC", mask=mask1, region=region1)
## use asymmetric score matrix
myscore <- iupacMatrix()
myscore[1, 4] <- 0.5
(hiv |> dnastring2dist(score=myscore, symmetric=FALSE))$distSTRING[1:2, 1:2]
}
\seealso{
\code{\link[ape]{dist.dna}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/globalDeletion.R
\name{globalDeletion}
\alias{globalDeletion}
\title{globalDeletion}
\usage{
globalDeletion(dna)
}
\arguments{
\item{dna}{\code{DNAStringSet} [mandatory]}
}
\value{
\code{DNAStringSet}
}
\description{
This function returns a \code{DNAStringSet} reduced by all
sites containing any gaps ("-", "+", ".") or missing ("N") sites.
}
\examples{
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
globalDeletion(cds1.cds2.aln)
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/region.R
\name{region}
\alias{region}
\title{region}
\usage{
region(seq)
}
\arguments{
\item{seq}{\code{DNAStringSet} or \code{AAStringSet} [mandatory]}
}
\value{
region \code{IRanges} object from \code{metadata}
}
\description{
This function shows the region slot from a
\code{DNAStringSet} or an \code{AAStringSet}
\code{metadata} information.\cr
}
\examples{
## load example sequence data
data(iupac, package="MSA2dist")
iupac.aa <- iupac |> cds2aa(shorten = TRUE)
## create region
region1 <- IRanges::IRanges(start=c(1,41), end=c(20,50))
## add region
iupac.aa <- iupac.aa |> addregion2string(region=region1)
iupac.aa |> region()
}
\seealso{
\code{\link[MSA2dist]{addpop2string}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/granthamMatrix.R
\name{granthamMatrix}
\alias{granthamMatrix}
\title{granthamMatrix}
\usage{
granthamMatrix()
}
\value{
\code{matrix}
}
\description{
This function creates a \code{granthamMatrix} object to be used
with the \code{rcpp_distSTRING} function. By default,the grantham matrix is
defined as from \code{Grantham 1974}.
(see \url{https://link.springer.com/article/10.1007/s00335-017-9704-9})
}
\examples{
granthamMatrix()
}
\references{
Grantham R. (1974). Amino Acid Difference Formula
to Help Explain Protein Evolution. \emph{Science},\bold{185(4154)},862-864.
}
\seealso{
\link[MSA2dist]{aastring2dist},\link[ape]{dist.dna}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AAMatrix-data.R
\docType{data}
\name{AAMatrix-data}
\alias{AAMatrix-data}
\alias{AAMatrix}
\title{AAMatrix-data}
\format{
an object of class \code{matrix}
}
\usage{
data(AAMatrix)
}
\value{
\code{score matrix}
}
\description{
\code{getAAMatrix()}  from the \code{alakazam} package.
}
\examples{
data("AAMatrix", package="MSA2dist")
}
\references{
Gupta N, Vander Heiden J, Uduman M, Gadala-Maria D, Yaari G,
Kleinstein S (2015) Change-O: a toolkit for analyzing large-scale B cell
immunoglobulin repertoire sequencing data. \emph{Bioinformatics.}
\bold{31(20)}, 3356-3358.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rcpp_distSTRING}
\alias{rcpp_distSTRING}
\title{rcpp_distSTRING}
\usage{
rcpp_distSTRING(dnavector, scoreMatrix, ncores = 1L, symmetric = 1L)
}
\arguments{
\item{dnavector}{StringVector [mandatory]}

\item{scoreMatrix}{NumericMatrix [mandatory]}

\item{ncores}{number of cores [default: 1]}

\item{symmetric}{symmetric score matrix [default: 1]}
}
\value{
list
}
\description{
calculates pairwise distances using a score matrix
}
\examples{
## load example sequence data
data("hiv", package="MSA2dist")
rcpp_distSTRING(dnavector=as.character(hiv), scoreMatrix=iupacMatrix())
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/codon2numberAMBIG.R
\name{codon2numberAMBIG}
\alias{codon2numberAMBIG}
\title{codon2numberAMBIG}
\usage{
codon2numberAMBIG(codon)
}
\arguments{
\item{codon}{[mandatory]}
}
\value{
An object of class \code{numeric}
}
\description{
This function converts a \code{codon} into a \code{number},
but accept N and -.
}
\examples{
#unlist(lapply(names(Biostrings::GENETIC_CODE), codon2numberAMBIG))
names(Biostrings::GENETIC_CODE) |> codon2numberAMBIG()
}
\seealso{
\code{\link[Biostrings]{GENETIC_CODE}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aln2dnastring.R
\name{aln2dnastring}
\alias{aln2dnastring}
\title{aln2dnastring}
\usage{
aln2dnastring(aln)
}
\arguments{
\item{aln}{\code{seqinr} \code{alignment} [mandatory]}
}
\value{
An object of class \code{DNAStringSet}
}
\description{
This function converts a \code{seqinr} \code{alignment} into
an \code{DNAStringSet}.
}
\examples{
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
## convert into alignment
#dnastring2aln(cds1.cds2.aln)
cds1.cds2.aln |> dnastring2aln()
## convert back into DNAStringSet
#aln2dnastring(dnastring2aln(cds1.cds2.aln))
cds1.cds2.aln |> dnastring2aln() |> aln2dnastring()
}
\seealso{
\code{\link[seqinr]{as.alignment}}
\code{\link[Biostrings]{DNAStringSet}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subString.R
\name{subString}
\alias{subString}
\title{subString}
\usage{
subString(x, s, e)
}
\arguments{
\item{x}{\code{DNAStringSet}, \code{RNAString}, \code{AAString},
\code{BString}, \code{DNAStringSet}, \code{RNAStringSet},
\code{AAStringSet}, \code{BStringSet} [mandatory]}

\item{s}{start vector [mandatory]}

\item{e}{end vector [mandatory]}
}
\value{
subsequence of an \code{Biostrings} object
}
\description{
This function gets a subsequence from a \code{DNAString},
\code{RNAString}, \code{AAString}, \code{BString}, \code{DNAStringSet},
\code{RNAStringSet}, \code{AAStringSet}, \code{BStringSet}
object from the \code{Biostrings} package.
}
\examples{
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
subString(cds1.cds2.aln, c(1,7), c(3,12))
}
\seealso{
\link[XVector]{subseq}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iupacMatrix.R
\name{iupacMatrix}
\alias{iupacMatrix}
\title{iupacMatrix}
\usage{
iupacMatrix()
}
\value{
\code{score matrix}
}
\description{
This function creates a \code{iupacMatrix} object to be used
with the \code{rcpp_distSTRING} function. By default, the \code{iupac matrix}
is defined as literal distance obtained from \code{Chang et al. 2017}.
(see \url{https://link.springer.com/article/10.1007/s00335-017-9704-9})
}
\examples{
iupacMatrix()
}
\references{
Chang,P. L.,Kopania,E.,Keeble,S.,Sarver,B. A.,Larson,
E.,Orth,A.,... & Dean,M. D. (2017). Whole exome sequencing of
wild-derived inbred strains of mice improves power to link phenotype and
genotype. \emph{Mammalian genome},\bold{28(9-10)},416-425.
}
\seealso{
\link[MSA2dist]{dnastring2dist},\link[ape]{dist.dna}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regionused.R
\name{regionused}
\alias{regionused}
\title{regionused}
\usage{
regionused(seq)
}
\arguments{
\item{seq}{\code{DNAStringSet} or \code{AAStringSet} [mandatory]}
}
\value{
population names from \code{metadata}
}
\description{
This function shows the region used slot from a
\code{DNAStringSet} or an \code{AAStringSet}
\code{metadata} information.\cr
}
\examples{
## load example sequence data
data("hiv", package="MSA2dist")
## create mask
mask1 <- IRanges::IRanges(start=c(11,41,71), end=c(20,50,80))
## use mask
hiv.region <- hiv |> cds2aa() |> string2region(mask=mask1)
#(hiv.region |> slot("metadata"))$regionUsed
hiv.region |> regionused()
}
\seealso{
\code{\link[MSA2dist]{addpop2string}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/codon2numberTCAG.R
\name{codon2numberTCAG}
\alias{codon2numberTCAG}
\title{codon2numberTCAG}
\usage{
codon2numberTCAG(codon)
}
\arguments{
\item{codon}{[mandatory]}
}
\value{
An object of class \code{numeric}
}
\description{
This function converts a \code{codon} into a \code{number}.
}
\examples{
#unlist(lapply(names(Biostrings::GENETIC_CODE), codon2numberTCAG))
names(Biostrings::GENETIC_CODE) |> codon2numberTCAG()
}
\seealso{
\code{\link[Biostrings]{GENETIC_CODE}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aastring2dist.R
\name{aastring2dist}
\alias{aastring2dist}
\title{aastring2dist}
\usage{
aastring2dist(
  aa,
  threads = 1,
  symmetric = TRUE,
  score = NULL,
  mask = NULL,
  region = NULL
)
}
\arguments{
\item{aa}{\code{AAStringSet} [mandatory]}

\item{threads}{number of parallel threads [default: 1]}

\item{symmetric}{symmetric score matrix [default: TRUE]}

\item{score}{\code{score matrix} use a score matrix to calculate distances
[mandatory]}

\item{mask}{\code{IRanges} object indicating masked sites
[default: NULL]}

\item{region}{\code{IRanges} object indicating region to use for dist
calculation (by default all sites are used) [default: NULL]}
}
\value{
A \code{data.frame} of pairwise distance values
\code{distSTRING}, sites used \code{sitesUsed} and region used
\code{regionUsed}
}
\description{
This function calculates pairwise distances for all combinations
of a \code{AAStringSet}.
}
\examples{
## load example sequence data
data("hiv", package="MSA2dist")
#aastring2dist(cds2aa(hiv), score=granthamMatrix())
hiv |> cds2aa() |> aastring2dist(score=granthamMatrix())
## create mask
mask1 <- IRanges::IRanges(start=c(11,41,71), end=c(20,50,80))
## use mask
hiv |> cds2aa() |> aastring2dist(score=granthamMatrix(), mask=mask1)
## use region
region1 <- IRanges::IRanges(start=c(1,75), end=c(45,85))
hiv |> cds2aa() |> aastring2dist(score=granthamMatrix(), region=region1)
## use mask and region
hiv |> cds2aa() |> aastring2dist(score=granthamMatrix(),
    mask=mask1, region=region1)
## use asymmetric score matrix
myscore <- granthamMatrix()
myscore[5, 6] <- 0
h <- hiv |> cds2aa() |> aastring2dist(score=myscore, symmetric=FALSE)
h$distSTRING[1:2, 1:2]
}
\seealso{
\code{\link[MSA2dist]{dnastring2dist}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addpop2string.R
\name{addpop2string}
\alias{addpop2string}
\title{addpop2string}
\usage{
addpop2string(seq, poplist)
}
\arguments{
\item{seq}{\code{DNAStringSet} or \code{AAStringSet} [mandatory]}

\item{poplist}{named \code{list} of populations either as index or names per
population (do not mix index and names in one population) [mandatory]}
}
\value{
An object of class \code{DNAStringSet} or \code{AAStringSet}
}
\description{
This function adds population information to a
\code{DNAStringSet} or an \code{AAStringSet} and puts them into the
\code{metadata} information.\cr
__Note__: All unassigned sequences will be put into pop "unassigned"!\cr
Do not use "unassigned" as a population name!\cr
__Note__: Names in a population in the poplist must match sequence names!\cr
__Note__: Duplicated assignments are allowed!\cr
}
\examples{
## load example sequence data
data(iupac, package="MSA2dist")
iupac.aa <- iupac |> cds2aa(shorten = TRUE)
## create poplist
poplist <- list(FRA = grep("Mmd.FRA", names(iupac)),
    GER = grep("Mmd.GER", names(iupac)),
    IRA = grep("Mmd.IRA", names(iupac)),
    AFG = grep("Mmm.AFG", names(iupac)))
iupac.aa <- iupac.aa |> addpop2string(poplist)
#(iupac.aa |> slot("metadata"))$pop.integer
iupac.aa |> popinteger()
#(iupac.aa |> slot("metadata"))$pop.names
iupac.aa |> popnames()
## mxixing index and names
poplist <- list(FRA = names(iupac)[grep("Mmd.FRA", names(iupac))],
    GER = grep("Mmd.GER", names(iupac)),
    IRA = names(iupac)[grep("Mmd.IRA", names(iupac))],
    AFG = grep("Mmm.AFG", names(iupac)))
iupac.aa <- iupac.aa |> addpop2string(poplist)
iupac.aa |> popinteger()
iupac.aa |> popnames()
## leaving out some sequences which will be assigned as "unassigned"
poplist <- list(FRA = names(iupac)[grep("Mmd.FRA", names(iupac))],
    GER = grep("Mmd.GER", names(iupac)),
    IRA = names(iupac)[grep("Mmd.IRA", names(iupac))])
iupac.aa <- iupac.aa |> addpop2string(poplist)
iupac.aa |> popinteger()
iupac.aa |> popnames()
}
\seealso{
\code{\link[MSA2dist]{addmask2string}},
\code{\link[MSA2dist]{addregion2string}},
\code{\link[MSA2dist]{addpos2string}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aa2selfscore.R
\name{aa2selfscore}
\alias{aa2selfscore}
\title{aa2selfscore}
\usage{
aa2selfscore(aa, scorematrix = "BLOSUM62")
}
\arguments{
\item{aa}{\code{AAStringSet} [mandatory]}

\item{scorematrix}{score matrix to use [default: BLOSUM62]}
}
\value{
\code{data.frame}
}
\description{
This function return the selfscore from an \code{AAStringSet}.
}
\examples{
data(woodmouse, package="ape")
#cds2aa(dnabin2dnastring(woodmouse), shorten=TRUE,
#genetic.code=Biostrings::getGeneticCode("2"))
woodmouse |> dnabin2dnastring() |> cds2aa(shorten=TRUE,
genetic.code=Biostrings::getGeneticCode("2")) |> aa2selfscore()
}
\seealso{
\code{\link[Biostrings]{XStringSet-class}},
\code{\link[pwalign]{substitution_matrices}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compareCodons.R
\name{compareCodons}
\alias{compareCodons}
\title{compareCodons}
\usage{
compareCodons(codA, codB)
}
\arguments{
\item{codA}{\code{codon} A [mandatory]}

\item{codB}{\code{codon} B [mandatory]}
}
\value{
vector of syn and non-syn sites
}
\description{
This function compares two codons and returns the number of syn
and non-syn sites according to \emph{Nei and Gojobori (1986)}.
}
\examples{
compareCodons("AAA","TTA")
compareCodons("AAA","TAT")
compareCodons("AAA","ATT")
compareCodons("AAA","TTT")
## load example sequence data
data("hiv", package="MSA2dist")
compareCodons(dnastring2codonmat(hiv)[1,1], dnastring2codonmat(hiv)[1,2])
}
\references{
Nei and Gojobori. (1986) Simple methods for estimating the
numbers of synonymous and nonsynonymous nucleotide substitutions.
\emph{Mol. Biol. Evol.}, \bold{3(5)}, 418-426.

Ganeshan et al. (1997) Human immunodeficiency virus type 1
genetic evolution in children with different rates of development of disease.
\emph{J. Virology.} \bold{71(1)}, 663-677.

Yang et al. (2000) Codon-substitution models for heterogeneous
selection pressure at amino acid sites. \emph{Genetics.}
\bold{155(1)}, 431-449.
}
\seealso{
\code{\link[seqinr]{kaks}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addregion2string.R
\name{addregion2string}
\alias{addregion2string}
\title{addregion2string}
\usage{
addregion2string(seq, region = NULL, append = TRUE)
}
\arguments{
\item{seq}{\code{DNAStringSet} or \code{AAStringSet} [mandatory]}

\item{region}{\code{IRanges} object [mandatory]}

\item{append}{indicate if region should be appended or overwritten
[default: TRUE]}
}
\value{
An object of class \code{DNAStringSet} or \code{AAStringSet}
}
\description{
This function adds region information as an \code{IRanges}
object, \code{START} and \code{END} information, to a
\code{DNAStringSet} or an \code{AAStringSet} and puts them into the
\code{metadata} information.
This information can be used to restrict the distance calculation to
specific regions of the \code{DNAStringSet} or the \code{AAStringSet}.
}
\examples{
## load example sequence data
data(iupac, package="MSA2dist")
iupac.aa <- iupac |> cds2aa(shorten = TRUE)
## create region
region1 <- IRanges::IRanges(start=c(1,41), end=c(20,50))
## add region
iupac.aa <- iupac.aa |> addregion2string(region=region1)
#(iupac.aa |> slot("metadata"))$region
iupac.aa |> region()
## append region
region2 <- IRanges::IRanges(start=c(21), end=c(30))
iupac.aa <- iupac.aa |> addregion2string(region=region2)
#(iupac.aa |> slot("metadata"))$region
iupac.aa |> region()
## overwrite region
iupac.aa <- iupac.aa |> addregion2string(region=region2, append=FALSE)
#(iupac.aa |> slot("metadata"))$region
iupac.aa |> region()
## reduce by region
#iupac.aa.region <- iupac.aa |> string2region(region=
#    (iupac.aa |> slot("metadata"))$region)
iupac.aa.region <- iupac.aa |> string2region(region=
    region(iupac.aa))
#iupac.aa.region |> slot("metadata")
iupac.aa.region |> region()
}
\seealso{
\code{\link[MSA2dist]{addmask2string}},
\code{\link[MSA2dist]{addpop2string}},
\code{\link[MSA2dist]{addpos2string}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addmask2string.R
\name{addmask2string}
\alias{addmask2string}
\title{addmask2string}
\usage{
addmask2string(seq, mask = NULL, append = TRUE)
}
\arguments{
\item{seq}{\code{DNAStringSet} or \code{AAStringSet} [mandatory]}

\item{mask}{\code{IRanges} object [mandatory]}

\item{append}{indicate if mask should be appended or overwritten
[default: TRUE]}
}
\value{
An object of class \code{DNAStringSet} or \code{AAStringSet}
}
\description{
This function adds mask information as an \code{IRanges} object,
\code{START} and \code{END} information, to a
\code{DNAStringSet} or an \code{AAStringSet} and puts them into the
\code{metadata} information.
This information can be used to restrict the distance calculation to
specific regions of the \code{DNAStringSet} or the \code{AAStringSet}.
}
\examples{
## load example sequence data
data(iupac, package="MSA2dist")
iupac.aa <- iupac |> cds2aa(shorten = TRUE)
## create mask
mask1 <- IRanges::IRanges(start=c(1,41), end=c(20,50))
## add mask
iupac.aa <- iupac.aa |> addmask2string(mask=mask1)
#(iupac.aa |> slot("metadata"))$mask
iupac.aa |> getmask()
## append mask
mask2 <- IRanges::IRanges(start=c(21), end=c(30))
iupac.aa <- iupac.aa |> addmask2string(mask=mask2)
#(iupac.aa |> slot("metadata"))$mask
iupac.aa |> getmask()
## overwrite mask
iupac.aa <- iupac.aa |> addmask2string(mask=mask2, append=FALSE)
#(iupac.aa |> slot("metadata"))$mask
iupac.aa |> getmask()
## reduce by mask
#iupac.aa.region <- iupac.aa |> string2region(mask=
#    (iupac.aa |> slot("metadata"))$mask)
iupac.aa.region <- iupac.aa |> string2region(mask=
    getmask(iupac.aa))
#iupac.aa.region |> slot("metadata")
iupac.aa.region |> getmask()
}
\seealso{
\code{\link[MSA2dist]{addregion2string}},
\code{\link[MSA2dist]{addpop2string}},
\code{\link[MSA2dist]{addpos2string}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cds2aa.R
\name{cds2aa}
\alias{cds2aa}
\title{cds2aa}
\usage{
cds2aa(
  cds,
  shorten = FALSE,
  frame = 1,
  framelist = NULL,
  genetic.code = NULL,
  return.cds = FALSE
)
}
\arguments{
\item{cds}{\code{DNAStringSet} [mandatory]}

\item{shorten}{shorten all sequences to multiple of three [default: FALSE]}

\item{frame}{indicates the first base of a the first codon [default: 1]}

\item{framelist}{supply vector of frames for each entry [default: NULL]}

\item{genetic.code}{The genetic code to use for the translation of codons
into Amino Acid letters [default: NULL]}

\item{return.cds}{return shorten cds instead of aa [default: FALSE]}
}
\value{
\code{AAStringSet}
}
\description{
This function translates a \code{DNAStringSet} into an
\code{AAStringSet}.
}
\examples{
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
#cds2aa(cds1.cds2.aln)
cds1.cds2.aln |> cds2aa()
## alternative genetic code
data(woodmouse, package="ape")
#cds2aa(dnabin2dnastring(woodmouse), shorten=TRUE)
woodmouse |> dnabin2dnastring() |> cds2aa(shorten=TRUE)
#cds2aa(dnabin2dnastring(woodmouse), shorten=TRUE,
#genetic.code=Biostrings::getGeneticCode("2"))
woodmouse |> dnabin2dnastring() |> cds2aa(shorten=TRUE,
genetic.code=Biostrings::getGeneticCode("2"))
woodmouse |> dnabin2dnastring() |> cds2aa(shorten=TRUE, return.cds=TRUE) |>
cds2aa(genetic.code=Biostrings::getGeneticCode("2"))
}
\seealso{
\code{\link[Biostrings]{XStringSet-class}},
\code{\link[seqinr]{translate}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GENETIC_CODE_TCAG.R
\name{GENETIC_CODE_TCAG}
\alias{GENETIC_CODE_TCAG}
\alias{codon2number}
\title{GENETIC_CODE_TCAG}
\usage{
codon2number(codon)
}
\arguments{
\item{codon}{\code{codon} [mandatory]}
}
\value{
An object of class \code{numeric}
}
\description{
\code{GENETIC_CODE} from \code{Biostrings} extended by codon
number and number of syn sites.
}
\examples{
GENETIC_CODE_TCAG
}
\seealso{
\code{\link[Biostrings]{GENETIC_CODE}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hiv-data.R
\docType{data}
\name{hiv-data}
\alias{hiv-data}
\alias{hiv}
\title{hiv-data}
\format{
an object of class \code{DNAStringSet}
see \code{\link[Biostrings]{XStringSet-class}}
}
\usage{
data(hiv)
}
\description{
Example cds sequences from HIV-1 sample 136 patient 1 from
Sweden envelope glycoprotein (env) gene, V3 region as \code{DNAStringSet}.
}
\examples{
data("hiv", package="MSA2dist")
}
\references{
Yang et al. (2000) Codon-substitution models for heterogeneous
selection pressure at amino acid sites. \emph{Genetics.}
\bold{155(1)}, 431-449.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getmask.R
\name{getmask}
\alias{getmask}
\title{getmask}
\usage{
getmask(seq)
}
\arguments{
\item{seq}{\code{DNAStringSet} or \code{AAStringSet} [mandatory]}
}
\value{
\code{IRanges} information from \code{metadata}
}
\description{
This function shows the mask slot from a
\code{DNAStringSet} or an \code{AAStringSet}
\code{metadata} information.\cr
}
\examples{
## load example sequence data
data(iupac, package="MSA2dist")
iupac.aa <- iupac |> cds2aa(shorten = TRUE)
## create mask
mask1 <- IRanges::IRanges(start=c(1,41), end=c(20,50))
## add mask
iupac.aa <- iupac.aa |> addmask2string(mask=mask1)
#(iupac.aa |> slot("metadata"))$mask
iupac.aa |> getmask()
}
\seealso{
\code{\link[MSA2dist]{addpop2string}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pal2nal.R
\name{pal2nal}
\alias{pal2nal}
\title{pal2nal}
\usage{
pal2nal(pal, nal, remove.gaps = FALSE)
}
\arguments{
\item{pal}{\code{AAStringSet} [mandatory]}

\item{nal}{\code{DNAStringSet} [mandatory]}

\item{remove.gaps}{specify if gaps in the codon alignment should be removed
[default: FALSE]}
}
\value{
codon alignment as \code{DNAStringSet}
}
\description{
This function takes an \code{AAStringSet} alignment and
its corresponding coding sequences \code{DNAStringSet} and converts
the protein alignment into a codon alignment.
}
\examples{
## define two cds sequences
cds <- Biostrings::DNAStringSet(c("ATGCAACATTGC", "ATGCATTGC"))
names(cds) <- c("cds1", "cds2")
## get protein alignment
aa <- MSA2dist::cds2aa(cds)
msa <- makePostalignedSeqs(pwalign::pairwiseAlignment(aa[1], aa[2]))[[1L]]
names(msa) <- names(aa)
## get codon alignment
nal <- MSA2dist::pal2nal(pal=msa, nal=cds)
nal
}
\references{
Pagès, H et al. (2014) Biostrings: Efficient manipulation of
biological strings. \emph{R package version}, \bold{2(0)}.
}
\seealso{
\code{\link[pwalign]{pairwiseAlignment}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/codonmat2pnps.R
\name{codonmat2pnps}
\alias{codonmat2pnps}
\title{codonmat2pnps}
\usage{
codonmat2pnps(codonmat)
}
\arguments{
\item{codonmat}{\code{codon matrix} of two columns to be
compared [mandatory]}
}
\value{
An object of class \code{pnps} which is a list with the following
components:\cr
\code{seq1} sequence1 name\cr
\code{seq2} sequence2 name\cr
\code{Codons} sequence2 name\cr
\code{Compared} sequence2 name\cr
\code{Ambigiuous} sequence2 name\cr
\code{Indels} sequence2 name\cr
\code{Ns} sequence2 name\cr
\code{Sd} sequence2 name\cr
\code{Sn} sequence2 name\cr
\code{S} sequence2 name\cr
\code{N} sequence2 name\cr
\code{ps} sequence2 name\cr
\code{pn} sequence2 name\cr
\code{pnps} sequence2 name\cr
\code{ds} sequence2 name\cr
\code{dn} sequence2 name\cr
\code{dnds} sequence2 name\cr
}
\description{
This function calculates pn/ps according to \emph{Nei and
Gojobori (1986)}.
}
\examples{
## load example sequence data
data("hiv", package="MSA2dist")
#codonmat2pnps(dnastring2codonmat(hiv)[,c(1, 2)])
(hiv |> dnastring2codonmat())[,c(1, 2)] |> codonmat2pnps()
}
\references{
Nei and Gojobori. (1986) Simple methods for estimating the
numbers of synonymous and nonsynonymous nucleotide substitutions.
\emph{Mol. Biol. Evol.}, \bold{3(5)}, 418-426.

Ganeshan et al. (1997) Human immunodeficiency virus type 1
genetic evolution in children with different rates of development of disease.
\emph{J. Virology.} \bold{71(1)}, 663-677.

Yang et al. (2000) Codon-substitution models for heterogeneous
selection pressure at amino acid sites. \emph{Genetics.}
\bold{155(1)}, 431-449.
}
\seealso{
\code{\link[seqinr]{kaks}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/globalDeletionAA.R
\name{globalDeletionAA}
\alias{globalDeletionAA}
\title{globalDeletionAA}
\usage{
globalDeletionAA(aa)
}
\arguments{
\item{aa}{\code{AAStringSet} [mandatory]}
}
\value{
\code{AAStringSet}
}
\description{
This function returns an \code{AAStringSet} reduced by all
sites containing any gaps ("-", "+", ".") or missing ("X") sites.
}
\examples{
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
#globalDeletionAA(cds2aa(cds1.cds2.aln))
cds1.cds2.aln |> cds2aa() |> globalDeletionAA()
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dnastring2dnabin.R
\name{dnastring2dnabin}
\alias{dnastring2dnabin}
\title{dnastring2dnabin}
\usage{
dnastring2dnabin(dna)
}
\arguments{
\item{dna}{\code{DNAStringSet} [mandatory]}
}
\value{
An object of class \code{DNAbin}
}
\description{
This function converts a \code{DNAStringSet} into an
\code{ape} \code{DNAbin}.
}
\examples{
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATG---CATTGC")
cds1.cds2.aln <- c(Biostrings::DNAStringSet(cds1),
    Biostrings::DNAStringSet(cds2))
## convert into DNAbin
#dnastring2dnabin(cds1.cds2.aln)
cds1.cds2.aln |> dnastring2dnabin()
}
\seealso{
\code{\link[seqinr]{as.alignment}}
\code{\link[ape]{as.DNAbin.alignment}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rcpp_pairwiseDeletionDNA}
\alias{rcpp_pairwiseDeletionDNA}
\title{rcpp_pairwiseDeletionDNA}
\usage{
rcpp_pairwiseDeletionDNA(dnavector, ncores = 1L, symmetric = 1L)
}
\arguments{
\item{dnavector}{StringVector [mandatory]}

\item{ncores}{number of cores [default: 1]}

\item{symmetric}{symmetric score matrix [default: 1]}
}
\value{
list
}
\description{
returns number of DNA sites used
}
\examples{
## load example sequence data
data("woodmouse", package="ape")
w <- woodmouse |> dnabin2dnastring() |> as.character()
rcpp_pairwiseDeletionDNA(dnavector=w, ncores=1)
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cds2codonaln.R
\name{cds2codonaln}
\alias{cds2codonaln}
\title{cds2codonaln}
\usage{
cds2codonaln(
  cds1,
  cds2,
  type = "global",
  substitutionMatrix = "BLOSUM62",
  gapOpening = 10,
  gapExtension = 0.5,
  remove.gaps = FALSE,
  ...
)
}
\arguments{
\item{cds1}{single sequence \code{DNAStringSet} or \code{DNAString}
[mandatory]}

\item{cds2}{single sequence \code{DNAStringSet} or \code{DNAString}
[mandatory]}

\item{type}{type of alignment (see
\code{\link[pwalign]{pairwiseAlignment}}) [default: global]}

\item{substitutionMatrix}{substitution matrix representing the fixed
substitution scores for an alignment (see
\code{\link[pwalign]{pairwiseAlignment}}) [default: BLOSUM62]}

\item{gapOpening}{the cost for opening a gap in the alignment (see
\code{\link[pwalign]{pairwiseAlignment}}) [default: 10]}

\item{gapExtension}{the incremental cost incurred along the length of the
gap in the alignment (see \code{\link[pwalign]{pairwiseAlignment}})
[default: 0.5]}

\item{remove.gaps}{specify if gaps in the codon alignment should be removed
[default: FALSE]}

\item{...}{other cds2aa parameters}
}
\value{
codon alignment as \code{DNAStringSet}
}
\description{
This function takes two single sequence \code{DNAString}'s or
two single sequence \code{DNAStringSet}'s, converts them into aa, calculates
a global alignment and converts this alignment back into a codon alignment.
}
\examples{
## define two cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATGCATTGC")
cds2codonaln(cds1, cds2)
}
\references{
Pagès, H et al. (2014) Biostrings: Efficient manipulation of
biological strings. \emph{R package version}, \bold{2(0)}.
}
\seealso{
\code{\link[pwalign]{pairwiseAlignment}}
}
\author{
Kristian K Ullrich
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dnastring2kaks.R
\name{dnastring2kaks}
\alias{dnastring2kaks}
\title{dnastring2kaks}
\usage{
dnastring2kaks(
  cds,
  model = "Li",
  threads = 1,
  isMSA = TRUE,
  sgc = "1",
  verbose = FALSE,
  ...
)
}
\arguments{
\item{cds}{\code{DNAStringSet} coding sequence alignment [mandatory]}

\item{model}{specify codon model either "Li" or "NG86" or
one of \code{KaKs_Calculator2} model "NG", "LWL", "LPB", "MLWL", "MLPB",
"GY", "YN", "MYN", "MS", "MA", "GNG", "GLWL", "GLPB", "GMLWL", "GMLPB",
"GYN", "GMYN" [default: Li]}

\item{threads}{number of parallel threads [default: 1]}

\item{isMSA}{cds \code{DNAStringSet} represents MSA [default: TRUE]}

\item{sgc}{standard genetic code (for KaKs Calculator models)
[default: 1]}

\item{verbose}{verbosity (for KaKs Calculator models) [default: FALSE]}

\item{...}{other codon alignment parameters}
}
\value{
A \code{data.frame} of \code{KaKs} values
}
\description{
This function calculates Ka/Ks (pN/pS)
for all combinations of a \code{DNAStringSet}.
If the sequences in the \code{DNAStringSet} are not a multiple-sequence
alignment, pairwise codon alignments can be calculated on the fly.
Models used and implemented according to
\emph{Li (1993)} (using \code{seqinr}) or
\emph{Nei and Gojobori (1986)} (own implementation) or models from
\code{KaKs_Calculator2} ported to \code{MSA2dist} with \code{Rcpp}.
}
\examples{
## load example sequence data
data("hiv", package="MSA2dist")
#dnastring2kaks(hiv, model="Li")
hiv |> dnastring2kaks(model="Li")
#dnastring2kaks(hiv, model="NG86")
hiv |> dnastring2kaks(model="NG86")
#dnastring2kaks(hiv, model="NG86", threads=2)
hiv |> dnastring2kaks(model="NG86", threads=2)

## define three unaligned cds sequences
cds1 <- Biostrings::DNAString("ATGCAACATTGC")
cds2 <- Biostrings::DNAString("ATGCATTGC")
cds3 <- Biostrings::DNAString("ATGCAATGC")
cds_sequences <- Biostrings::DNAStringSet(list(cds1, cds2, cds3))
names(cds_sequences) <- c("cds1", "cds2", "cds3")
## set isMSA to FALSE to automatically create pairwise codon alignments
#dnastring2kaks(cds_sequences, model="Li", isMSA=FALSE)
cds_sequences |> dnastring2kaks(model="Li", isMSA=FALSE)
}
\references{
"MS/MA/GNG/GLWL/GLPB/GMLWL/GMLPB/GYN:" Wang et al. (2010)
KaKs_Calculator 2.0: a toolkit incorporating
gamma-series methods and sliding window strategies.\emph{Genomics,
proteomics & bioinformatics.} \bold{8(1)}, 77-80.

"Li/LWL:" Li et al. (1985) A new method for estimating synonymous
and nonsynonymous rates of nucleotide substitution considering the relative
likelihood of nucleotide and codon changes. \emph{Mol. Biol. Evol.},
\bold{2(2)}, 150-174.

"Li/LPB:" Li (1993). Unbiased estimation of the rates of
synonymous and nonsynonymous substitution. Journal of molecular evolution,
36(1), pp.96-99.

"NG86/NG:" Nei and Gojobori. (1986) Simple methods for estimating
the numbers of synonymous and nonsynonymous nucleotide substitutions.
\emph{Mol. Biol. Evol.}, \bold{3(5)}, 418-426.

"LPB:" Pamilo and Bianchi. (1993) Evolution of the Zfx and Zfy
genes: Rates and interdependence between genes. \emph{Mol. Biol. Evol.},
\bold{10}, 271-281.

"MLWL/MLPB:" Tzeng et al. (2004). Comparison of three methods for
estimating rates of synonymous and nonsynonymous nucleotide substitutions.
\emph{Mol. Biol. Evol.}, \bold{21(12)}, 2290-2298.

"GY:" Goldman and Yang (1994). A codon-based model of nucleotide
substitution for protein-coding DNA sequences. \emph{Mol. Biol. Evol.},
\bold{11(5)} 725-736.

"YN:" Yang et al. (2000) Codon-substitution models for
heterogeneous selection pressure at amino acid sites. \emph{Genetics.}
\bold{155(1)}, 431-449.

"MYN:" Zhang et al. (2006). Computing Ka and Ks with a
consideration of unequal transitional substitutions.
\emph{BMC evolutionary biology}, \bold{6(1)}, 1-10.

"data(hiv):" Ganeshan et al. (1997) Human immunodeficiency virus
type 1 genetic evolution in children with different rates of development of
disease. \emph{J. Virology.} \bold{71(1)}, 663-677.

Wang et al. (2009). gamma-MYN: a new algorithm for estimating Ka
and Ks with consideration of variable substitution rates.
\emph{Biology Direct}, \bold{4(1)}, 1-18.
}
\seealso{
\code{\link[seqinr]{kaks}}
}
\author{
Kristian K Ullrich
}
